/**************************************************************************************************
 * fifo_test_pipe_in.c
 * Test program for the FifoBuffer library, which provides an inbound fifo pipe stream for testing
 * purposes.  Parameter 1 is fifo to connect to, parameter 1 is test data file to read.  In file:
 * Sleep = displays message and sleeps for number of seconds indicated, then continues.
 * Pause = displays message and pauses for input from test data file.
 * Rewind = rewinds test data file and starts over.
 * Other records are sent to fifo. quits on eof or break.
 *-------------------------------------------------------------------------------------------------
 * gcc -g fifo_test_pipe_in.c fifo.c ini.c dbg.c -o fifo_test_pipe_in -lsocket -lnsl
 *************************************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fifo.h"

int main(int numpars, char * par[])
{  char name[128], pfn[128];
   char buffer[1000], *p;
   FIFO * fifo;
   FILE * fp;
   int rc;
   
   debug_on("stdout");
   if (numpars < 3) { printf("pars are 1=fifo name, 2=test data pfn"); exit(1); }
   strcpy(name,par[1]);
   strcpy(pfn,par[2]);
   
   fifo = fifoOpen(name, FIFO_OUTPUT, FIFO_BLOCK | FIFO_IOLBF);
   if (!fifo) { PrtFifoErrorString(stdout, "fifoOpen"); exit(1); }
   
   fp = fopen(pfn, "r");
   if (!fp) { perror("fopen"); exit(1); }

   while (1)
   {  memset(buffer,0,sizeof(buffer));
      if (!fgets(buffer, sizeof(buffer), fp)) { printf("eof"); exit(0); }
      fputs(buffer,stdout);
      if (!memcmp(buffer,"Rewind",6) || !memcmp(buffer,"rewind",6)) 
      {  rewind(fp); 
         continue; 
      }
      if (!memcmp(buffer,"Pause",5) || !memcmp(buffer,"pause",5))
      {  fgets(buffer,sizeof(buffer), stdin);
         continue;
      }
      if (!memcmp(buffer,"Sleep",5) || !memcmp(buffer,"sleep",5))
      {  rc = atoi(buffer+6);
         if (rc > 0) sleep(rc);
         continue;
      }
      rc = fifoPut(fifo, buffer);
      if (rc == FIFO_R_FAIL) PrtFifoErrorString(stdout, "fifoPut");
   }
}
