static char sccs_id[] = "// %W% %U% %G%";

//  MBroadcastAccountant
//
//  Steve Moore, 4-Jan-1999

#include <iostream>
#include <fstream>
using namespace std;
#include "ctn_os.h"

//#include <stdio.h>
//#include <stdlib.h>
//#include <string.h>
//#include <unistd.h>

#include "MBroadcastAccountant.hpp"
#include "MStatusMessage.hpp"
#include "MReportManager.hpp"
#include "dcm_api.h"


MBroadcastAccountant::MBroadcastAccountant( ) :
  mCurrentDirectory(""),
  mCurrentAETitle("")
{
  mStartDate[0] = '\0';
  mStartTime[0] = '\0';
  mEndDate[0] = '\0';
  mEndTime[0] = '\0';
}

/*virtual*/
MBroadcastAccountant::~MBroadcastAccountant() { }

void MBroadcastAccountant::print (ostream& s) const { s << "MBroadcastAccountant"; }

/*---------------------------------------------------------------------------------------
 * Creates the .broadcast.map.DEST file, if it does not already exist.
 * This routine has been modified.  The information is pulled from the .ini file rather
 * than from an msql database.  The actual creation of the file is in the main function.
 *-------------------------------------------------------------------------------------*/
void MBroadcastAccountant::broadcastMap(const char* directory, const char* AETitle)
{
  char path[1024];
  sprintf(path, "%s/.broadcast.map.%s", directory, AETitle);
  ifstream mapFile(path);

  if (mapFile) return;

  printf("Error: map file not found");

}

vector<char*>
MBroadcastAccountant::destinationVector(const char* directory, const char* AETitle)
{
  vector<char*> v;

  char path[1024];
  sprintf(path, "%s/%s.%s", directory, ".broadcast.map", AETitle);

  ifstream mapFile(path);
  if (!mapFile) { cerr << "Could not open: " << mapFile << endl; exit(1); }

  while (1) {
    char buffer[1024];
    mapFile.getline(buffer, sizeof(buffer));
    if (mapFile.eof()) break;

    char *token;
    token = strtok(buffer, " \t");
    if (strcmp(token, AETitle) != 0) {
      cerr << "Requesting AE Titles differ: <" << AETitle;
      cerr << ":" << token << ">" << endl;
      exit(1);
    }
    token = strtok(NULL, " \t");

    char* c = new char[strlen(token) + 1];
    strcpy(c, token);
    v.push_back(c);
  }

  return v;
}

int
MBroadcastAccountant::storageResult (const char* directory,
				     const char* AETitle, const char* image,
				     const char* destination, int result)
{
  if (mCurrentDirectory == "") {
    mCurrentDirectory = directory;
    mCurrentAETitle = AETitle;
  }
  if (!(mCurrentDirectory == directory))
    return -1;
  if (!(mCurrentAETitle == AETitle))
    return -1;

  MStorageKey key(image, destination);
  mba_map::iterator i = mResultMap.find(key);
  if (i == mResultMap.end()) {
    MStorageResult r(image, destination, 1, result);
    mResultMap.insert(mba_map::value_type(key, r));
  } else {
    (*i).second.result(result);
    int c = (*i).second.count() + 1;
    (*i).second.count(c);
  }
  return 0;
}

int
MBroadcastAccountant::storageResult (const char* directory,
				     const char* AETitle, const char* image,
				     const char* destination)
{
  MStorageKey key(image, destination);
  mba_map::iterator i = mResultMap.find(key);
  if (i == mResultMap.end()) return -1; 
  else return (*i).second.result();
}

int
MBroadcastAccountant::recordStatus(const char* directory, const char* AETitle)
{
  char path[1024];

  sprintf(path, "%s/%s.%s", directory, ".broadcast.rslt", AETitle);

  ofstream f(path);

  mba_map::iterator i = mResultMap.begin();
  while (i != mResultMap.end()) {
    MStorageResult r = (*i).second;
    f << r.fileName()
      << "^" << r.destination()
      << "^" << r.count()
      << "^" << r.result()
      << endl;

    i++;
  }
  return 0;
}

int
MBroadcastAccountant::readStatus(const char* directory, const char* AETitle)
{
  if (mCurrentDirectory == "") {
    mCurrentDirectory = directory;
    mCurrentAETitle = AETitle;
  }
  if (!(mCurrentDirectory == directory)) return -1;
  if (!(mCurrentAETitle == AETitle)) return -1;

  char path[1024];
  sprintf(path, "%s/%s.%s", directory, ".broadcast.rslt", AETitle);

  ifstream resultFile(path);
  if (!resultFile) { return 0; }

  while (1) {
    char buffer[1024];
    resultFile.getline(buffer, sizeof(buffer));
    if (resultFile.eof()) break;

    char *token;
    token = strtok(buffer, "^");
    string fileName(token);
    token = strtok(NULL, "^");
    string destination(token);
    token = strtok(NULL, "^");
    int count = atoi(token);
    token = strtok(NULL, "^");
    int result = atoi(token);

    MStorageKey key(fileName, destination);
    MStorageResult r(fileName, destination, count, result);

    mResultMap.insert(mba_map::value_type(key, r));
  }
  return 0;
}

void
MBroadcastAccountant::cleanup(const char* directory, const char* AETitle)
{
  char path[1024];

  sprintf(path, "%s/%s.%s", directory, ".broadcast.rslt", AETitle);
  ::unlink(path);

  sprintf(path, "%s/%s.%s", directory, ".broadcast.map", AETitle);
  ::unlink(path);
}

void
MBroadcastAccountant::beginExport(const char* reportMgr,
  int count, const char* dir, const char* path,
  const char* host, const char* queueName, const char* exportSequence,
  long queueNumber, long jobSequence)
{
  if (reportMgr[0] == '\0') return; 

  MStatusMessage m;

  m.messageType("BEGIN_EXPORT");

  char buf[128];

  ::sprintf(buf, "%-d", count);
  m.addAttribute("imagecount", buf);

  DCM_OBJECT* obj;
  obj = this->openFile(dir, path);

  if (obj == 0) { cout << "Unable to open file: " << dir << "/" << path; ::exit(1); }

  char patientID[128] = "";
  char accessionNumber[128] = "";
  this->getString(obj, 0x00100020, patientID);
  this->getString(obj, 0x00080050, accessionNumber);

  m.addAttribute("patientid", patientID);
  m.addAttribute("accession", accessionNumber);
  m.addAttribute("host", host);

  ::UTL_GetDicomDate(mStartDate);
  mStartDate[8] = '\0';
  ::UTL_GetDicomTime(mStartTime);
  mStartTime[6] = '\0';

  m.addAttribute("startdate", mStartDate);
  m.addAttribute("starttime", mStartTime);

  char seq[128];
  ::sprintf(seq, "%-d.%-d", queueNumber, jobSequence);

  m.addAttribute("exportsequence", exportSequence);
  m.addAttribute("queuename", queueName);
  m.addAttribute("jobsequence", seq);
  m.addAttribute("exportindex", "1");

  char txt[1024];
  int len = 0;
  m.serialize(txt, sizeof(txt), len);

  MReportManager mgr;
  mgr.sendMessage(reportMgr, m);

  ::DCM_CloseObject(&obj);
}


void
MBroadcastAccountant::endExport(const char* reportMgr,
  int count, const char* dir, const char* path,
  const char* host, const char* queueName, const char* exportSequence,
  long queueNumber, long jobSequence, unsigned long bytesExported,
  const char* statusText)
{
  if (reportMgr[0] == '\0') return; 

  MStatusMessage m;

  m.messageType("END_EXPORT");

  char buf[128];

  ::sprintf(buf, "%-d", count);
  m.addAttribute("imagecount", buf);

  DCM_OBJECT* obj;
  obj = this->openFile(dir, path);

  if (obj == 0) { cout << "Unable to open file: " << dir << "/" << path; ::exit(1); }

  char patientID[128] = "";
  char accessionNumber[128] = "";
  this->getString(obj, 0x00100020, patientID);
  this->getString(obj, 0x00080050, accessionNumber);

  m.addAttribute("patientid", patientID);
  m.addAttribute("accession", accessionNumber);
  m.addAttribute("host", host);

  ::UTL_GetDicomDate(mEndDate);
  mEndDate[8] = '\0';
  ::UTL_GetDicomTime(mEndTime);
  mEndTime[6] = '\0';

  m.addAttribute("startdate", mStartDate);
  m.addAttribute("starttime", mStartTime);
  m.addAttribute("enddate", mEndDate);
  m.addAttribute("endtime", mEndTime);

  char seq[128];
  ::sprintf(seq, "%-d.%-d", queueNumber, jobSequence);

  m.addAttribute("exportsequence", exportSequence);
  m.addAttribute("queuename", queueName);
  m.addAttribute("jobsequence", seq);
  m.addAttribute("exportindex", "1");

  ::sprintf(buf, "%-d", bytesExported);
  m.addAttribute("bytecount", buf);

  m.addAttribute("status", statusText);

  char txt[1024];
  int len = 0;
  m.serialize(txt, sizeof(txt), len);

  MReportManager mgr;
  mgr.sendMessage(reportMgr, m);

  ::DCM_CloseObject(&obj);
}

void
MBroadcastAccountant::beginImage(const char* reportMgr, const char * host,
        const char* queueName,
        int index, int totalCount)
{
  if (reportMgr[0] == '\0') return; 

  MStatusMessage m;

  m.messageType("BEGIN_IMAGE");

  char buf[128];

  ::sprintf(buf, "%-d", index);
  m.addAttribute("imageindex", buf);

  ::sprintf(buf, "%-d", totalCount);
  m.addAttribute("broadcastcount", buf);

  m.addAttribute("queuename", queueName);
  m.addAttribute("host", host);

  char txt[1024];
  int len = 0;
  m.serialize(txt, sizeof(txt), len);

  MReportManager mgr;
  mgr.sendMessage(reportMgr, m);
}

// Private methods below

DCM_OBJECT* MBroadcastAccountant::openFile(const char* dir, const char* path)
{
  DCM_OBJECT* obj = 0;
  CONDITION cond;
  char fullPath[1024];

  ::sprintf(fullPath, "%s/%s", dir, path);
  cond = ::DCM_OpenFile(fullPath, DCM_ORDERLITTLEENDIAN, &obj);
  if (cond != DCM_NORMAL) 
  {   cout << "Unable to open file: " << fullPath << endl;
      ::exit(1);
  }

  return obj;
}

void
MBroadcastAccountant::getString(DCM_OBJECT* obj, DCM_TAG tag, char* buf)
{
  DCM_ELEMENT e;
  e.tag = tag;
  ::DCM_LookupElement(&e);

  e.d.string = buf;
  e.length = 127;

  CONDITION cond;

  cond = ::DCM_ParseObject(&obj, &e, 1, 0,0, 0);
  if (cond != DCM_NORMAL) 
  {   ::COND_DumpConditions();
      ::exit(1);
  }
}

int
MBroadcastAccountant::nextSequenceNumbers(const char* queue,
        long& queueNumber, long& jobSequence)
{
  char path[1024];

  ::sprintf(path, "/usr/local/acq/seq/%s", queue);

  ifstream f(path);

  if (!f){cout << "Unable to open sequence file for: " << queue << endl; ::exit(1); }

  f >> queueNumber ;
  f >> jobSequence ;
  jobSequence++;

  f.close();

  ofstream v(path, ios::out | ios::trunc);
  v << queueNumber << endl;
  v << jobSequence << endl;
  v.close();

  return 0;
}
