static char sccs_id[] = "// %W% %U% %G%";

//  MStorageClient
//
//  Steve Moore, 4-Jan-1999

#include <iostream>
using namespace std;
#include "ctn_os.h"
#include "MStorageClient.hpp"

extern "C" {
   #include "dbg.h"
   #include "ini.h"
   #include "log.h"
}

extern void * msg_pointer;
extern char * msg_text;

#define FREE(A) {if(A)free(A);A=NULL;}
#define STR(A,B) A=strdup(B)

MStorageClient::MStorageClient( ) :
  _networkKey(0),
  _associationKey(0),
  _currentServer(""),
  _currentAETitle(""),
  _currentSOPClass(""),
  mBytesTransmitted(0)
{
}

/*virtual*/
MStorageClient::~MStorageClient() { disconnect(); }

void
MStorageClient::print (ostream& s) const { s << "MStorageClient"; }

int
MStorageClient::storeImage(const char* directory, const char* AETitle,
			   const char* fileName, const char* destination,
			   unsigned long& bytesExported)
{
  DCM_OBJECT* obj;
  int rslt;

  rslt = openFile(directory, fileName, &obj);
  if (rslt != 0) return rslt;

  rslt = connectToServer(AETitle, destination, &obj);
  if (rslt != 0) return rslt;

  rslt = storeImage(&obj);
  closeFile(&obj);

  if (rslt == DCM_STATUS_SUCCESS) { bytesExported += mBytesTransmitted; }

  return rslt;
}


// Private methods are defined below

int
MStorageClient::openFile(const char* directory, const char* fileName, DCM_OBJECT** obj)
{
  char path[1024];
  sprintf(path, "%s/%s", directory, fileName);
  CONDITION cond, mcond;
  char cond_msg[128];

  cond = DCM_OpenFile(path, DCM_ORDERLITTLEENDIAN, obj);
  if (cond != DCM_NORMAL) 
  {   memset(cond_msg, 0, sizeof(cond_msg));
      COND_TopCondition(&mcond, cond_msg, (unsigned long) sizeof(cond_msg) -1);
      sprintf(em, "OpenFile %d - %s", mcond, cond_msg); dbg(em);
      FREE(msg_text)
      STR(msg_text,em);
      cerr << "Could not open image: " << path << endl;
      ::COND_DumpConditions();
      return -1;
  }

  return 0;
}

int
MStorageClient::closeFile(DCM_OBJECT** obj)
{
  (void)::DCM_CloseObject(obj);
  (void)::COND_PopCondition(TRUE);
  return 0;
}

int MStorageClient::disconnect()
{
  if (_currentAETitle == "") return 0;

  ::DUL_ReleaseAssociation(&_associationKey);
  ::DUL_DropAssociation(&_associationKey);
  ::DUL_ClearServiceParameters(&_parameters);
  (void)::COND_PopCondition(TRUE);

  _currentServer = "";
  _currentAETitle = "";
  _currentSOPClass = "";

  return 0;
}


int
MStorageClient::connectToServer(const char* AETitle, const char* destination, DCM_OBJECT** obj)
{
  CONDITION cond, mcond;
  char sopClass[100];
  char cond_msg[128];

  if (!(_currentAETitle == AETitle)) disconnect();
  if (!(_currentServer == destination)) disconnect();

  {
    DCM_ELEMENT e;
    memset(&e, 0, sizeof(e));
    e.tag = DCM_IDSOPCLASSUID;
    e.representation = DCM_UI;
    e.length = sizeof(sopClass);
    e.d.string = sopClass;

    DCM_ParseObject(obj, &e, 1, NULL, 0, NULL);
    if (!(_currentSOPClass == sopClass))
      disconnect();
  }

  if (_currentAETitle == "") {
    if (_networkKey == 0) {
      DUL_InitializeNetwork(DUL_NETWORK_TCP, DUL_AEREQUESTOR, NULL, DUL_TIMEOUT,
			    DUL_ORDERBIGENDIAN, &_networkKey);
    }
    memset(&_parameters, 0, sizeof(_parameters));
    ::DUL_DefaultServiceParameters(&_parameters);
    char buffer[128];
    strcpy(buffer, destination);
    char* token = strtok(buffer, ":");
    strcpy(_parameters.calledAPTitle, token);
    token = strtok(NULL, ":");
    strcpy(_parameters.calledPresentationAddress, token);
    strcat(_parameters.calledPresentationAddress, ":");
    token = strtok(NULL, ":");
    strcat(_parameters.calledPresentationAddress, token);

    strcpy(_parameters.callingAPTitle, AETitle);
    strcpy(_parameters.callingPresentationAddress, "host");

    ::SRV_RequestServiceClass(sopClass, DUL_SC_ROLE_SCU, &_parameters);
    cond = ::DUL_RequestAssociation(&_networkKey, &_parameters, &_associationKey);
    if (cond != DUL_NORMAL) {
      cerr << _parameters.calledAPTitle
	   <<":" << _parameters.calledPresentationAddress
	   << endl;
      if (cond == DUL_ASSOCIATIONREJECTED) {
	cerr << "Association Rejected, result = " << _parameters.result
	  << " reason = " << _parameters.resultSource
	  << endl;
      }
      memset(cond_msg, 0, sizeof(cond_msg));
      COND_TopCondition(&mcond, cond_msg, (unsigned long) sizeof(cond_msg) -1);
      sprintf(em, "Parse %x - %s", mcond, cond_msg); dbg(em);
      FREE(msg_text)
      STR(msg_text,em);
      ::COND_DumpConditions();
      return -1;
    }
  }
  _currentAETitle = AETitle;
  _currentServer = destination;
  _currentSOPClass = sopClass;
  return 0;
}

static CONDITION
sendCallback(MSG_C_STORE_REQ* request, MSG_C_STORE_RESP* response,
	     unsigned long transmitted, unsigned long total,
	     void* string)
{
  unsigned long* l = (unsigned long*) string;
  (*l) = transmitted;

  return SRV_NORMAL;
}

int MStorageClient::storeImage(DCM_OBJECT** obj)
{
  MSG_C_STORE_REQ request;
  MSG_C_STORE_RESP response;
  CONDITION cond, mcond;
  char cond_msg[256];
  char sopClass[DICOM_UI_LENGTH+1] = "";
  char instanceUID[DICOM_UI_LENGTH+1] = "";

  memset(&request, 0, sizeof(request));
  memset(&response, 0, sizeof(response));

  request.type = MSG_K_C_STORE_REQ;
  request.messageID = SRV_MessageIDOut();
  request.dataSetType = DCM_CMDDATAIMAGE;
  request.dataSet = *obj;
  request.fileName = NULL;

  DCM_ELEMENT e[] = {
    { DCM_IDSOPCLASSUID, DCM_UI, "", 1, sizeof(sopClass), sopClass },
    { DCM_IDSOPINSTANCEUID, DCM_UI, "", 1, sizeof(instanceUID), instanceUID }
  };

  cond = DCM_ParseObject(obj, e, 2, NULL, 0, NULL);
  if (cond != DCM_NORMAL) 
  {   memset(cond_msg, 0, sizeof(cond_msg));
      COND_TopCondition(&mcond, cond_msg, (unsigned long) sizeof(cond_msg) -1);
      sprintf(em, "Parse %x - %s", mcond, cond_msg); dbg(em);
      FREE(msg_text)
      STR(msg_text,em);
      ::COND_DumpConditions();
      return -1;
  }

  strcpy(request.classUID, sopClass);
  strcpy(request.instanceUID, instanceUID);

  mBytesTransmitted = 0;
  cond = SRV_CStoreRequest(&_associationKey, &_parameters, &request,
			   &response, sendCallback, &mBytesTransmitted, "");
  if (cond != SRV_NORMAL) 
  {   memset(cond_msg, 0, sizeof(cond_msg));
      COND_TopCondition(&mcond, cond_msg, (unsigned long) sizeof(cond_msg) -1);
      sprintf(em, "CStore %d - %s", mcond, cond_msg); dbg(em);
      FREE(msg_text)
      STR(msg_text,em);
      ::COND_DumpConditions();
      return -1;
  }

/*-----------------------------------------------------------------------------
 * At this point, if the status is non-zero, it is a message facility status
 * which we set up to log
 *---------------------------------------------------------------------------*/
  if (response.status) msg_pointer = (void *) &response;
  
  return response.status;
}
