#ifndef _ALERT_H_
#define _ALERT_H_

/**************************************************************************************************
 * alerts.h                                                                               R Moulton
 * Header file for alert routines, which process print, e-mail, pager alerts, and process alerts, 
 * sent to a server.  Optionally prints log-like version to open file
 *-------------------------------------------------------------------------------------------------
 * 10-3 R Moulton Initial write.
 * 4-5  R Moulton Added process alerts.
 *************************************************************************************************/

#define ALERT_MAX_PRT 2     /* Maximum number of printer queues on a message */
#define ALERT_MAX_EMAIL 5   /* Maximum number of e-mail/pager addresses on a message */
#define ALERT_MAX_PRS   2   /* Maximum number of processes on a message */
#define ALERT_MAX_MSG_SIZE 10240   /* Maximum total message size */

unsigned int alert_errno;

#define ALERT_R_OK   1
#define ALERT_R_FAIL 0

#define ALERT_OK       0  /* Returns OK */
#define ALERT_INVMSG   1  /* Missing or empty message */
#define ALERT_NODEST   2  /* No destinations */
#define ALERT_EMLIM    3  /* Too many e-mail addresses */
#define ALERT_PRTLIM   4  /* Too many printer queues */
#define ALERT_PRSLIM   5  /* Too many process alerts */
#define ALERT_MEM      6  /* Memory allocation failure */
#define ALERT_MAXMSG   7  /* Message size limit exceeded */
#define ALERT_SOCKET   8  /* Unable to create TCP/IP socket */
#define ALERT_HOST     9  /* gethostbyname failure */
#define ALERT_CONNECT 10  /* connect failure */
#define ALERT_SELECT  11  /* send select failure */
#define ALERT_SEND    12  /* send failure */
#define ALERT_RELECT  13  /* receive select failure */
#define ALERT_RECV    14  /* receive failure */
#define ALERT_NAK     15  /* Server returned a NAK */
#define ALERT_GHBN    16  /* gethostby name:*/
#define ALERT_SIGNAL  17  /* Missing, invalid signal name */
#define ALERT_DIST    18  /* Invalid distributor name */

int alertSendEmail(char * e_mail_address, char * title, char * message, FILE * fp);
int alertSendPrint(char * printer_queue, char * message, FILE * fp);
int alertSendPrintEmail(char * printer_queue, char * e_mail_address, char * title, char * message, FILE * fp);
int alertSendSignal(char * signal_name, char * process_name, char * distributor);
int alertSendRun(char * process_name, char * distributor);
int alertSend(char * e_mail_addresses[], char * printer_queues[], char * process_names[],
    char * title, char * signal_name, char * message, FILE * fp);

int alertSetServer(char * host, char * ip, int port, int wait);

const char * GetAlertErrorString(void);
void PrtAlertErrorString(FILE * file, char * prefix);

#endif
