#!/bin/csh

#************************************************************************
# AnalyzeStudy.csh                                              R Moulton
# 
# This string is run at the time a new study is "found" by the Router.
# It generates the entries to the SQL database, and examines studies to 
# look for potential problem situations. This is an internal script, so 
# params are assumed to be valid.
#
# Input parameters are 
#   1 images directory for study
#   2 source of study
#   3 modality of study
#   4 study id
#   5 internal queue
#------------------------------------------------------------------------
# 5-8-2 R Moulton - Modified to eliminate external server parameter
# 6-26-2 R Moulton - Modified to add update of SQL database.
# 10-7-2 R Moulton - Check for rogue study only on CR's
#************************************************************************
set PACS = "kleypasd@mir.wustl.edu"
set study = "$1"
set SOURCE = "$2"
set MODALITY = "$3"
set study_id = "$4"
set IQ = "$5"
set S = $SNAME
#------------------------------------------ Get mode from study directory
set a = `echo "$study" | cut -d'/' -f2`
set mode = test
if ($a == distributor) set mode = production
#----------------------------------------------------- set up directories
set ROOT = $HOME/$mode/$S
set BIN  = $ROOT/bin
set LOGS = $ROOT/log
set TMP  = $ROOT/tmp
#------------------------------------------------------------- File names
set LOG  = $LOGS/Analyze.log
set temp = $study/Analyze.tmp
set DUMP = $BIN/dcm_dump_file
set PFix = $BIN/PFix

cd $study
#***********************************************************************
# Here we update the SQL database.  When we do this, we store the dba
# in a file (.dba) in the study directory.  Most times, this will not
# exist at this point, and we add the study, pending, and the xmit rows. 
# If the dba does exist, the study "dest..." files were cleared, so we 
# are rerouting.  In this case, delete the xmits and recreate them, and 
# update the pending file.  Some parameters are passed, others are in
# The dicom record header.
#******************************************************** Get parameters
set T1 = $study/ASQL.tmp
set kbytes = `du -k -s $study | cut -d'	' -f1`

#----------------------------- Count images, first and last arrival time
@ nimages = 0
set FIA = "99999999999999"
set LIA = "00000000000000"

set tags_pulled = NO
foreach series ([1-9]*)
   ls $series/[1-9]* >& /dev/null
   if ($status != 0) continue
   foreach image ($series/[1-9]*)
      @ nimages += 1
      if (-e $image) then
         if ($tags_pulled == "NO") then
            $BIN/dcm_dump_file $image | egrep "0010 0010|0010 0020|0008 0050|0008 0020|0008 0030|0008 0080|0008 1010" > $T1
	    if ($status == 0) set tags_pulled = YES
	 endif
         set ctime = `"$BIN"/Itime "$image"`
	 $BIN/Iftime.csh $ctime $FIA
	 if ($status == 2) set FIA = $ctime
	 $BIN/Iftime.csh $ctime $LIA
	 if ($status == 1) set LIA = $ctime
      endif
   end
end

if ($tags_pulled == YES) then
   set PNAME = "`cat $T1 | grep '0010 0010' | cut -d'/' -f5 | $PFix`"
   set PID   = "`cat $T1 | grep '0010 0020' | cut -d'/' -f5 | $PFix`"
   set ACCN  = "`cat $T1 | grep '0008 0050' | cut -d'/' -f5 | $PFix`"
   set STDT  = "`cat $T1 | grep '0008 0020' | cut -d'/' -f5 | $PFix date`"
   set STTM  = "`cat $T1 | grep '0008 0030' | cut -d'/' -f5 | $PFix time`"
   set INSTN = "`cat $T1 | grep '0008 0080' | cut -d'/' -f5 | $PFix`"
   set STNM  = "`cat $T1 | grep '0008 1010' | cut -d'/' -f5 | $PFix`"
   set STDT = "$STDT$STTM"
else
   set PNAME = unk
   set PID   = unk
   set ACCN  = unk
   set STDT  = 0
   set INSTN = unk
   set STNM  = unk
endif

#---------------------------------------------- Write the SQL statements
echo "use image_log_db;" > $T1
#---------------- If .dba exists, do conditional delete of these records
if (-e .dba) then
   set dba = `cat .dba`
   if ("$dba" != "") then
      echo "DELETE FROM studylog WHERE dba = $dba;" >> $T1
      echo "DELETE FROM pending WHERE pending_dba = $dba;" >> $T1
      echo "DELETE FROM xmits WHERE study_dba = $dba;" >> $T1
   endif
endif
#---------------------------------------- now insert the regular records
echo "INSERT INTO studylog VALUES (NULL," >> $T1
echo "'$SNAME',$IQ,'$study_id','$INSTN','$STNM'," >> $T1
echo "'$SOURCE','$MODALITY'," >> $T1
echo "'$PID','$PNAME','$ACCN'," >> $T1
echo "$nimages,$kbytes,$STDT," >> $T1
echo "$FIA,$LIA,0);" >> $T1

echo "INSERT INTO pending VALUES (LAST_INSERT_ID());" >> $T1
#------------------------------------- First line of multiple dest insert
echo "INSERT INTO xmits VALUES " >> $T1
set dcode = ""
foreach dest (dest.*.sendto)
   if ($dcode != "") then
      echo "(LAST_INSERT_ID(),'$dcode',0,0,0,0,0)," >> $T1
   endif
   set dcode = `echo "$dest" | cut -d'.' -f2`
end
if ($dcode != "") then
   echo "(LAST_INSERT_ID(),'$dcode',0,0,0,0,0);" >> $T1
endif
#-------------------------------- Select out dba to insert into .dba file
echo "SELECT LAST_INSERT_ID();" >> $T1
#---------------------------------------------- insert row into SQL table
/usr/local/mysql/bin/mysql -u dealer -pdlrpw < $T1 | tail -1 > .dba
rm $T1

#***********************************************************************
# now we pass the images, Looking for the tag Dave Melson was talking
# about. This only applies to CR's
#***********************************************************************
if ("$MODALITY" != "CR") exit 0
@ totalimages = 0
@ flaggedimages = 0
set animage = ""
foreach series ([1-9]*)
   ls $series/[1-9]* >& /dev/null
   if ($status != 0) continue
   foreach image ($series/[1-9]*)
      @ totalimages += 1
      $DUMP $image | grep "0008 0016" > $temp
      set a = `cat "$temp" | cut -d'/' -f5`
      if ($a == "1.3.46.670589.1.8.1") @ flaggedimages += 1
      if ($animage == "") set animage = $image
   end
end
rm $temp
if ($flaggedimages == 0) exit 0
#--------------------------------------------------------------- Log it
echo "`date` Suspect tag found" >> $LOG
echo "study: $study" >> $LOG
echo "$totalimages checked, $flaggedimages flagged" >> $LOG
echo "Demographics..." >> $LOG
$DUMP $animage | egrep "0010 0010|0010 0020|0008 0050|0008 0060|0008 0016" >> $LOG

#-------------------------------------------------------- e-mail to PACS
echo "`date` Suspect tag found" > $temp
echo "study: $study" >> $temp
echo "$totalimages checked, $flaggedimages flagged" >> $temp
echo "Demographics..." >> $temp
$DUMP $animage | egrep "0010 0010|0010 0020|0008 0050|0008 0060|0008 0016" >> $temp
mail $PACS < $temp
rm $temp
exit 0
