#!/bin/csh

#****************************************************************************
# BounceReceiver.csh                                                R Moulton
# This script provides a quick and relatively safe way to bounce or kill the
# dicom receiver on a unix server.  Usage is: BounceReceiver.csh serv [kill], 
# If the kill (or just k) parameter is included, the receiver will not be 
# restarted.  Otherwise, BounceReceiver will restart the dicom receiver. 
# The script functions by finding the invocation of simple_storage with 1 
# as its parent process and killing it, killing any process which had that 
# process as its parent, and repeating until no child process is found.  
# Returns: 0 Bounced OK. 1 Error. 
#----------------------------------------------------------------------------
#****************************************************************************
set S = $SNAME
#----------------------------------------------------------------------------
# receiver is run under user spectrum, so you must be logged in as that
# to kill it
#----------------------------------------------------------------------------
if (`whoami` != spectrum) then
   echo Must be user spectrum to bounce dicom receiver
   exit 1
endif

#------------------------------------------ Kill if param, otherwise bounce
set KILL = NO
if ("$1" == "k" || "$1" == "K")        set KILL = YES
if ("$1" == "kill" || "$1" == "KILL")  set KILL = YES

#------------------------------------------------------- validate directories
set ROOT = $home/production/$S
if (!(-d $ROOT)) then
   echo "$ROOT directory does not exist"
   exit 1
endif
set TMP = $ROOT/tmp
if (!(-d $TMP)) then
   echo "directory $TMP does not exist"
   exit 1
endif
set LOGS = $ROOT/log
if (!(-d $LOGS)) then
   echo "directory $LOGS does not exist"
   exit 1
endif
set BIN = $ROOT/bin
if (!(-d $BIN)) then
   echo" directory $BIN does not exist"
   exit 1
endif

#----------------------------------------------- some temp files we will need
set T1 = $TMP/BounceReceiver1
set T2 = $TMP/BounceReceiver2

while (1)
#------------------------------------list of running processes (newest first)
   ps -ae -o etime,pid,ppid,user,args > $T1
#------------------------------------------ Pull top level receiver from list
   cat $T1 | grep simple_stor | grep "spectrum" > $T2
   if ($status != 0) break
#---------------------------------------------------- kill top level receiver
   set PROCESS = `cat $T2`
   set PID = $PROCESS[2]
   kill -9 $PID
end
#------------------------------------------------------ Restart unless "kill"
rm $T1
rm $T2
if ($KILL == NO) then
   $BIN/StartReceiver.csh 
   echo `date` Bounced receiver >> $LOG/BounceReceiver.log
exit 0
