#!/bin/csh
#*******************************************************************************
# Checkdf.csh                                                          R Moulton
# Script designed to monitor available disk space on remote Unix servers.
# attaches to remote servers using an expect script session and retrieves a
# df display, which is then processed to retrieve the % of capacity filled, 
# logged, and alerted if over the threshold percentage.  Script is driven by the
# Checkdf.cfg file
#-------------------------------------------------------------------------------
# 9-19-06 R Moulton - Initial write
#*******************************************************************************
set RUNLOG = $LOG/Checkdf.log
set STOP   = $TMP/stopCheckdf
set PREV   = "$TMP/.NONE.tmp"
set MSG    = "$BIN/Checkdf.msg"

#---------------------------------- Must be logged in as spectrum to run GetLogs
if ($USER != spectrum) then
   echo "Must be user spectrum to run GetLogs"
   exit 1
endif

if (-e $STOP) exit 0
   
touch $RUNLOG
if (!(-e $RUNLOG)) then
   echo "Unable to create $RUNLOG file"
   exit 1
endif

echo "Begin processing -- `date`" > $RUNLOG

#------------------------------------------------------------------------------
# This loop processes one line from the config table, which is std in.
#------------------------------------------------------------------------------
while (1)
#---------------------------------------------------------- Check for stop file
   if (-e $STOP) then
      echo "=========================================== `date`" >> $RUNLOG
      echo "Exiting early, $STOP found" >> $RUNLOG
      break
   endif

#------------------------------------------------ Load next file from .cfg file
   set TABLELINE = $<

#------------------------------------------------------------------ End of file
   if ("$TABLELINE" == "end-of-cfg") then
      echo "=========================================== `date`" >> $RUNLOG
      echo "end-of-cfg found" >> $RUNLOG
      break
   endif
   
#------------------------------------------------------------------------------
# Blank lines and lines with "#" in column 1 are ignored as comments
#------------------------------------------------------------------------------
   if ("$TABLELINE" == "") continue
   set COMCHAR = `echo "$TABLELINE" | cut -c1`
   if ("$COMCHAR" == "#") continue

   echo "=========================================== `date`" >> $RUNLOG
   echo "Processing: $TABLELINE" >> $RUNLOG
#------------------------------------------ Parse out components of format line
   set SERVER    = `echo "$TABLELINE" | cut -d',' -f1`
   set SERVLOG   = "$LOG/Checkdf_$SERVER.log"
   set GREP      = `echo "$TABLELINE" | cut -d',' -f2`
   set LOG_PCT   = `echo "$TABLELINE" | cut -d',' -f3`
   set ALERT_PCT = `echo "$TABLELINE" | cut -d',' -f4`
   set ALERT_HDR = `echo "$TABLELINE" | cut -d',' -f5`
   set ALERT_DESC = `echo "$TABLELINE" | cut -d',' -f6`
   if ("$ALERT_HDR" == "") set ALERT_HDR = "$BIN/Checkdf.hdr"
   else set ALERT_HDR = "$BIN/$ALERT_HDR"
   endif

#--------------- tmp file for dialog results, might be there from previous line
   set T1 = "$TMP/Checkdf_$SERVER.tmp"

   if ($T1 != $PREV && -e $PREV) rm $PREV
   set PREV = $T1

   if (!(-e $T1)) $BIN/Checkdf.dlg $SERVER > /dev/null

   set PCT = `grep "$GREP" $T1 | cut -d':' -f2 | cut -d'%' -f1`
   
   echo "`date` server $SERVER, partition $GREP $ALERT_DESC at $PCT %" >> $RUNLOG

   if ($PCT < $LOG_PCT) continue

   echo "`date` server $SERVER, partition $GREP $ALERT_DESC at $PCT %" >> $SERVLOG

   if ($PCT < $ALERT_PCT) continue

   if (-e "$ALERT_HDR") then
      cp $ALERT_HDR $MSG
   else
      echo "e=moultonr@mir.wustl.edu" > $MSG
      echo "e=3145388960@myairmail.com" >> $MSG
      echo "t=Disk Full" >> $MSG
      echo "d=%x %X" >> $MSG
   endif

   if ("$ALERT_DESC" != "") then 
      echo "$ALERT_DESC" >> $MSG
   else 
      echo "$SERVER $GREP"   >> $MSG
   endif
   echo " at $PCT%" >> $MSG

   $BIN/SendAlert $MSG

end

if (-e $PREV) rm $PREV

echo "Checkdf completed `date`" >> $RUNLOG
exit 0
