#!/bin/csh
#******************************************************************************
# DCheck.csh                                                          R Moulton
#
# This script checks for hung Dealer Scripts, starting with the crontab, then
# looking for scripts running from it which have been on an Xmit too long.
# Processing is controlled by entries in the DCheck.cfg file, which are of the
# form: 
#    Dest:DEST_CODE,threshold_minutes,[Alert],[.hdr pfn]  
#    e.g., Dest:IMG6,60,Alert,IMG6.hdr 
# means, "Bounce Dealer scripts going to IMG6 after 60 minutes and send alert
# using the IMG6.hdr file in the bin directory."
# Alerts are controlled by the header file in the DCheck.cfg, the DCheck.hdr
# file, and a default in this script, in that order.
# Invocation: crontab -l | DCheck.csh 
# NOTE: Make sure spectrum crontab is readable by spectrum.
# M.Suchko	2009 July 24	failed check
#******************************************************************************

set DEFBOUNCE = 90                            
set DEFALERT  = Alert                         
set CFGFILE   = $BIN/DCheck.cfg                 
set LOGFILE   = $LOG/DCheck.log         
set DEFHFILE  = $BIN/DCheck.hdr        
set BOUNCELOG = $LOG/DCheckBounces.log
set T         = $TMP/DCheck.tmp

#------------------------------------------------ Must be logged in as spectrum 
if ($user != "spectrum") then
   echo "Must be user spectrum to run DCheck"
   exit 1
endif

echo "Begin :: DCheck.csh `date`" > $LOGFILE

#-------------------------------------------------- Pass spectrum crontab input
while (1)
   set LINE = $<
   if ("$LINE" == "") break                    
   if (`echo "$LINE" | cut -c1` == "#") continue      

   set A = (`echo "$LINE" | cut -d' ' -f6-`)          
   if ("$A[1]" != "D.csh") continue                   
   if ("$A[2]" != "Dealer") continue

   set EQ   = "$A[3]"                                 
   set IQ   = "$A[4]"
   set DEST = "$A[5]"

   set BOUNCE = $DEFBOUNCE                            
   set ALERT  = $DEFALERT
   set HFILE  = "NONE"
#--------------------------------- Overide defaults with values from DCheck.cfg
   if (-e $CFGFILE) then
      set CFG = `cat $CFGFILE | grep "^Dest:$DEST"`
      if ("$CFG" != "") then
         set BOUNCE = `echo "$CFG" | cut -d',' -f2`
         set ALERT  = `echo "$CFG" | cut -d',' -f3`
         set HFILE  = `echo "$CFG" | cut -d',' -f4`
      endif
   endif

   set DLOG = "$LOG/Dealer.$SNAME$EQ.log"    

   echo "Checking Dealer $EQ $IQ $DEST" >> $LOGFILE

   if (!(-e $DLOG)) then
      echo "   $DLOG not found" >> $LOGFILE
      continue
   endif

   set A = (`tail -1 $DLOG`)

#------------------------------------------------------------------------------
# Here is the check on a regular transmit (not large file)
#------------------------------------------------------------------------------
   if ("$A[1]" == "Xmit") then
      echo "=>   $A" >> $LOGFILE
      if ("$A[3]" == "failed") continue
#--------------------------------------------- Pull xmit start day, hr & minute
      set XDA = $A[9]
      set XHR = `echo "$A[10]" | cut -d':' -f1`
      set XMN = `echo "$A[10]" | cut -d':' -f2`

      set NOW = `date`
      set NDA = $NOW[3]
      set NHR = `echo "$NOW[4]" | cut -d':' -f1`
      set NMN = `echo "$NOW[4]" | cut -d':' -f2`

#------------------------------------------------ Compute time since xmit start

      @ E = 0
      if ($XDA != $NDA) @ E = 24
      @ E = ($E + $NHR - $XHR) * 60
      @ E = $E + $NMN - $XMN

      echo "   Xmit $E min, $BOUNCE threshold" >> $LOGFILE

      if ($E < $BOUNCE) continue

#--------------------------------------------------------------- Bounce and log
      $HOME/bounces $EQ > /dev/null
      echo "   Bounced `date`" >> $LOGFILE
      echo "$A - bounced at `date`" >>  $BOUNCELOG
      if ($ALERT != "Alert" && $ALERT != "ALERT") continue

#------------------------------------------------------------------ Send Alert
      echo "e=suchkom@mir.wustl.edu" > $T
      echo "t=Bounce Dealer" >> $T
      if (-e $DEFHFILE) cp $DEFHFILE $T
      if (("$HFILE" != "NONE") && (-e $BIN/$HFILE)) cp $BIN/$HFILE $T
      echo "Bounce Dealer" >> $T
      echo "$SNAME $EQ => $DEST" >> $T
      $BIN/SendAlert $T
      continue
   endif

#------------------------------------------------------------------------------
# Here is the check on a large file transmit 
#------------------------------------------------------------------------------
   if ("$A[1]" == "Xmiting") then
      echo "=>   $A" >> $LOGFILE
#--------------------------------------------- Pull xmit start day, hr & minute
      set XDA = $A[4]
      set XHR = `echo "$A[5]" | cut -d':' -f1`
      set XMN = `echo "$A[5]" | cut -d':' -f2`

      set NOW = `date`
      set NDA = $NOW[3]
      set NHR = `echo "$NOW[4]" | cut -d':' -f1`
      set NMN = `echo "$NOW[4]" | cut -d':' -f2`

#------------------------------------------------ Compute time since xmit start

      @ E = 0
      if ($XDA != $NDA) @ E = 24
      @ E = ($E + $NHR - $XHR) * 60
      @ E = $E + $NMN - $XMN

      echo "   Xmit $E min, $BOUNCE threshold" >> $LOGFILE
      if ($E < $BOUNCE) continue

#--------------------------------------------------------------- Bounce and log
      $HOME/bounces $EQ > /dev/null
      echo "   Bounced `date`" >> $LOGFILE
      echo "$A - bounced at `date`" >>  $BOUNCELOG
      if ($ALERT != "Alert" && $ALERT != "ALERT") continue

#------------------------------------------------------------------ Send Alert
      echo "e=suchkom@mir.wustl.edu" > $T
      echo "t=Bounce" >> $T
      if (-e $BIN/$HFILE) cp $BIN/$HFILE $T
      echo "Bounce Dealer" >> $T
      echo "$SNAME $EQ" >> $T
      echo "$DEST" >> $T
      $BIN/SendAlert $T
      continue
   endif

   echo "OK" >> $LOGFILE
end

echo "Completed at `date`" >> $LOGFILE
if (-e $T) rm $T
exit 0

