#!/bin/csh
#*******************************************************************************
# GetAutoSynchLogs.csh                                                 R Moulton
# Script designed to retrieve auto synch log files from msrd1 and organize them
# for analysis and review.  The files are retrieved using FTP, then processed and 
# stored in log files for each day.
#*******************************************************************************

#---------------------------------- Must be logged in as spectrum to run GetLogs
if ($USER != spectrum) then
   echo "Must be user spectrum to run GetLogs"
   exit 1
endif

set LOGS = $LOG
set LOG = $LOGS/GetAutoSynchLogs.log
set ERRLOG = $LOGS/GetAutoSynchLogs_problems.log
set COMMAND = $TMP/GetAutoSynchLogs.cmd
set OUTPUT = $TMP/GetAutoSynchLogs.out
set TODAYFILE = $TMP/GetAutoSynchLogs.today
set YESTERDAYFILE = $TMP/GetAutoSynchLogs.yesterday
set T1 = $TMP/GetAutoSynchLogs.tmp

touch $LOG
if (!(-e $LOG)) then
   echo "Unable to create $LOG file"
   exit 1
endif

touch $ERRLOG
if (!(-e $ERRLOG)) then
   echo "Unable to create $ERRLOG file"
   exit 1
endif

echo "Begin processing -- `date`" > $LOG

set TODAY    = `date '+%Y%m%d'`
set PTODAY   = `date '+%Y%.m%.d'`
set YESTERDAY = `$BIN/OTime 1 '%Y%m%d'`
set PYESTERDAY = `$BIN/OTime 1 '%Y.%m.%d'`
#------------------------------------------------------------------------------
# This loop processes one line from the config table, which is std in.
#------------------------------------------------------------------------------
while (1)
#------------------------------------------------------------- Clear work files
   rm $TMP/GetAutoSynch* >& /dev/null

#------------------------------------------------ Load next file from .cfg file
   set TABLELINE = $<

#------------------------------------------------------------------ End of file
   if ("$TABLELINE" == "end-of-file") then
      echo "=========================================== `date`" >> $LOG
      echo "end-of-file found" >> $LOG
      break
   endif
   
#------------------------------------------------------------------------------
# Blank lines and lines with "#" in column 1 are ignored as comments
#------------------------------------------------------------------------------
   if ("$TABLELINE" == "") continue
   set COMCHAR = `echo "$TABLELINE" | cut -c1`
   if ("$COMCHAR" == "#") continue

   echo "=========================================== `date`" >> $LOG
   echo "Processing: $TABLELINE" >> $LOG
#------------------------------------------ Parse out components of format line
   set NAME    = `echo "$TABLELINE" | cut -d',' -f1`
   set PREFIX    = `echo "$TABLELINE" | cut -d',' -f2`
   set SUFFIX = `echo "$TABLELINE" | cut -d',' -f3`
   set DIRECTORY  = `echo "$TABLELINE" | cut -d',' -f4`
   set PREFIX_DATE    = `echo "$TABLELINE" | cut -d',' -f5`
	set INSERT_CODE     = `echo "$TABLELINE" | cut -d',' -f6`

	echo "Saving logs" >> $LOG
#---------------------------------------------------- Generate ftp command file
   echo "cd /images_1/autosync/log" > $COMMAND
   echo "get $PREFIX$TODAY$SUFFIX $TODAYFILE" >> $COMMAND
   echo "get $PREFIX$YESTERDAY$SUFFIX $YESTERDAYFILE" >> $COMMAND
   echo "quit" >> $COMMAND

#---------------------------------------------------------- Run the ftp session
   ftp msrd1 < $COMMAND > $OUTPUT

#----------------------------------------------------------------------- Log it
   echo "ftp to msrd1 for $NAME log, ${PREFIX}YYYYMMDD$SUFFIX" >> $LOG
#--------------------------------------- Error log if files not there, and loop
   cat $OUTPUT >> $LOG
   if (!(-e $YESTERDAYFILE) && !(-e $TODAYFILE)) then
      echo "Error on ftp - `date`" >> $ERRLOG
      cat $OUTPUT >> $ERRLOG
      continue
   endif
#-------------------------------------------------- dummy file if nothing found 
   if (!(-e $YESTERDAYFILE)) then
      echo "00:00:00  Dummy entry, FNF" > $YESTERDAYFILE
   endif
   if (!(-e $TODAYFILE)) then 
      echo "00:00:00  Dummy entry, FNF" > $TODAYFILE
   endif

#---------------------------------------------------- Add insert code if needed
   if ("$INSERT_CODE" != "") then
      sed "s/  / $INSERT_CODE /" $TODAYFILE > $T1
      cp $T1 $TODAYFILE
      sed "s/  / $INSERT_CODE /" $YESTERDAYFILE > $T1
      cp $T1 $YESTERDAYFILE
   endif
#-------------------------- prepend the date to each line in the file if needed 
   if ("$PREFIX_DATE" == "Y") then
      sed "s/^/$PTODAY /" $TODAYFILE > $T1
      cp $T1 $TODAYFILE
      sed "s/^/$PYESTERDAY /" $YESTERDAYFILE > $T1
      cp $T1 $YESTERDAYFILE
   endif

#--------------------------------------------------- make sure directory exists
   if (!(-d $DIRECTORY)) mkdir -p $DIRECTORY
#------------------------------------------------ Copy files to proper location
   cp $TODAYFILE $DIRECTORY/$TODAY.log
   cp $YESTERDAYFILE $DIRECTORY/$YESTERDAY.log

   continue
end

echo "GetLog completed `date`" >> $LOG
exit 0
