#!/bin/csh
#*******************************************************************************
# Getbjmv1Logs.csh
# 
# Author: Cliff Barlow using GetLogs.csh by R Moulton as the foundation
#
# Purpose: The script retrieves hippa log files from the inputed PACS Servers
#    that are defined in Getbjmv1Logs.cfg. These logs are not housed on the 
#    Pacs Server but must be generated. The log is generated by using expect to
#    emulate a telnet session. Options are passed to log in and generate the
#    the log per a date range. The date range if giving from the last successful
#    the script ran to the current time. The time is put in a hippadate.log file
#    that is located in the target directory. Once the output is generated, it
#    is scrubbed by HippaScrubber.csh. There are lots of erelevant data that is
#    generated that we don't necessarily want to see. Each server to be logged
#    should be in the cfg file with the appropriate parameters and also 
#    located in /etc/hosts. This script runs off of the cron job D.csh.   
#
# Date Created: 8/30/2007
# 2008 Jan 20 M.Suchko various debugging changes
# 2008 Feb 17 M.Suchko Special version just for bjmv1 (hopefully temporary)
#*******************************************************************************
set Kleypas = "KleypasD@mir.wustl.edu"
set Melson = "MelsonD@mir.wustl.edu"
set Pashia = "PashiaR@mir.wustl.edu"
set Suchko = "SuchkoM@mir.wustl.edu"
set PROGNAME = `echo $0|nawk '{FS="/";print $NF}'`


#----------------------------- Must be logged in as spectrum to run Getbjmv1Logs
if ($USER != spectrum) then
   echo "Must be user spectrum to run GetLogs"
   exit 1
endif

#---------------------------Set all the directories needed
set STORE = "/distributor/logs/hippa"
if (!(-d $STORE)) then
   echo "$STORE is not a valid directory"
   exit 1
endif

set ROOT = $HOME/production/$SNAME
if (!(-d $ROOT)) then
   echo "$ROOT is not a valid directory"
   exit 1
endif

set BIN = $ROOT/bin
if (!(-d $BIN)) then
   echo "$BIN is not a valid directory"
   exit 1
endif

set LOGS = $ROOT/log
if (!(-d $LOGS)) then
   echo "$LOGS is not a valid directory"
   exit 1
endif

set TMP = $ROOT/tmp
if (!(-d $TMP)) then
   echo "$TMP is not a valid directory"
   exit 1
endif

#----------------------Set all global variables used
set STOP = $TMP/stopGetbjmv1Logs
set LOG = $LOGS/Getbjmv1Logs.log
set ERRLOG = $LOGS/Getbjmv1Logs_problems.log
set COMMAND = $TMP/Getbjmv1Logs.cmd
set OUTPUT = $TMP/Getbjmv1Logs.out
set T1 = Getbjmv1Logs.tmp
set LOCKFILE = $TMP/Getbjmv1Logs.lock
set TEMPERRFILE = $TMP/Getbjmv1Logs.temp.error
set TEMPERRFILE2 = $TMP/Getbjmv1Logs.temp2.error

if (-e $STOP) exit 0

if (-e $LOCKFILE) then
   echo "Already running. Exiting program and sending alert"
   mailx -s "$PROGNAME lock file exists" $Suchko <<EOF
The lock file $LOCKFILE
exists on $SNAME and is preventing
$0 from running.
See if an instance of the program is still running. If not then delete the lock file.
EOF
   exit 1
endif

touch $LOCKFILE
   
touch $LOG
if (!(-e $LOG)) then
   echo "Unable to create $LOG file"
   exit 1
endif

touch $ERRLOG
if (!(-e $ERRLOG)) then
   echo "Unable to create $ERRLOG file"
   exit 1
endif

echo "Begin processing -- `date`" > $LOG

set TODAYHR = `date '+%Y%m%d%H'`
set TODAY   = `date '+%Y%m%d'`
set YESTERDAY = `$BIN/OTime 1 '%Y%m%d'`
set CURRENT=`date +%Y%m%d%H%M%S`

#-----------------------------------------------------------------------------
# This loop processes one line from the config table, which is std in.
#------------------------------------------------------------------------------
while (1)
#---------------------------------------------------------- Check for stop file
   if (-e $STOP) then
      echo "=========================================== `date`" >> $LOG
      echo "Exiting early, $STOP found" >> $LOG
      break
   endif
#------------------------------------------------ Load next file from .cfg file
   set TABLELINE = $<

#------------------------------------------------------------------ End of file
   if ("$TABLELINE" == "end-of-file") then
      echo "=========================================== `date`" >> $LOG
      echo "end-of-file found" >> $LOG
      break
   endif
#------------------------------------------------------------------------------
# Blank lines and lines with "#" in column 1 are ignored as comments
#------------------------------------------------------------------------------
   if ("$TABLELINE" == "") continue
   set COMCHAR = `echo "$TABLELINE" | cut -c1`
   if ("$COMCHAR" == "#") continue

   echo "=========================================== `date`" >> $LOG
   echo "Processing: $TABLELINE" >> $LOG
#------------------------------------------ Parse out components of format line
   set SERVER    = `echo "$TABLELINE" | cut -d',' -f1`
   set SERIAL = `echo "$TABLELINE" | cut -d',' -f2`
   set PASSWD  = `echo "$TABLELINE" | cut -d',' -f3`
   set ROOT = `echo "$TABLELINE" | cut -d',' -f4` 
#--------------------------------------------------- make sure directory exists
   set TARGETDIR = $STORE/$SERVER
   if (!(-d $TARGETDIR)) mkdir -p $TARGETDIR
   set OUTFILE = $TARGETDIR/`date +%Y%m%d`.log
   set HIPPADATEFILE=$HIPPA/$SERVER/hippadate.log
   set DATE = 0 
   if (-e $HIPPADATEFILE) then
     set DATE=`cat $HIPPADATEFILE`
   endif
#---------------------------------------------------- Generate dlg expect file
  set DLGFILE = $TMP/$SERVER.dlg
  echo "#\!/usr/local/bin/expect " > $DLGFILE
  echo 'set timeout 60' >> $DLGFILE 
  echo 'spawn telnet '$SERVER >> $DLGFILE 
  echo 'expect "login: "' >> $DLGFILE 
  echo 'send "aremote\r"' >> $DLGFILE 
  echo 'expect "password: "' >> $DLGFILE
  echo 'send "\r"' >> $DLGFILE
  echo 'expect "'$SERIAL' "' >> $DLGFILE
  echo 'send "'$PASSWD'\r"' >> $DLGFILE
  echo 'expect "$ "' >> $DLGFILE
  echo 'send "su.old\r"' >> $DLGFILE
  echo 'expect "password: "' >> $DLGFILE
  echo 'send "'$ROOT'\r"' >> $DLGFILE
  echo 'expect "'$SERVER'# "' >> $DLGFILE
  echo 'send "su - logadmin\r"' >> $DLGFILE
  echo 'expect "'$SERVER':/home/sn_root/bin/apc_log> "' >> $DLGFILE 
  echo 'send "apc_log\r"' >> $DLGFILE
  echo 'expect ""' >> $DLGFILE
  echo 'send "2\r"' >> $DLGFILE
  echo 'expect "From (YYYYMMDDhhmmss) : "' >> $DLGFILE
  echo 'send "'$DATE'\r"' >> $DLGFILE
  echo 'expect "To   (YYYYMMDDhhmmss) : "' >> $DLGFILE
  echo 'send "'$CURRENT'\r"' >> $DLGFILE
  echo 'expect ""' >> $DLGFILE
  echo 'send "0\r"' >> $DLGFILE
  echo 'expect "'$SERVER':/home/sn_root/bin/apc_log>  "' >> $DLGFILE
  echo 'send "exit\r"' >> $DLGFILE
  echo 'expect "'$SERVER'# "' >> $DLGFILE
  echo 'send "exit\r"' >> $DLGFILE
  echo 'expect "'$SERVER'# $ "' >> $DLGFILE
  echo 'send "exit\r"' >> $DLGFILE

#----------------------------------------------------Change file to executable
  chmod 774 $DLGFILE 
#------------------------- Run the Expect session
   $DLGFILE > $OUTPUT

#  If there are login errors, run the Expect script one more time to make sure they are real.
   egrep 'Invalid|failure|Authentication error|Failure during Authentication|incorrect password attempts' $OUTPUT
   set errorCode = $status
   if ($errorCode == 0) then
      sleep 10
      $DLGFILE > $OUTPUT
   endif

   echo "" >> $OUTPUT
   echo "end-of-file" >> $OUTPUT 
#--------------------------------------------------------------Scrub the output   
   $BIN/HippaScrubber.csh < $OUTPUT >> $OUTFILE
#------------------------------Get a count of the number of records in the file   
   set COUNT = `cat -n "$OUTPUT" | tail -1 | cut -c1-6`
   @ TOTALCOUNT = $COUNT - 112 
   echo "Log for $SERVER, extracted $TOTALCOUNT records" >> $LOG
#----------------------------------------------------------------
#      Scrub the date if the script ran successfully
#      Change the date if the script ran successfully   
#----------------------------------------------------------------
   if($TOTALCOUNT > 0) then
      $BIN/HippaScrubber.csh < $OUTPUT >> $OUTFILE
      echo $CURRENT > $HIPPADATEFILE
   endif
#--------------------------------------------------Remove the tmp used files
#DEBUG
#   rm $DLGFILE
#   rm $OUTPUT
    mv  $OUTPUT  $OUTPUT.$SERVER
    egrep 'Invalid|failure|Authentication error|Failure during Authentication|incorrect password attempts' $OUTPUT.$SERVER > $TEMPERRFILE
    set errorCode = $status
    if ($errorCode == 0) then
      echo "retrieval error for $SERVER" >> $LOG
      cat $TEMPERRFILE >> $LOG

      rm -f $TEMPERRFILE2
      echo "HIPPA log retrieval error for $SERVER" >> $TEMPERRFILE2
      echo "This is an error message from program" >> $TEMPERRFILE2
      echo "$0 $1 $2 $3" >> $TEMPERRFILE2
      echo "running on $SNAME" >> $TEMPERRFILE2
      echo "Complete output for < $SERVER > is in the file" >> $TEMPERRFILE2
      echo "$OUTPUT.$SERVER" >> $TEMPERRFILE2
      echo "Error is ..."
      cat $TEMPERRFILE >> $TEMPERRFILE2
      echo "Complete log is appended below ..." >> $TEMPERRFILE2
      cat $OUTPUT.$SERVER >> $TEMPERRFILE2
      cat $TEMPERRFILE2 | mailx -s "HIPPA log retrieval error for $SERVER" $Kleypas,$Melson,$Pashia,$Suchko
   endif


end
echo "GetLog completed `date`" >> $LOG
rm $LOCKFILE
exit 0
