#!/bin/csh

#***************************************************************
# Images.csh                                           R Moulton
# This script scans all the subdirectories of designated direct
# looking for studies, and prints out the directories containing
# studies with images in them either directly or under series
# directories.  The parameters, if present, represent 
# directories under images directory which are to be skipped 
# (e.g., problems, # todelete).  Standard input, if present,
# represents a sorted file of source modalities (modalities may
# be blank) which are skipped. This is formatted such that it
# could be the server .cfg files (e.g., Fleming.cfg, etc) for 
# all the logical servers on the physical server, concatenated
# and then sorted.  Should be run out of the distributor bin 
# directory corresponding to the server name. Invocation format:
#  Images.csh servername root_directory [[skipdir1]skipdir2...] 
#     [< source modalities file]
# Report is to stdout.  Exit statuses 0=successful, no studies
# found, 1=error, 2=successful, studies found.
#---------------------------------------------------------------
# 8-29-06 R Moulton - modified to vary imgs root directory
#***************************************************************
set STUDIESFOUND = NO
#--------------------------------- Must be logged in as spectrum 
if (`whoami` != "spectrum") then
   echo "Must be user spectrum to run this script"
   exit 1
endif

#------------ Get physical server name and images root directory
set S = "$1" 
set IMG = "$2"
if (!(-d "$IMG")) then
   echo "$IMG not a valid directory"
   exit 1
endif
#---------------------------------------------------- Sub script
set STUDIES = $BIN/Studies.csh

#--------------------------------- Pull in parameter directories
set SKIPDIRS = ($argv[3-])

#---------------------------------- Init source modality to skip
while (1)
   set INLINE = $<
   if ("$INLINE" ==  "") then
      set SKIPSOURCE = "zzzzz"
      set SKIPMODALITY = "zzzzz"
      break
   endif
   if (`echo "$INLINE" | cut -c1` == "#") continue
   set SKIPSOURCE   = `echo "$INLINE" | cut -d',' -f1`
   set SKIPMODALITY = `echo "$INLINE" | cut -d',' -f2`
   if ("$SKIPSOURCE" == SetDefaults) continue
   if ("$SKIPSOURCE" == end-of-file) continue
   break
end

cd $IMG 

#------------------------------------------ Check base directory
$STUDIES $IMG $BIN $TMP
if ($status == 2) set STUDIESFOUND = YES

#---------------------------------------------------------------
# Level 1 directories (under images directory).  Normally, 
# these are the source directories
#---------------------------------------------------------------
ls * >& /dev/null
if ($status != 0) exit 0
foreach one (*)
   if (!(-d $one)) continue
#----------------------------- Skip directories in skipdirs list
   set skipit = NO
   foreach skip ($SKIPDIRS)
      if ($one != "$skip") continue
      set skipit = YES
      break
   end
   if ($skipit == YES) continue
#--------------- skip directories which may be study directories
   set a = `echo $one | cut -c1-2`
   if ($a == "1." || $a == "2." || $a == "3." || \
       $a == "4." || $a == "5." || $a == "6." || \
       $a == "7." || $a == "8." || $a == "9." ) continue
#------------------- Load next skip record from stdin, if needed
   while (`expr "$SKIPSOURCE" \< "$one"`)
      set INLINE = $<
      if ("$INLINE" ==  "") then
         set SKIPSOURCE = "zzzzz"
	 set SKIPMODALITY = "zzzzz"
	 break
      endif
      if (`echo "$INLINE" | cut -c1` == "#") continue
      set SKIPSOURCE   = `echo "$INLINE" | cut -d',' -f1`
      set SKIPMODALITY = `echo "$INLINE" | cut -d',' -f2`
      if ("$SKIPSOURCE" == SetDefaults) continue 
      if ("$SKIPSOURCE" == end-of-file) continue 
   end
#------------------------------------------ check this directory
# skipped if source matches and no modality
#---------------------------------------------------------------
   pushd $one >& /dev/null
   if ("$SKIPSOURCE" != $one || "$SKIPMODALITY" != "") then
      $STUDIES `pwd` $BIN $TMP 
      if ($status == 2) set STUDIESFOUND = YES
   endif
#---------------------------------------------------------------
# Level 2 directories.  Normally, these are the modalities
#---------------------------------------------------------------
   ls * >& /dev/null
   if ($status != 0) then
      popd > /dev/null
      continue
   endif
   foreach two (*)
      if (!(-d $two)) continue
#--------------- skip directories which may be study directories
      set a = `echo $two | cut -c1-2`
      if ($a == "1." || $a == "2." || $a == "3." || \
          $a == "4." || $a == "5." || $a == "6." || \
          $a == "7." || $a == "8." || $a == "9." ) continue
#------------------- Load next skip record from stdin, if needed
      while ("$SKIPSOURCE" == "$one" && `expr "$SKIPMODALITY" \< "$two"`)
         set INLINE = $<
         if ("$INLINE" ==  "") then
            set SKIPSOURCE = "zzzzz"
	    set SKIPMODALITY = "zzzzz"
	    break
         endif
         if (`echo "$INLINE" | cut -c1` == "#") continue
         set SKIPSOURCE   = `echo "$INLINE" | cut -d',' -f1`
         set SKIPMODALITY = `echo "$INLINE" | cut -d',' -f2`
         if ("$SKIPSOURCE" == SetDefaults) continue 
         if ("$SKIPSOURCE" == end-of-file) continue 
      end
      pushd $two >& /dev/null
      if ("$SKIPSOURCE" != $one || "$SKIPMODALITY" != $two) then
         $STUDIES `pwd` $BIN $TMP 
         if ($status == 2) set STUDIESFOUND = YES
      endif
#------------------------------------ Level three, usually errors
      ls * >& /dev/null
      if ($status != 0) then
         popd > /dev/null
         continue
      endif
      foreach three (*)
         if (!(-d $three)) continue
#--------------- skip directories which may be study directories
         set a = `echo $three | cut -c1-2`
         if ($a == "1." || $a == "2." || $a == "3." || \
            $a == "4." || $a == "5." || $a == "6." || \
            $a == "7." || $a == "8." || $a == "9." ) continue
         pushd $three >& /dev/null
         $STUDIES `pwd` $BIN $TMP 
         if ($status == 2) set STUDIESFOUND = YES
         endif
#--------------------------------------- Level four, also errors
         ls * >& /dev/null
         if ($status != 0) then 
            popd > /dev/null
            continue
         endif
         foreach four (*)
            if (!(-d $four)) continue
#--------------- skip directories which may be study directories
            set a = `echo $four | cut -c1-2`
            if ($a == "1." || $a == "2." || $a == "3." || \
               $a == "4." || $a == "5." || $a == "6." || \
               $a == "7." || $a == "8." || $a == "9." ) continue
            pushd $four >& /dev/null
            $STUDIES `pwd` $BIN $TMP 
            if ($status == 2) set STUDIESFOUND = YES
#--------------------------------------- Level 5, one last check
            ls * >& /dev/null
            if ($status != 0) then
               popd > /dev/null
               continue
            endif
            foreach five (*)
               if (!(-d $five)) continue
#--------------- skip directories which may be study directories
               set a = `echo $five | cut -c1-2`
               if ($a == "1." || $a == "2." || $a == "3." || \
                  $a == "4." || $a == "5." || $a == "6." || \
                  $a == "7." || $a == "8." || $a == "9." ) continue
               pushd $five >& /dev/null
               $STUDIES `pwd` $BIN $TMP
               popd >& /dev/null
            end
            popd >& /dev/null
         end
         popd >& /dev/null
      end
      popd >& /dev/null
   end
   popd >& /dev/null
end
echo Complete
if ($STUDIESFOUND == YES) exit 2
exit 0
