#!/bin/csh 
#******************************************************************
# Loose.csh                                               R Moulton
# This script generates a listing of studies and loose images which
# are in the /distributor/imgs directory tree but which are not
# accounted for in the Server.cfg file(s).  If any are found the
# listing is e-mailed to the PACS group for action.  This is 
# designed to be run as a cron job, perhaps once a day.
# Invocation: Loose.csh servername
#------------------------------------------------------------------
# 08-30-02 R Moulton - Modified to add relay1 and relay2
#******************************************************************
if (!(-e $HOME/PACS)) then
   echo "PACS email file $HOME/PACS not found"
   exit 1
endif
set PACS = `cat $HOME/PACS`          

if ($user != spectrum) then
   echo "Must be user spectrum to run Loose.csh"
   exit 1
endif

set S = $SNAME
set CS = $CSNAME

set ROOT = $HOME/production/$S
if (!(-d $ROOT)) then
   echo "Root directory $ROOT not found"
   exit 1
endif

set BIN = $ROOT/bin
if (!(-d $BIN)) then
   echo "Bin directory $BIN not found"
   exit 1
endif

set TMP = $ROOT/tmp
if (!(-d $TMP)) then
   echo "Tmp directory $TMP not found"
   exit 1
endif

#-------------------------- Get sorted version of Server.cfg file
if (!(-e $BIN/$CS.cfg)) then
   echo "Server.cfg file $CS not found"
   exit 1
endif
sort $BIN/$CS.cfg > $TMP/$CS.srt

#----------------------------------------------- Generate listing
echo "Unrouted studies and images report" > $TMP/$CS.lst
echo "For server $S" >> $TMP/$CS.lst
date >> $TMP/$CS.lst
echo >> $TMP/$CS.lst
$BIN/Images.csh $S problems todelete < $TMP/$CS.srt >> $TMP/$CS.lst
if ($status == 2) mail $PACS < $TMP/$CS.lst
rm $TMP/$CS.srt $TMP/$CS.lst
exit 0
