#!/bin/csh

#***********************************************************************
# PurgeDirectory.csh                 R Moulton, from a script by S Moore
#
# This script deletes empty directories which have not been touched in 
# a give age threshold.  It has four parameters, the directory, the
# threshold age in hours, the stop file and the log file.  This script
# is called from other scripts, the parameters are assumed valid.
# Command format:
#
# PurgeDirectory.csh directory age lockfile logfile bin
#-----------------------------------------------------------------------
#***********************************************************************

#--------------------------- load command line parameters into mnemonics
set DIR = $1
set AGE = $2
set STOP = $3
set LOG = $4
set BIN = $5

#-------------------------------------------------- Check for early stop
# Because this is called from a find command, the find will continue to
# pass entries after the stop file has been created, so we put in a "Q"
# stop file to silence subsequent log file entries.  If the stop file
# is then removed, the Q file will be removed also.
#-----------------------------------------------------------------------
if (-e $STOP) then 
   if (-e $STOP.Q) exit 1
   echo "*** Stopping early, $STOP file found -- `date` ***" >> $LOG
   touch $STOP.Q
   exit 1
endif
rm -f $STOP.Q 

#---------------------------------- problems and todelete are not touched
if ("$DIR" == "/distributor/imgs/todelete") exit 0
if ("$DIR" == "/distributor/imgs/problems") exit 0
#------------------------------------- Get age in hours since last touch
$BIN/age $DIR
@ ageInHours = $status

#-----------------------------------------------------------------------
# If the age is over the threshold, we check to see if the directory
# is empty of image files and subdirectories.  If so, the other files
# (including hidden files) are deleted, and the directory is deleted.
#-----------------------------------------------------------------------
if ($ageInHours > $AGE) then
   ls $DIR/[1-9]* >& /dev/null
   if ($status != 0) then
      rm -f $DIR/* >& /dev/null
      rm -f $DIR/.br* >& /dev/null
      rm -f $DIR/.dba >& /dev/null
      rmdir $DIR >& /dev/null
      if ($status == 0) echo "  $DIR purged" >> $LOG
   endif
endif

exit 0
