#!/bin/csh

#***********************************************************************
# PurgeFile.csh                      R Moulton, from a script by S Moore
#
# This script deletes empty files which have not been touched in 
# a give age threshold.  It has four parameters, the file, the
# threshold age in hours, the stop file and the log file.  This script
# is called from other scripts, the parameters are assumed valid.
# Command format:
#
# PurgeFile.csh directory age lockfile logfile bin            
#-----------------------------------------------------------------------
#***********************************************************************

#--------------------------- load command line parameters into mnemonics
set FILE = $1
set AGE = $2
set STOP = $3
set LOG = $4
set BIN = $5

#-------------------------------------------------- Check for early stop
# Because this is called from a find command, the find will continue to
# pass entries after the stop file has been created, so we put in a "Q"
# stop file to silence subsequent log file entries.  If the stop file
# is then removed, the Q file will be removed also.
#-----------------------------------------------------------------------
if (-e $STOP) then 
   if (-e $STOP.Q) exit 1
   echo "*** Stopping early, $STOP file found -- `date` ***" >> $LOG
   touch $STOP.Q
   exit 1
endif
rm -f $STOP.Q 

#------------------------------------- Get age in hours since last touch
$BIN/age $FILE
@ ageInHours = $status

#------------------------------ Delete the file if age is over threshold          
if ($ageInHours > $AGE) then
   rm -f $FILE >& /dev/null
   echo "  $FILE purged" >> $LOG
endif

exit 0
