#!/bin/csh

#************************************************************************
# SentTest.csh                                                  R Moulton
#
# This string examines a study to determine if the images currently 
# present have already been sent to a particular destination.
# Method is as follows:
#  1. Must be .broadcast.rslt & .broadcast.map for dest
#  2. The number of images in the study times the number of lines in the
#     .broadcast.map file must not exceed the number of lines in the
#     .broadcast.rslt file which end in "^0"
#  3. If conditions 1 & 2 are met, the SentTest program is run, which
#     verifies that each image still on file does have a "^0" entry in
#     the rslt file for each destination.
# If all these conditions are met, SentTest.csh returns a status of 2;
# otherwise it returns 0. Errors return 1.
#
# Input parameters are (1) study directory, (2) dest code, (3) bin dir
#                      (4) log file pfn
#------------------------------------------------------------------------
# 2-21-2 R Moulton Modified to allow for cases where some images have 
#        been deleted (they would be in rslt file)
# 2-22-2 R Moulton Modified to log images xmitted data
#************************************************************************
set study = "$1"
if (!(-d $study)) exit 1
cd $study

set dest = "$2"
set BIN = "$3"
set LOG = "$4"

#----------------- This .X file sometimes appears, but just get rid of it
if (-e .broadcast.map.$dest.X) rm .broadcast.map.$dest.X

#----------------------- Both these files have to exist to make our tests
if (!(-e .broadcast.map.$dest)) exit 0
if (!(-e .broadcast.rslt.$dest)) exit 0

#-------------------------------------------- Number of lines in map file
set a = `cat -n ".broadcast.map.$dest" | tail -1 | cut -c1-7`

#------------------------------------ number of images currently in study
find . -type f -name "[1-9]*" > images.$dest
set b = `cat -n "images.$dest" | tail -1 | cut -c1-7`

#-------------------------- Number of successful image xmits in rslt file
cat ".broadcast.rslt.$dest" | grep '\^0$' > rslt.$dest
set c = `cat -n "rslt.$dest" | tail -1 | cut -c1-7`

#--------------------------------- if not enough results, can't skip xmit
@ d = $c / $a 
if ($b > $d) then
   rm images.$dest 
   rm rslt.$dest
   exit 0
endif

#-------------------------------------------- Now do the match by image #
$BIN/SentTest images.$dest rslt.$dest $a >> $LOG
set st = $status
rm images.$dest
rm rslt.$dest
exit $st

