#!/bin/csh
#******************************************************************
# Studies.csh                                             R Moulton
# subscript for Images.csh, takes a directory and prints 
# information concerning studies in that directory to stdout.
# return status 0=ok, no studies found, 1=error, 2=ok studies found
# Invocation:
#   Studies.csh studydirectory, bindirectory, tmpdirectory
#******************************************************************
set DIR = $1
set BIN = $2
set TMP = $3
set T1 = $TMP/Images.temp1
set DIRPRINTED = NO
#---------------------------------------- total usage for directory
set usage = `du -k -s | cut -d'	' -f1`
#-- init totals: loose files, loose images, studies, subdirectories
@ cloosei = 0 
@ bloosei = 0
@ cloosef = 0 
@ bloosef = 0
set nloose = ""
@ cstudies = 0
@ bstudies = 0
@ csubdir = 0
@ bsubdir = 0

#-------------------------------- Pass entries in directory, if any
ls * >& /dev/null
if ($status == 0) then
   foreach entry (*)
      set type = `echo $entry | cut -c1-2`
#------------------- Tally loose files and non-image subdirectories
      if ($type != "1." && $type != "2." && $type != "3." && \
          $type != "4." && $type != "5." && $type != "6." && \
	  $type != "7." && $type != "8." && $type != "9.") then
         if (-d $entry) then
            @ csubdir += 1
            @ bsubdir += `du -k -s $entry | cut -d'	' -f1`
         else
            @ cloosef += 1
            @ bloosef += `du -k -s $entry | cut -d'	' -f1`
         endif
         continue
      endif
#----------------------------------------------------- Loose images
      if (-f $entry) then
         set nloose = $entry
         @ cloosei += 1
         @ bloosei += `du -k -s $entry | cut -d'	' -f1`
         continue
      endif
#----------------------------------- Image directories (by default)
# But they have to have images; empty we don't care about
#-----------------------------------------------------------------
      set image = `find $entry -type f -name "[1-9]*" | tail -1`
      if ("$image" == "") continue
      @ cstudies += 1
      @ bstudy = `du -k -s $entry | cut -d'	' -f1`
      @ bstudies += $bstudy 
#------------------------------ print detail information for study
      $BIN/dcm_dump_file $image | egrep "0010 0010|0010 0020|0008 0050" > $T1
      set PNAME = "`cat $T1 | grep '0010 0010' | cut -d'/' -f5`"
      set PID   = "`cat $T1 | grep '0010 0020' | cut -d'/' -f5`"
      set ACCN  = "`cat $T1 | grep '0008 0050' | cut -d'/' -f5`"
      if ("$PNAME" == "")  set PNAME = blank
      if ("$PID" == "")    set PID   = blank
      if ("$ACCN" == "")   set ACCN  = blank

      set temp = "`ls -l $image`"
      set DATE = ""
      @ i = 0
      foreach word ($temp)
         @ i += 1
         if ($i == 6 || $i == 7 || $i == 8) set DATE = "$DATE $word"
      end

#-------------------------------------------------- convert to visual units
      @ size = $bstudy
      set unit = KB
      if ($size > 1024) then
         @ size = $size / 1024
         set unit = MB
      endif
      if ($size > 1024) then
         @ g = $size / 1024
         @ r = $size - ($g * 1024)
         @ r = $r / 102
         set size = "$g.$r"
         set unit = GB
      endif
      if ($DIRPRINTED == NO) then
         set DIRPRINTED = YES
         echo
	 echo "=== $DIR"
      endif
      echo "  Study :: $entry"
      echo "   $size $unit $DATE for Name=$PNAME PID=$PID AN=$ACCN"
   end
endif
#------------------------------ End of directory entries, print total info
if ($cloosei > 0) then
   @ size = $bloosei
   set unit = KB
   if ($size > 1024) then
      @ size = $size / 1024
      set unit = MB
   endif
   if ($size > 1024) then
      @ g = $size / 1024
      @ r = $size - ($g * 1024)
      @ r = $r / 102
      set size = "$g.$r"
      set unit = GB
   endif
   if ($DIRPRINTED == NO) then
      set DIRPRINTED = YES
      echo
      echo "=== $DIR"
   endif
   echo "  $cloosei loose images - $size $unit"
   $BIN/dcm_dump_file $nloose | egrep "0010 0010|0010 0020|0008 0050" > $T1
   set PNAME = "`cat $T1 | grep '0010 0010' | cut -d'/' -f5`"
   set PID   = "`cat $T1 | grep '0010 0020' | cut -d'/' -f5`"
   set ACCN  = "`cat $T1 | grep '0008 0050' | cut -d'/' -f5`"
   if ("$PNAME" == "")  set PNAME = blank
   if ("$PID" == "")    set PID   = blank
   if ("$ACCN" == "")   set ACCN  = blank

   set temp = "`ls -l $nloose`"
   set DATE = ""
   @ i = 0
   foreach word ($temp)
      @ i += 1
      if ($i == 6 || $i == 7 || $i == 8) set DATE = "$DATE $word"
   end
   echo "    $DATE for Name=$PNAME PID=$PID AN=$ACCN"
endif
if ($DIRPRINTED == NO) exit 0
exit 2
