#!/bin/csh

#************************************************************************
# Todelete.csh                                                 R Moulton
#
# Script functions like Pending.csh, but listing files in the todelete
# directory. If there are parameters, they are destination codes (e.g.,
# IMG4, NORTH) and only studies routed to one of those codes are 
# included in the listing.
#------------------------------------------------------------------------
# 12-3-01 R Moulton - Initial write      
#************************************************************************
#------------------------------------------ Must be logged in as spectrum 
if ($user != "spectrum") then
   echo Must be user spectrum
   exit 1
endif

#---------------------------------------------- Server we are running for
set S = $SNAME 

#---------------------------------------------------- Validate image root
set IMGS = /distributor/imgs/todelete
if (!(-d $IMGS)) then
   echo "$IMGS is not a valid directory"
   exit 1
endif

#------------------------------------------------------------- temp files
set T1 = $TMP/Todelete.temp1

echo "Searching for todelete images..."

@ totalstudies = 0
@ substudies = 0
@ totalkbytes = 0
@ subkbytes = 0
#------------------------------------------------------------------------
# The loops process the source and modality directories in todelete.  
#------------------------------------------------------------------------
cd $IMGS
ls * >& /dev/null
if ($status != 0) then
   echo "Nothing in todelete directory"
   exit 0
endif
foreach sdir (*)
   cd $IMGS
   if (!(-d $sdir)) continue
   cd $IMGS/$sdir
   set header = 1   
   ls * >& /dev/null
   if ($status != 0) continue
   foreach mdir (*)
      cd $IMGS/$sdir
      if (!(-d $mdir)) continue
      cd $IMGS/$sdir/$mdir
#--------------------------- Make sure there are some studies in modality
      ls [1-9]* >& /dev/null
      if ($status != 0) continue
#----------------------------------------- Pass studies in base directory
      foreach study ([1-9]*)
         cd $IMGS/$sdir/$mdir
         set imagesexist = NO
         set image = "empty"
#--------- Validate that it is a directory, and that files exist under it
         if (!(-d $study)) continue
         ls $study/[1-9]* >& /dev/null
         if ($status != 0) continue
         
         cd $IMGS/$sdir/$mdir/$study
#----------------------- Check for destinations in parameter list (if any)
         if ("$*" != "") then
            set flag = 1
            foreach dest ($*)
               if (-e dest.$dest.sendto) then
                  set flag = 0
                  break
               endif
            end
            if ($flag) continue
         endif
#------------------------------------------------ Fetch an image file name  
         foreach series ([1-9]*)
            if (!(-d $series)) continue   
            ls $series/[1-9]* >& /dev/null
            if ($status == 0) then
               set imagesexist = YES
#---------------------------------------- Save an image name for later use
               set image = `ls $series/[1-9]* | tail -1` 
               break
            endif
         end
         if ($imagesexist == NO) continue
#----------------------------------------- display source header if needed
         if ($header) then
            echo "  "
            echo "=================================================="
            echo "   $sdir"
            echo "=================================================="
            set header = 0
         endif
#------------------------------------------------------------------------
# extract dicom demographics, display Message
#------------------------------------------------------------------------
         set i = $IMGS/$sdir/$mdir/$study/$image
         $BIN/age $i
         set fileage = $status
         $BIN/dcm_dump_file $i | egrep "0010 0010|0010 0020|0008 0050" >& $T1
         set PNAME = "`cat $T1 | grep '0010 0010' | cut -d'/' -f5`"
         set PID   = "`cat $T1 | grep '0010 0020' | cut -d'/' -f5`"
         set ACCN  = "`cat $T1 | grep '0008 0050' | cut -d'/' -f5`"
         if ("$PNAME" == "")  set PNAME = blank
         if ("$PID" == "")    set PID   = blank
         if ("$ACCN" == "")   set ACCN  = blank

         set kbytes = `du -k -s $IMGS/$sdir/$mdir/$study | cut -d'	' -f1`
         set temp = "`ls -l $i`"
         set DATE = ""
         @ i = 0
         foreach word ($temp)
            @ i += 1
            if ($i == 6 || $i == 7 || $i == 8) set DATE = "$DATE $word"
         end

#-------------------------------------------------- convert to visual units
         @ size = $kbytes
         set units = KB    
         if ($size > 1024) then
            @ size = $size / 1024
            set units = MB
         endif
         if ($size > 1024) then
            @ g = $size / 1024
            @ d = $size - ($g * 1024)
            @ d = $d / 102
            set size = $g.$d
            set units = GB
         endif

         echo "=== Study :: $study"
         echo "  $mdir $size $units arr=$DATE a=$fileage for Name=$PNAME PID=$PID AN=$ACCN" 
         ls dest.*

         @ totalstudies += 1
         @ substudies +=1
         @ totalkbytes += $kbytes
         @ subkbytes += $kbytes
      end                           # End of foreach loop for studies
      @ substudies = 0
      @ subkbytes = 0
   end                              # End of foreach loop for modality directories
end                                 # End of foreach loop for source directories

if (-e $T1) rm $T1
echo "Finished searching"

#-------------------------------------------------- convert to visual units
@ size = $totalkbytes
set units = KB    
if ($size > 1024) then
   @ size = $size / 1024
   set units = MB
endif
if ($size > 1024) then
   @ g = $size / 1024
   @ d = $size - ($g * 1024)
   @ d = $d / 102
   set size = $g.$d
   set units = GB
endif

if ($totalstudies == 0) then
    echo "  No todelete images found"
else
   echo "$totalstudies studies, $size $units"
endif
#-------------------------------------------------------------- gross estimate
@ delk = `du -k -s /distributor/imgs/todelete | cut -d'	' -f1`

set units = KB    
if ($delk > 1024) then
   @ delk = $delk / 1024
   set units = MB
endif
if ($delk > 1024) then
   @ g = $delk / 1024
   @ d = $delk - ($g * 1024)
   @ d = $d / 102
   set delk = $g.$d
   set units = GB
endif
echo "todelete = $delk $units"

exit 0
