#!/bin/csh

#*****************************************************************
# XmitTest.csh                                           R Moulton
# This script will broadcast a test study to the DICOM receiver on
# the distributor and log the results.  If the test fails, it will
# restart the DICOM receiver and send an alert message.  
#-----------------------------------------------------------------
# 05-10-03 R Moulton - Initial write
# 05-12-14 R Moulton - Modified to use small CT, no OK log
#*****************************************************************
set STUDY = "CT/1.test.ct.study"
set TARGET = "$IMGS/TEST_IMGS/$STUDY"
set CLR = "$IMGS/$ACSNAME/$STUDY"
set DEST = $ACSNAME
set XLOG = "$LOG/XmitTest.log"
set ELOG = "$LOG/XmitTestError.log"
set MSG = $TMP/XmitTest.msg
#------------------------------------------- Must be user spectrum
if (`whoami` != spectrum) then
   echo Must be user spectrum to run XmitTest
   exit 1
endif

if (!(-d $TARGET)) then
   echo $TARGET directory does not exist
   exit 1
endif

cd $TARGET
if (-e .broadcast.rslt.$DEST) rm .broadcast.rslt.$DEST
$BIN/MIRCS_dicom_broadcast $TARGET $DEST
if ($status == 0) then
   echo "`date` XmitTest OK" >> $XLOG
   rm -r $CLR
else
   $BIN/BounceReceiver.csh $SNAME
   echo "`date` XmitTest Failed" >> $ELOG
   cp $BIN/XmitTest.hdr $MSG
   echo $SNAME >> $MSG
   $BIN/SendAlert $MSG
   rm $MSG
endif

