#****************************************************************************
# create log db for script/queue based distributor
#----------------------------------------------------------------------------
# 6-4-2 R Moulton Modified to include start time, number pass and number fail
#       in xmits, and to add creation of dealer and spectrum accounts.
#****************************************************************************

CREATE DATABASE IF NOT EXISTS image_log_db;
USE image_log_db;

DROP TABLE IF EXISTS studylog;
CREATE TABLE studylog
(
        dba           INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	server        ENUM ("bond","goldfinger","fleming","bigben") NOT NULL,
	queue         TINYINT UNSIGNED NOT NULL,
	study_id      VARCHAR(255) NOT NULL DEFAULT "",
	source        VARCHAR(24) NOT NULL DEFAULT "",
	modality      VARCHAR(24) NOT NULL DEFAULT "",
	pid           VARCHAR(24) NOT NULL DEFAULT "",
	pname         VARCHAR(100) NOT NULL DEFAULT "",
	accn          VARCHAR(24) NOT NULL DEFAULT "",
	images        SMALLINT UNSIGNED NOT NULL,
	kbytes        INT UNSIGNED NOT NULL,
	first_arrive  DATETIME NOT NULL,
	last_arrive   DATETIME NOT NULL,
	completed     DATETIME NOT NULL,
	INDEX name (pname),
	INDEX id   (pid),
	INDEX seq  (accn)
);

DROP TABLE IF EXISTS xmits;
CREATE TABLE xmits
(
	study_dba     INT UNSIGNED NOT NULL,
	dest_code     VARCHAR(24) NOT NULL DEFAULT "",
	xmit_start    DATETIME NOT NULL,
	xmit_complete DATETIME NOT NULL,
        pass_count    SMALLINT UNSIGNED NOT NULL,
        fail_count    SMALLINT UNSIGNED NOT NULL,
	int_count     SMALLINT UNSIGNED NOT NULL,
	PRIMARY KEY study_dest (study_dba, dest_code)
);

DROP TABLE IF EXISTS pending;
CREATE TABLE pending
(
	pending_dba   INT UNSIGNED NOT NULL,
	INDEX pstudy (pending_dba)
);

DESCRIBE pending;

#---------------------------------------------------- user accounts
USE image_log_db;
GRANT INSERT ON image_log_db.* TO dealer@localhost IDENTIFIED BY 'dlrpw';
GRANT UPDATE ON image_log_db.* TO dealer@localhost IDENTIFIED BY 'dlrpw';
GRANT DELETE ON image_log_db.* TO dealer@localhost IDENTIFIED BY 'dlrpw';
GRANT SELECT ON image_log_db.* TO dealer@localhost IDENTIFIED BY 'dlrpw';
GRANT CREATE ON image_log_db.* TO spectrum@localhost;
GRANT SELECT ON studylog TO spectrum@localhost;
GRANT SELECT ON xmits TO spectrum@localhost;

FLUSH PRIVILEGES;
