# Dispostion of studies in the EQC logs.
# nawk -f idqcLog.awk stationInfoFromDB log file(s) ...
#
# Throw out studies with "RAD query failed".
#
# M.Suchko	2008 Jun 27
BEGIN	{
   OFS="\t"
   FS="\t"
   "uname -n" | getline distributorName

   recordCtr=0
}

# Main
{

if (NR == FNR)
{
FS="\t"
# Read in the file of stations by facility and group.
# NOTE: A station CAN BE IN MORE THAN ONE QC_GROUP
   if ($1 != "" && $2 != "" && $3 != "")
      {
      sttnNm=toupper($3)
      listOfFacilities[$1]=$1

      listOfQc_groups[$2]=$2

      stationInGroup[$2,sttnNm]="Y"
      stationInFacility[$1,sttnNm]="Y"
      stationDescription[sttnNm]=$4
      }
}

if (NR != FNR)
{
FS=" "
if ($1 == "Study" && $2 == "received")
   {
   applicationEntityCurrent=$4
   if ( match($0," - .+") )
      {
      dispositionCurrent=substr($0,RSTART+3)
      }
#   if ( match($0,"no IDQC hold") )
   if ($0 ~ /no IDQC hold/)
      {
#print "DEBUG6>" $0
      noIDQChold="Y"
      }
   }
else if ($1 == "study" && $2 == "UID")
   {
   studyUIDCurrent=$3
   }

# Harvest the DICOM header info
keyWord=substr($0,7,12)
sub(/[ ]+$/,"",keyWord)
if (keyWord == "accession")
   {
   accessionNumberCurrent=$2
   risAccessionNumber=$3
   }
else if (keyWord == "modality")
   {
   modalityCurrent=$2
   }
else if (keyWord == "patient DOB")
   {
   patientDob=$3
   }
else if (keyWord == "patient id")
   {
   patientIdCurrent=substr($0,20,32)
   gsub(/[ ]+$/,"",patientIdCurrent)
   }
else if (keyWord == "patient name")
   {
   patientNameCurrent=substr($0,20,32)
   gsub(" *$","",patientNameCurrent)
   }
else if (keyWord == "station name")
   {
   stationNameCurrent=toupper(substr($0,20))
   gsub(/[ ]+$/,"",stationNameCurrent)
   if (stationNameCurrent == "")
      {
      stationNameCurrent=applicationEntityCurrent
      }
   }
else if (keyWord == "study date")
   {
   studyDateCurrent=substr($3,1,4) "-" substr($3,5,2) "-" substr($3,7,2)
   }
else if (keyWord == "study time")
   {
   studyTimeCurrent=substr($3,1,2) ":" substr($3,3,2)
   }

# Looking for error messages
if ($0 ~ /^   [^ ]/ && $0 !~ /Study received from/)
   {
   tempString=substr($0,4)
#print "DEBUG9>" tempString "><" noIDQChold "><" patientNameCurrent ">"
# Overwrite the variable so that we only keep the last error if there is more than one.
   errorExplanationCurrent=tempString

# RAD query failed occurs when IDXRad is down for backup. We will be ignoring this
# occurance of the study because we will see it again later when. The status when
# IDXRad is up is the one we are interested in.
   if (tempString == "RAD query failed")
      {
      radQueryFailed="Y"
      }
   }
# End of data for a study.
if ( match($0,"^=+$") )
   {
# Ignore this study by overwriting it.
   if (radQueryFailed != "Y" && noIDQChold != "Y")
      {
      applicationEntity[recordCtr]=applicationEntityCurrent
      disposition[recordCtr]=dispositionCurrent
      studyUID[recordCtr]=studyUIDCurrent
      accessionNumber[recordCtr]=accessionNumberCurrent
      modality[recordCtr]=modalityCurrent
      patientId[recordCtr]=patientIdCurrent
      patientName[recordCtr]=patientNameCurrent
      stationName[recordCtr]=stationNameCurrent
      listOfStationNames[stationName[recordCtr]]="Y"
      studyDate[recordCtr]=studyDateCurrent
      studyTime[recordCtr]=studyTimeCurrent
      errorExplanation[recordCtr]=errorExplanationCurrent
      listOfErrorMessages[errorExplanation[recordCtr]]="Y"
      listOfStationNames[stationNameCurrent]="Y"
      errorMessage[stationName[recordCtr],errorExplanation[recordCtr]]++
      recordCtr++
      }
   noIDQChold=""
   radQueryFailed=""
   applicationEntityCurrent=""
   dispositionCurrent=""
   studyUIDCurrent=""
   accessionNumberCurrent=""
   modalityCurrent=""
   patientIdCurrent=""
   patientNameCurrent=""
   stationNameCurrent=""
   studyDateCurrent=""
   studyTimeCurrent=""
   errorExplanationCurrent=""
   }
#print "DEBUG1>" NR "><" recordCtr ">"
}

}

END	{
   for ( facility in listOfFacilities )
      {
      for ( qc_group in listOfQc_groups )
         {
         for ( ii=0; ii < recordCtr; ii++)
            {
#print "DEBUG>" ii "><" qc_group "><" facility "><" stationInGroup[qc_group,sttnNm] "><" stationInFacility[facility,sttnNm] ">"
            sttnNm=toupper(stationName[ii])
            if ( stationInGroup[qc_group,sttnNm]=="Y" && stationInFacility[facility,sttnNm]=="Y")
               {
               if (stationDescription[sttnNm] != "")
                  {
                  displayStation=stationDescription[sttnNm]
                  }
               else
                  {
                  displayStation=sttnNm
                  }

               if (accessionNumber[ii] != "" || patientId[ii] != "")
                  {
                  print facility,qc_group,modality[ii],displayStation,accessionNumber[ii],patientId[ii],patientName[ii],studyDate[ii] " " studyTime[ii],errorExplanation[ii],studyUID[ii]
                  }
               wasPrinted[ii]="Y"
               }
            }
         }
      }

# No facility or QC Group for the station
#print ">\t-\t-"
for (ii=0; ii < recordCtr; ii++)
   {
   if (wasPrinted[ii] != "Y" && (accessionNumber[ii] != "" || patientId[ii] != ""))
      {
      print "?","?",modality[ii],stationName[ii],accessionNumber[ii],patientId[ii],patientName[ii],studyDate[ii] " " studyTime[ii],errorExplanation[ii],studyUID[ii]
      }
   }
}
