/****************************************************************************************
 * MagicStoreStat.c                                                             R Moulton
 * Processes process information retrieved using expect from mspds priv account.
 * Generates logs and sendsalerts for 
 *---------------------------------------------------------------------------------------
 * gcc -g MagicStoreStat.c -o MagicStoreStat `pkg-config --cflags --libs MagicStoreStat`
 *---------------------------------------------------------------------------------------
 * 2006-06 R Moulton - Initial write
 *                   - Added multiple pass for e-mail or page
 ***************************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>

#include <dbg.h>
#include <ini.h>
#include <log.h>
#include <alert.h>

/*---------------------------- macros -------------------------------------------------*/
#define FREE(A) if(A){free(A);A=NULL;}
#define STR_EQ(A,B) !strcasecmp(A,B)
#define STR(A,B) {if(A)free(A);A=strdup(B);}
#define LCHR(A) A[strlen(A) - 1]
#define CLEAR(A) memset(A,0,sizeof(A));
#define ISOFF(A) (!strcasecmp(A,"OFF") || !strcasecmp(A,"NO"))

/*------------------------------- function prototypes ---------------------------------*/
static void read_ini(void);
static char * chomp(char * a);
static char input_class(char chr);
static void perform_action(char input, char action);
static void prs_screen(char first, char last);
static void wrap(void);
static void clr_prs(char * prs_id);

/*------------------------------ working storage --------------------------------------*/
static char * srvr = "mspds";
#define INI_FILE_STR "%s/MagicStoreStat.ini"
static char inipfn[256] = "";
#define INPUT_FILE_STR "%s/%sStat.tmp"
static char inp[256] = "";
static FILE * inp_fp = NULL;
static FILE * debug_fp = NULL;

static char * OKStatus = NULL;
static char status_ok = 1;
/*----------------------------------------------------- status and reporting structure */
struct _stat_ {
   char * stat;                     /* status description to look for */
   int count;                       /* # of consequtive occurences to warn on (def = 1)*/
};
/*-------------------------------------------------------- e-mail address and statuses */
static char * e_mail = NULL;
#define EMSTAT em_stat[emc]
static struct _stat_ * em_stat = NULL;
static char emc, em_max = 0;
static char need_to_email = 0;
/*---------------------------------------------------------- pager number and statuses */
static char * pager = NULL;
#define PGSTAT pg_stat[pg]
static struct _stat_ * pg_stat = NULL;
static char pg, pg_max = 0;
static char need_to_page = 0;

static char * log_name = NULL;

/*---------------------------------------------------------------------------------------
 * Input segment from expect dialog is processed into screen lines using a next-state 
 * algorhythm.
 *-------------------------------------------------------------------------------------*/
#define EO_SCR_STR "[0m"
#define ROW 24
#define COL 80

static char screen [ROW][COL];  /* Screen */
static char row = 1, col = 1;   /* row and col are base 1 - sub 1 to go to array */
static char state = 1;          /* current state */
static char input;              /* input class */
#define MAX_ESCN 25
static int escn[MAX_ESCN];      /* numbers from ESC sequences */
static char escc;

/*------------------------------------ Next state table for processing screen segments */
static struct _next_state_ {
   char state;
   char input;
   char action;
   char next_state;
}  next_state[] = {
/*----------------------------------------- state 1, processing normal data characters */
   {  1,  1,  0,  2},                                   /* ESC */
   {  1,  2,  1,  1},                                   /* tab */
   {  1,  3,  2,  1},                                   /* new line */
   {  1,  4,  3,  1},                                   /* back space */
   {  1,  9,  9,  1},
   {  1,  8,  0,  1},                                   /* ^O - ignored */
   {  1,  0,  4,  1},                                   /* any other character */
/*----------------------------------------------------- state 2, Found ESC, next char? */
   {  2,  5,  5,  3},                                   /* [ */
   {  2,  0,  0,  1},                               /* didn't find [, we ignore others */
/*------------------------------------------------------------ state 3, ESC seq number */
   {  3,  6,  6,  3},                               /* decimal digit */
   {  3,  7,  7,  3},                               /* ; - separates #'s in ESC seq */
   {  3,  0,  8,  1},                               /* ending character in ESC seq */
   { -1,  0,  0,  0},                               /* EOT line */
};
#define NSTATE next_state[ns]
static int ns, max_ns = sizeof(next_state) / sizeof(struct _next_state_);

static char msg[8012];

static time_t now;
static struct tm * tm;
static char ws[1024] = "\0";
static int sws = sizeof(ws);
static char section[16], psection[16];

/****************************************************************************************
 ****************************************************************************************
 ********************************** main function ***************************************
 ****************************************************************************************
 ***************************************************************************************/
int main(int argc, char * argv[])
{  char * p = NULL, * q = NULL, * buff = NULL;
   struct stat st;
   int s, fd, br;

   now = time(NULL);
   tm = localtime(&now);
/*------------------------------- Parameter 1 is sever (mspds or msrd[1-3], def mspds) */
   if (argc > 1) srvr = argv[1];
   if (strcmp(srvr,"mspds") && strcmp(srvr,"msrd1") && strcmp(srvr,"msrd2") && strcmp(srvr,"msrd3"))
   {  fprintf(stderr, "Usage: mspdsStat [mspds|msrd1|msrd2|msrd3]\n");
      exit (1);
   }
      
   read_ini();

   sprintf(msg,"%s Process Status Report\n", section);
/*------------------------------- Open input file and read segment from expect session */
   if (access(inp,F_OK)) dbg_exit("expect session file missing",1);    /* file OK? */
   if (access(inp,R_OK)) dbg_exit("can't read expect session file",1);
   
   if (stat(inp,&st)) dbge_exit("stat expect session file",1);   /* buffer */
   s = st.st_size + 5;                                    
   if (!(buff=malloc(s))) dbge_exit("malloc",1);
   memset(buff,0,s);                                    

   fd = open(inp,O_RDONLY);                             /* read buffer */
   if (fd < 0) dbge_exit("open",1);
   br = read(fd, buff, s);
   if (br < 0) dbge_exit("read",1);
   if (br < st.st_size) 
   {  sprintf(em,"%d bytes read, of %d in file", br, st.st_size); 
      dbg(em); 
   }

/*---------------------------------------------- process buffer using next state table */
   memset(screen,' ',ROW * COL);
   p = buff;
   while (*p)
   {  input = input_class(*p);
      for (ns = 0; ns < max_ns; ns++)
      {  if (NSTATE.state < state) continue;
         if (NSTATE.state > state || NSTATE.state == -1)
         {  fprintf(stderr,"next state table failure"); 
            exit (1);
         }
         if (NSTATE.input == input || NSTATE.input == 0)
         {  perform_action(*p, NSTATE.action);
            state = NSTATE.next_state;
            break;
         }
      }
      p++;
      if (strcmp(srvr,"mspds") && !strncmp(p,EO_SCR_STR,strlen(EO_SCR_STR))) prs_screen(10, 21);
/*------------------------------------------------------- Have just finished an action */
   }
/*----------------------------------------------------------------- process last lines */
   if (!strcmp(srvr,"mspds"))prs_screen(10, 20);
   else prs_screen(11,21);

   if (need_to_email)
   {  sprintf(em,"%s Process Warning", srvr);
      alertSendEmail(e_mail, em, msg, NULL);
      sprintf(em,"   *** Emailed to %s\n", e_mail); strcat(msg,em);
   }
   if (need_to_page)
   {  alertSendEmail(pager, srvr, "Prs Warning", NULL);
      sprintf(em,"   *** Paged %s\n", strtok(pager,"@")); strcat(msg,em);
   }
/*-------------------------------------------------------------------------------- Log */
   if (status_ok && !need_to_email && !need_to_page)
   {  sprintf(msg,"%s - All prs OK", srvr);
      logNoteLevel(log_name, msg, 2);
   }
   else logMsgLevel(log_name, msg, 1);
   return (0);
}

/*---------------------------------------------------------------------------------------
 * Given input char, returns input category for next state table
 *-------------------------------------------------------------------------------------*/
static char input_class(char chr)
{  if (chr == '\033') return 1;          /* ESC */
   if (chr == '\t')   return 2;          /* tab */
   if (chr == '\n')   return 3;          /* new line */
   if (chr == '\b')   return 4;          /* back space */
   if (chr == '[')    return 5;
   if (chr >= '0' && chr <= '9') return 6;   /* ASCII encoded decimal digit */
   if (chr == ';')    return 7;
   if (chr == '\017') return 8;          /* ^O */
   if (chr == '\r') return 9;            /* carriage return */
   return 10;                            /* Everything else */
}

/*---------------------------------------------------------------------------------------
 * Performs actions from next state table.
 *-------------------------------------------------------------------------------------*/
static void perform_action(char input, char action)
{  if (action == 0) return;  /* no action */
   switch (action)
   {  case 1:                        /* tab */
         col = (((col - 1) / 8) + 1) * 8 + 1; wrap();
         break;
      case 2:                        /* new line */
         row++; wrap();
         break;
      case 3:                        /* back space */
         col--; wrap();
         break;
      case 4:                        /* put character in display */
         screen [row - 1][col - 1] = input;
         col++; wrap();
         break;
/*--------------------------------------------------- Processing of ESC seqs ^[[#;#s */
      case 5:                        /* found ^[[, get ready to prs ESC sequence */
         for (escc = 0; escc < MAX_ESCN; escc++) escn[escc] = 0;
         escc = 0;
         break;
      case 6:                        /* decimal digit, part of ESC sequence parameter */
         escn[escc] = 10 * escn[escc] + (input - '0');
         break;
      case 7:                        /* ; separates numerical components of ESC seq */
         escc++;
         if (escc >= MAX_ESCN) { fprintf(stderr,"ESC sequence par # error"); exit (1); }
         break;
/*------------------------------------------- Handles character at end of ESC sequence */
      case 8:
         switch (input)
         {  case 'H':                          /* set cursor */
               row = escn[0]; col = escn[1];
               wrap();
               break;
            case 'A':                          /* cursor up */
               if (escn[0] == 0) escn[0] = 1;
               row -= escn[0];
               wrap();
               break;
            case 'B':                           /* cursor down */
               if (escn[0] == 0) escn[0] = 1;
               row += escn[0];
               wrap();
               break;
            case 'C':                           /* cursor foward */
               if (escn[0] == 0) escn[0] = 1;
               col += escn[0];
               wrap();
               break;
            case 'D':                           /* cursor backward */
               if (escn[0] == 0) escn[0] = 1;
               col -= escn[0];
               wrap();
               break;
            case 's':                           /* home cursor */
               row = col = 1;
               wrap();
               break;
            case 'm':                   /* set display attributes, which we ignore */
               break;
            default:
               fprintf(stderr,"Unknown ESC ending code: %s\n", input);
               break;
         }
         break;
      case 9:
         col = 1;
         break;
      default:
         fprintf(stderr,"Unknown action code: %d\n", action);
         exit (1);
   }
   return;
}

/*------------------------------ process buffer, pulling out process information lines */
static void prs_screen(char first, char last)
{  int row;
   char line[COL + 5];
   char prs_id[16];
   char ok[64];
   int shr, ehr, count;
   for (row = first-1; row <= last-1; row++)
   {  memset(line,0,sizeof(line));
      memcpy(line,screen[row],COL);
      chomp(line);
      sprintf(em,"   %s\n", line); strcat(msg,em);                   /* Add to msg */

/*-------------------------------------------------------------------- Pull process id */
      if (sscanf(line," (%[^)]", prs_id) != 1) strcpy(prs_id,"UNK");
      
      if (strstr(line,OKStatus)) { clr_prs(prs_id); continue; }  /* General OK status */
/*---------------------------------------------------------------------------------------
 * Specific processes may have their own additional OK status at certain times 
 *-------------------------------------------------------------------------------------*/
      sprintf(em,"OKStatus%s", prs_id);
      GetIniString(section,em,"NOF",ws,sws,inipfn);
      if (strcasecmp(ws,"NOF"))
         if (sscanf(ws,"%[^|]|%d|%d ", ok, &shr, &ehr) == 3)
            if (strstr(line,ok) && shr <= tm->tm_hour && ehr >= tm->tm_hour) 
            {  clr_prs(prs_id); 
               continue; 
            }
/*---------------------------------- None of the OK statuses matched, so we are not OK */
      status_ok = 0;     

      for(emc=0; emc < em_max && !need_to_email; emc++) 
      {  if(strstr(line,EMSTAT.stat)) 
         {  if (EMSTAT.count > 1)
            {  sprintf(em,"%s_em_%s", prs_id, EMSTAT.stat);
               GetIniInt(psection,em,0,&count,inipfn);
               count++;
               PutIniInt(psection,em,count,inipfn);
               sprintf(em,"%s - prs %s e-mail status %s count = %d", srvr, prs_id, EMSTAT.stat, count);
               logNote(log_name, em);
               if (count >= EMSTAT.count) need_to_email = 1;
            }
            else need_to_email=1;
         }
      }
      for(pg=0; pg < pg_max && !need_to_page;  pg++) 
      {  if(strstr(line,PGSTAT.stat)) 
         {  if (PGSTAT.count > 1)
            {  sprintf(em,"%s_pg_%s", prs_id, PGSTAT.stat);
               GetIniInt(psection,em,0,&count,inipfn);
               count++;
               PutIniInt(psection,em,count,inipfn);
               sprintf(em,"%s - prs %s page status %s count = %d", srvr, prs_id, PGSTAT.stat, count);
               logNote(log_name, em);
               if (count >= PGSTAT.count) need_to_page = 1;
            }
            else need_to_page =1;
         }
      }
   }
}
   
static void read_ini(void)
{  int s, i;
   char * p, * q;
/*---------------------------------------------------------------------- .ini file pfn */
   if (!(p=getenv("BIN"))) { fprintf(stderr,"$BIN not defined"); exit(1); }
   sprintf(inipfn, INI_FILE_STR, p);

/*--------------------------- section in .ini file corresponds to server name, in caps */ 
   strcpy(section,srvr);
   for (i = 0; i < strlen(section); i++) section[i] = (char) toupper(section[i]);
   strcpy(psection,section); strcat(psection,"_PRS");
   
/*------------------------------------------------ Status=OFF will turn off processing */
   if (!GetIniString(section,"Status","ON",ws,sws,inipfn)) 
   {  fprintf(stderr,GetIniErrorString()); 
      exit(1); 
   }
   if (ISOFF(ws)) exit(0);

/*----------------------------------------------------------------------------- Debug */
   GetIniString("[MAIN]","Debug","NULL",ws,sws,inipfn);
   debug_fp = debug_on(ws);

/*---------------------------------------- Status code which signifies that prs is OK */
   GetIniString(section,"OKStatus","responded",ws,sws,inipfn);
   STR(OKStatus,ws)

/*------------------------------------------------------ E-mail address & page number */
   GetIniString("[MAIN]","Email",NULL,ws,sws,inipfn);
   if (ws) 
   {  STR(e_mail,ws)
      if (GetIniString(section,"EmailStatus","NONE",ws,sws,inipfn) && !ISOFF(ws))
      {  char a[sws]; strcpy(a,chomp(ws));
         for (p=strtok(a,","), em_max=0; p; p=strtok(NULL,",")) if (strlen(chomp(p))) em_max++;
         if (em_max)
         {  em_stat = (struct _stat_ *) calloc (em_max, sizeof(struct _stat_));
            if (!em_stat) dbg_exit("malloc failure", 1);
            for(p = (char *) strtok_r(ws,",",&q), emc=0; p; p = (char *) strtok_r(q,",",&q)) 
            {  if(strlen(chomp(p)))
               {  p=strtok(p,"=");
                  STR(EMSTAT.stat,p)
                  EMSTAT.count = 1; 
                  if ((p=strtok(NULL,", ")) && strlen(p)) EMSTAT.count = atoi(p); 
                  emc++;
               }
            }
         }
      }
   }
      
   GetIniString("[MAIN]","Pager",NULL,ws,sws,inipfn);
   if (ws) 
   {  STR(pager,ws)
      if (GetIniString(section,"PagerStatus","NONE", ws,sws,inipfn) && !ISOFF(ws))
      {  char a[sws]; strcpy(a,chomp(ws));
         for (p=strtok(a,","), pg_max=0; p; p=strtok(NULL,",")) if (strlen(chomp(p))) pg_max++;
         if (pg_max)
         {  pg_stat = (struct _stat_ *) calloc (pg_max, sizeof(struct _stat_));
            if (!pg_stat) dbg_exit("malloc failure", 1);
            for(p=(char*)strtok_r(ws,",",&q), emc=0; p; p=(char*)strtok_r(q,",",&q)) 
            {  if(strlen(chomp(p)))
               {  p=strtok(p,"=");
                  STR(PGSTAT.stat,p)
                  PGSTAT.count = 1; 
                  if ((p=strtok(NULL,", ")) && strlen(p)) PGSTAT.count = atoi(p); 
                  pg++;
               }
            }
         }
      }
   }

   GetIniString(section,"LogName","NULL",ws,sws,inipfn);
   if (strcasecmp(ws,"NULL")) 
   {  STR(log_name,ws)
      logInitialize(inipfn);
   }

/*-------------------------------------------------------------------- input file name */
   if (!(p=getenv("TMP"))) dbg_exit("$TMP not defined",1);
   sprintf(inp, INPUT_FILE_STR, p, srvr);
}


/*----------------------- Remove leading and trailing spaces and new lines from string */
static char * chomp(char * a)
{  int i;
   if (!a) return a;
   i = strspn(a," ");
   memmove(a,a+i,strlen(a+i)+1);
   while (LCHR(a) == ' ' || LCHR(a) == '\n') LCHR(a) = '\0';
   return a;
}

static void wrap(void) 
{  if (col < 1) col = 1;
   if (row < 1) row = 1;
   if (col > COL) {col = 1; row++;} 
   if (row > ROW) row = 1; 
}

static void clr_prs(char * prs_id)
{  for (emc = 0; emc < em_max; emc++)
   {  if (EMSTAT.count > 1) 
      {  sprintf(em,"%s_%s", prs_id, EMSTAT.stat);
         PutIniString(psection, em, NULL, inipfn);
      }
   }
   for (pg = 0; pg < pg_max; pg++)
   {  if (PGSTAT.count > 1) 
      {  sprintf(em,"%s_%s", prs_id, PGSTAT.stat);
         PutIniString(psection, em, NULL, inipfn);
      }
   }
}
