/***********************************************************************
 * Program is used to "homogenize" input parameters which have been 
 * retrieved from DICOM image file headers so that they can reliably be
 * used as fields in a SQL statement.  The "raw" parameter is passed as
 * stdin, and the "homogenized" version is delivered on stdout.  There 
 * are three types of processing done: (1) An alpha parameter, in which
 * any occurrence of the single quote "'" should be replaced by a space,
 * in order to allow it to be used as an alpha parameter, (2) a date,
 * which should be checked for numeric and proper length, with zeroes 
 * returned if it is invalid, and (3) a time, processed in a similar
 * manner.  The script invocation of the program should be:
 *
 *  alpha:   parameter-in | PFix
 *  date:    parameter-in | PFix date
 *  time:    parameter-in | PFix time
 *
 *  Returns 0 if input was processed without error, 1 otherwise
 *
 ************************************************************************
 *
 * Compile and link with: gcc PFix.c -o PFix 
 *
 ***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#define ALPHA 0
#define DATE  1
#define TIME  2
/***********************************************************************
 * main function
 **********************************************************************/
int main(int numpars, char * par[])
{  int mode = ALPHA, c, i = 0, error = 0;
   char a[512];
   int min[3] = {0,8,6}; /* Minimum length by param type */
/*----------------------------------------- Set mode from parameter 1 */
   if (numpars > 1)
   {  if (!strcmp("date", par[1])) mode = DATE;
      if (!strcmp("time", par[1])) mode = TIME;
   }
/*----------------------------------- Pull input parameter from stdin */
   memset (a,0,sizeof(a));
   while ((c = getchar()) != EOF)
   {  switch (mode)
      {	 case ALPHA: /* Replace single quotes with spaces */
	    if ((char)c == '\'')
	    {  c = ' ';
	       error = 1;
	    }
	    a[i++] = c;
	    break;
	 default: /* Numerics, ignore non-numeric characters */
	    if (isdigit(c)) a[i++] = c;
	    else error = 1;
	    break;
      }
   }
/*----------------------- Pad parameter with zeroes to minimum length */
   while (strlen(a) < min[mode]) a[strlen(a)] = '0'; 
/*----------------- Trim date & time parameters to length (also max)  */
   if (mode == DATE || mode == TIME) a[min[mode]] = '\0';
   for (i = 0; i < strlen(a); i++) putchar((int)a[i]);
   exit (error);
}
