/***********************************************************************************
* SendAlert.c                                                              R Moulton
* Program sends alerts to the alerts server on rentgn.  If one param, that is pfn of
* message file, otherwise it is stdin. Lines can be:
* p=printerqueue
* e=emailaddress
* t=title
* d=format string (Inserts current date time using this format string
* All other lines are the message itself.  Messages are logged.
*-----------------------------------------------------------------------------------
* gcc -g SendAlert.c alert.c dbg.c -o SendAlert -lresolv -lsocket -lnsl
***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "alert.h"

#define MAKE_STR(A,B) if (A=malloc(strlen(B)+3)) strcpy(A,B);
#define STR_EQ(A,B) !strcasecmp(A,B)
#define LAST_CHR(A) A[strlen(A) -1]

#define ME "moultonr@mir.wustl.edu"

void BadAlert(void);

int main(int numpars, char *par[])
{	int rc, ec, pc;
   static char * e_mail_addresses[64], * printer_queues[64], * title, message[10240];
   char pfn[256], buffer[1024], dt[256], *p;
   FILE * log = NULL;
   FILE * msg = stdin;
   time_t now = time(NULL);

/*------------------------------------------------------------------- Message file */
   if (numpars > 1)
   {  msg = fopen(par[1],"r");
      if (!msg){perror("msg file open"); BadAlert();}
   }
/*---------------------------------------------------------------- set up log file */
   if (p=getenv("LOG"))
   {  strcpy(pfn,p); strcat(pfn,"/Alert.log");
      log = fopen(pfn,"a");
   }
/*---------------------------------------------------- Build message from msg file */
   ec = pc = 0;
   memset(message,0,sizeof(message));

   while (fgets(buffer, sizeof(buffer), msg))
   {  if (*(buffer+1) == '=')
      {  switch (*(buffer))
         {  case 'e':
               MAKE_STR(e_mail_addresses[ec],buffer+2)
               LAST_CHR(e_mail_addresses[ec]) = '\0';
               ec++;
               break;
            case 'p':
               MAKE_STR(printer_queues[pc],buffer+2)
               LAST_CHR(printer_queues[pc]) = '\0';
               pc++;
               break;
            case 't':
               if (title) free(title);
               MAKE_STR(title,buffer+2)
               LAST_CHR(title) = '\0';
               break;
            case 'd':
               cftime(dt,buffer+2,&now);
               strcat(message,dt);
               break;
            default:
               strcat(message,buffer);
               break;
         }
      }
      else (strcat(message,buffer));
   }
   if (ferror(msg)) BadAlert();
   if (!ec && !pc) MAKE_STR(e_mail_addresses[0],ME)
   if (alertSend(e_mail_addresses, printer_queues, title, message, log) == ALERT_R_FAIL)
      PrtAlertErrorString(stderr, "alertSend");
                  
   exit(0);
}

void BadAlert(void)
{  alertSendEmail(ME, "SendAlert", "Bad mail file", NULL);
   exit(1);
}
