#ifndef _INI_H_
#define _INI_H_

/******************************************************************************
 * ini.h                                                              R Moulton
 * Header file for ini routines, which allow storage of program parameters in
 * an ini file for the application.  These are similar to the MS Windows 3.x
 * "PrivateProfile" calls. All return 1=OK, 0=Failed. ini_errno is set at each
 * call to indicate the specifics of the error. GetIniErrorString will return
 * a pointer to a string describing the error, PrtIniErrorString will print the 
 * string to stdout.
 *-----------------------------------------------------------------------------
 * 9-3 R Moulton Initial write.
 *****************************************************************************/

unsigned int ini_errno;

#define INI_OK       0  /* Returns OK */
#define INI_NORSLT   1
#define INI_PFN      2
#define INI_OPEN     3
#define INI_IO       4
#define INI_OVFRSLT  5  /* Returns OK */
#define INI_NOSEC    6  /* Returns OK */
#define INI_SECNOF   7 
#define INI_KEYNOF   8  /* Returns OK */
#define INI_INVSEC   9
#define INI_INVKEY  10
#define INI_INT     11
#define INI_CWDF    12
#define INI_CDF     13

int GetIniString(char * sname, char * kname, char * dvalue, char * rvalue, int size, char * pfn);
int GetIniInt(char * sname, char * kname, int dvalue, int * rvalue, char * pfn);   
int GetIniSection(char * sname, char * rvalue, int size, char * pfn);
int GetIniSectionNames(char * rvalue, int size, char * pfn);

int PutIniString(char * sname, char * kname, char * nvalue, char * pfn);
int PutIniInt(char * sname, char * kname, int nvalue, char * pfn);
int PutIniSection(char * sname, char * nvalue, char * pfn);

char * GetIniErrorString(void);
void PrtIniErrorString(FILE * file, char * prefix);
char * MakeProgDirCurrent(char * par0);
int unique_process_check(char * file);
FILE * open_log_file(char * pfn, unsigned char dates_allowed);
void close_log_file(FILE * fp);

#endif
