/******************************************************************************
 * ts.c test server.
 * Opens, waits for messages, echoes them back to client and logs them.
 *-----------------------------------------------------------------------------
 * Comp/link with: gcc -g ts.c -o ts `pkg-config --cflags --libs ts`
 *****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>

#include "dbg.h"
#include "ini.h"
#include "log.h"


/*-------------------------------------------------------------------------- Function prototypes */
int read_ini(void);
void Wrap(char * msg);
void Unwrap(char * msg);
void cleanup(void);
void prs_message(void);

/*--------------------------------------------------------------------------------------- Macros */
#define STR(A,B) A=strdup(B)
#define FREE(A) if(A){free(A);A=NULL;}
#define STR_EQ(A,B) !strcasecmp(A,B)
#define CLEAR(A) memset(A,0,sizeof(A))
#define LCHR(A) A[strlen(A) - 1]

#define BUFFER_SIZE 1024 * 60 * 4000
#define HNS 64
char message[BUFFER_SIZE];
char out_message[BUFFER_SIZE];
char rbuf[BUFFER_SIZE];
char host_name[HNS]        = " ";
int  port_num_in = 0;	/* port to listen to */
int  port_num_out = 0;	/* port to write to */
struct hostent *phostent, shostent;
struct sockaddr_in server_socket_name;
struct sockaddr * sptr = (struct sockaddr*)&server_socket_name;
time_t timer;
time_t * ptimer = &timer; 
struct tm * ptime;
int server_socket, i, j, retval;
int on = 1;
int socket_num = 0;
char * ptr;
static struct timeval timeval = {60, 0};
static fd_set rsock, wsock, esock;
static char rts = 0;

/*-------------------------------------------------------------- Environment variables */
#define DEF_INI_PFN "ts"
char inipfn[256];
char a[1024];
int sa = sizeof(a);
char * cwd;
char * process_fifo = "None";
static FILE * debug_fp;
static char IPValidation = 0;
static char * Logname  = NULL;
static char * Hostname = NULL;


int main(int numpars, char *par[])
{	char * p;
/*---------------------------------------------------------------------------------------
 * Program runs out of directory binary is in.  Make that current.  Then use parameter 1
 * or default to complete .ini file pfn.
 *-------------------------------------------------------------------------------------*/
   if (p = MakeProgDirCurrent(par[0])) STR(cwd,p);
   else { PrtIniErrorString(stdout,"Startup"); exit(1); }
   strcpy(inipfn,cwd); 
   if (numpars > 1) strcat(inipfn,par[1]);
   else strcat(inipfn,DEF_INI_PFN);
   strcat(inipfn,".ini");
   
   if (read_ini()) return(1);            /* process .ini file */
   
   if (strcasecmp(process_fifo,"None")) unique_process_check(process_fifo);


/****************************************************************************************
 ******************************* server socket loop *************************************
 ***************************************************************************************/
   while (1)
   {  int r;
/*---------------------------------------------- Set up server socket */
      dbg("Initializing server");
      server_socket = socket(AF_INET, SOCK_STREAM, 0);
      if (server_socket == -1) dbge_exit("socket",1); 

      if (setsockopt(server_socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on))) dbge_exit("setsockopt",1);

      if (Hostname)strcpy(host_name,Hostname);
      else if (gethostname(host_name, HNS)) dbge_exit("gethostname", 1); 

      phostent = gethostbyname(host_name);
      if (!phostent) dbge_exit("gethostbyname",1);
   
      shostent = *phostent;
      server_socket_name.sin_family = shostent.h_addrtype;
      server_socket_name.sin_port = htons(port_num_in);
      server_socket_name.sin_addr = *((struct in_addr *)shostent.h_addr);

      if (bind(server_socket, sptr, sizeof(server_socket_name))) dbge_exit("bind",1);

      if (listen(server_socket, 2)) dbge_exit("listen", 1); 

      dbg("Server initialized");
      cleanup();
      
/****************************************************************************************
 ********************************** connection loop *************************************
 ***************************************************************************************/
      while (1)
      {  int i;
         size_t sl = sizeof(server_socket_name);
         char ip[20];
         unsigned char * ptr;
         
         if (!socket_num) dbg("Waiting for connection");
         if (Logname) logPurge(Logname);
/*------------------------------------------------------------------ look for activity */
		   FD_ZERO(&rsock); FD_ZERO(&wsock); FD_ZERO(&esock);
         FD_SET(server_socket, &rsock);                            /* On listen socket */
         if (socket_num)                                           /* On active socket */
         {  FD_SET(socket_num, &rsock);
            FD_SET(socket_num, &esock);
            if (rts) FD_SET(socket_num, &wsock);                   /* Have msg to send */
         }
         
         r = select (FD_SETSIZE, &rsock, &wsock, &esock, &timeval);
		   sprintf(em,"select returned %d", r); dbg(em); 
/*---------------------------------------------------------------- timeout */
         if (!r) continue;

/*-------------------------------- Activity on listen socket is new connect */
         while (FD_ISSET(server_socket, &rsock))
         {  cleanup();
            socket_num = accept(server_socket, sptr, &sl);
            if (socket_num == -1) 
            {  dbge("accept"); 
               cleanup(); 
               break;
            }
            if (setsockopt(socket_num, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof(on)))
            {  perror("keepalive"); 
               cleanup(); 
               break;
            }
            ptr = (unsigned char *)&server_socket_name.sin_addr;
            sprintf(ip,"%01d.%01d.%01d.%01d",ptr[0],ptr[1],ptr[2],ptr[3]);
            if (IPValidation)
            {  GetIniString("[ClientIPs]",ip,"NOF",a,sa,inipfn);
               if (strcasecmp(a,"NOF"))
               {  sprintf(em, "Attempt to connect from: %s rejected", ip); dbg(em);
                  cleanup();
                  break;
               }
            }
            sprintf(em, "Connected to %s", ip); dbg(em);
            break;
         }

/*----------------------------------------------------------------------- Send message */
         while (socket_num && rts && FD_ISSET(socket_num, &wsock))
         {  r=send(socket_num, out_message, strlen(out_message), 0);
	 	      if (r < 0) 
            {  dbge("send"); 
               cleanup(); 
               break;
            }
	 	      sprintf(em,"%d bytes sent", r); dbg(em);
            Unwrap(out_message);
            logMsgLevel(Logname,message,2);
            CLEAR(out_message);
            rts = 0;
            break;
         }

/*-------------------------------------------------------------------- Receive message */
         while (socket_num && FD_ISSET(socket_num, &rsock))
         {  CLEAR(rbuf);
            r = recv(socket_num, rbuf, BUFFER_SIZE, 0);
            if (r < 0)
            {  dbge("recv");
               cleanup();
               break;
            }
            if (!r)
            {  dbg("Client disconnected");
               cleanup();
               break;
            }
            sprintf(em,"%d bytes received", r); dbg(em);
            
            if (p=strchr(rbuf,11)) strcpy(message,p);
            else strcat(message,rbuf);
            prs_message();
            break;
         }
      }
   }
}
   

void prs_message(void)
{  char * p;
   if (message[0] != 11 || !strchr(message,28)) return;
   strcpy(out_message,message);
   rts = 1;
   Unwrap(message);
   while (p = strchr(message,'\r')) *p = '\n';
   logMsgLevel(Logname,message,1);
   CLEAR(message);
}
   


int read_ini()
{  char main[] = "[MAIN]", *p;
   
   if (!GetIniString(main, "Debug", "stdout", a, sa, inipfn))
   {  sprintf(em,"Debug %s", GetIniErrorString()); 
      dbge_exit(em, 1);
   }
   debug_fp = debug_on(a);

   GetIniString(main,"Hostname","NOF",a,sa,inipfn);
   if (strcasecmp(a,"NOF")) STR(Hostname,a);
   
   GetIniInt(main,"PortIn",port_num_in, &port_num_in, inipfn);
   if (!port_num_in) {dbg_exit("No port in # in .ini file",1);}

   GetIniInt(main,"PortOut",port_num_out, &port_num_out, inipfn);
   if (!port_num_out) {dbg_exit("No port out # in .ini file",1);}

   GetIniString(main,"ProcessFifo","None",a,sa,inipfn);
   if (strcasecmp(a,"None")) STR(process_fifo,a);

   GetIniString(main,"IPValidation","NO",a,sa,inipfn);
   if (STR_EQ(a,"YES") || STR_EQ(a,"ON")) IPValidation = 1;

   GetIniString(main,"Logname","None",a,sa,inipfn);
   if (strcasecmp(a,"None")) STR(Logname,a);
   
   if (logInitialize(inipfn)) dbg(GetLogErrorString());

   return (0);
}

   
   
void cleanup(void)
{  if(socket_num) 
   {  if (shutdown(socket_num, 2)) dbge("shutdown socket");
      if (close(socket_num)) dbge("close socket");
      socket_num = 0;
   }
   CLEAR(message);
   rts = 0;
}

/***********************************************************************
 * Add/Remove start and stop message characters (11 & 28)
 **********************************************************************/
void Wrap(char * msg)
{  char buffer[strlen(msg)+5];
   CLEAR(buffer);
   buffer[0] = 11;
   strcat(buffer,msg);
   buffer[strlen(buffer)] = 28;
   strcpy(msg,buffer);
}
void Unwrap(char * msg)
{  if (msg[0] == 11) memmove(msg,msg+1,strlen(msg));
   if (LCHR(msg) == 28) LCHR(msg) = 0;
}

