/***********************************************************************************
* Run.c                                                                    R Moulton
* Program sends message to named server. If one param, that is pfn of
* message file, otherwise it is stdin. Lines can be:
* s=server       
* p=process to run
* Messages are logged along with server response.
*-----------------------------------------------------------------------------------
* gcc -g Run.c -o Run `pkg-config --cflags --libs run`
***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "dbg.h"
#include "alert.h"

#define MAKE_STR(A,B) if (A=malloc(strlen(B)+3)) strcpy(A,B);
#define STR_EQ(A,B) !strcasecmp(A,B)
#define LAST_CHR(A) A[strlen(A) -1]

void BadRun(void);

int main(int numpars, char *par[])
{  static char * dserver, * name, message[10240];
   char pfn[256], buffer[1024], dt[32], *p;
   FILE * log = NULL;
   FILE * msg = stdin;
   int ret;
   time_t now = time(NULL);
   struct tm * tm = localtime(&now);
   strftime(dt, sizeof(dt), "%Y-%m-%d %H:%M:%S", tm);

/*------------------------------------------------------------------- Message file */
   if (numpars > 1)
   {  msg = fopen(par[1],"r");
      if (!msg){perror("msg file open"); BadRun();}
   }
/*---------------------------------------------------------------- set up log file */
   if (p=getenv("LOG"))
   {  strcpy(pfn,p); strcat(pfn,"/Run.log");
      log = fopen(pfn,"a");
   }
/*---------------------------------------------------- Build message from msg file */
   memset(message,0,sizeof(message));

   while (fgets(buffer, sizeof(buffer), msg))
   {  while (LAST_CHR(buffer) == '\n' || LAST_CHR(buffer) == ' ') LAST_CHR(buffer) = 0;
      if (*(buffer+1) == '=')
      {  switch (*(buffer))
         {  case 's':
               MAKE_STR(dserver,buffer+2)
               break;
            case 'p':
               MAKE_STR(name,buffer+2)
               break;
         }
      }
   }
   if (ferror(msg)) BadRun();
  
/*--------------------------------------------------------- send request to server */
   sprintf(em,"calling alertSendRun(%s, %s)", name, dserver); dbg(em); 
   ret = alertSendRun(name, dserver);

/*-------------------------------------------------- Log message sent and response */
   sprintf(em,"%s - run %s on %s - Ret = %s\n", dt, name, dserver, 
      ret == ALERT_R_FAIL ? "Failed" : "Ok"); 
   if (fputs(em, log) == EOF) perror("fputs error");
   
   if (ret == ALERT_R_FAIL)
   {  dbg("returned not OK");
      PrtAlertErrorString(log, "error");
      exit(1);
   }
   dbg("returned OK");
   exit(0);
}

void BadRun(void)
{  alertSendEmail("moultonr@mir.wustl.edu", "Run", "Bad run file", NULL);
   exit(1);
}
