#****************************************************************************
# create image QC processing db 
#----------------------------------------------------------------------------
#****************************************************************************

CREATE DATABASE IF NOT EXISTS image_qc_db;
USE image_qc_db;

#----------------------------------------------------------------------------
# station: Contains one line for each station (modality, scanner) which is to
# be processed using the qc program, plus lines for groupings of those 
# stations into qc_areas and facilities, and one global line.
#----------------------------------------------------------------------------
DROP TABLE IF EXISTS station;
CREATE TABLE station
(  dba              INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
   facility         VARCHAR(24)  NOT NULL DEFAULT "",
   qc_area          VARCHAR(24)  NOT NULL DEFAULT "",
   station          VARCHAR(24)  NOT NULL DEFAULT "",
   description      VARCHAR(24)  NOT NULL DEFAULT "",
   dicom_tag        VARCHAR(48)  NOT NULL DEFAULT "",
#-------------- process studies during this time window (0 means indefinite )
   begin_process    DATETIME NOT NULL DEFAULT 0,
   end_process      DATETIME NOT NULL DEFAULT 0,
#-------------------------------------------------------------------- indexes
   INDEX dicom     (dicom_tag),
   INDEX hierarchy (facility, qc_area, station),
   UNIQUE INDEX combo (description)
);

#----------------------------------------------------------------------------
# resend: Contains one line for each station in the station table for each
# possible resend destination for that table.
#----------------------------------------------------------------------------
DROP TABLE IF EXISTS resend;
CREATE TABLE resend
(  dba              INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
   station          VARCHAR(24)  NOT NULL DEFAULT "",
   dest_code        VARCHAR(24)  NOT NULL DEFAULT "",
   description      VARCHAR(24)  NOT NULL DEFAULT "",
   def              TINYINT UNSIGNED NOT NULL DEFAULT 0,
   INDEX statn (station)
);

#----------------------------------------------------------------------------
# login: Contains one line for every authorized user of the QC program
#----------------------------------------------------------------------------
DROP TABLE IF EXISTS login;
CREATE TABLE login
(  dba              INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
   user_id          VARCHAR(15)  NOT NULL DEFAULT "",
   password         VARCHAR(15)  NOT NULL DEFAULT "",
   name             VARCHAR(64)  NOT NULL DEFAULT "",
   qc_entry         VARCHAR(24)  NOT NULL DEFAULT "",
#----------------------------------------------------------------------------
# top level this user can go to (0=all, 1=facility, 2=area, 3=station)
#----------------------------------------------------------------------------
   top_level        TINYINT UNSIGNED not NULL DEFAULT 2,
   top              VARCHAR(24)  NOT NULL DEFAULT "",
#------------------ Does this user have right to delete a study? (1=yes,0=no)
   delete_rights    TINYINT UNSIGNED NOT NULL DEFAULT 0,
   last_login       DATETIME     NOT NULL,
#-------------------- dba from message file of last message sent to this user
   last_message     INT UNSIGNED NOT NULL DEFAULT 0,
#-------------------------------------------------------------------- indexes
   UNIQUE INDEX id (user_id)
);

DROP TABLE IF EXISTS message;
CREATE TABLE message
(  dba              INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
   text             VARCHAR(255) NOT NULL DEFAULT ""
);

#----------------------------------------------------------------------------
# study: Contains one line for every study currently on hold for QC.
#----------------------------------------------------------------------------
DROP TABLE IF EXISTS study;
CREATE TABLE study
(
   dba              INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
#----------------------------------------------------------------------------
# QC status 0=needs to be qcd
#           1=QCd, processing in progress
#----------------------------------------------------------------------------
   qc_status        TINYINT UNSIGNED NOT NULL DEFAULT 0,
#----------------------------------------------------------- Source of images
	server           VARCHAR(24)  NOT NULL DEFAULT "",
	study_id         VARCHAR(255) NOT NULL DEFAULT "",
	source           VARCHAR(24)  NOT NULL DEFAULT "",
	modality         VARCHAR(24)  NOT NULL DEFAULT "",
	station_name     VARCHAR(255) NOT NULL DEFAULT "",
	institution_name VARCHAR(255) NOT NULL DEFAULT "",
#------------------------------------------------------- Patient demographics
	pid              VARCHAR(24)  NOT NULL DEFAULT "",
   dnumber          VARCHAR(24)  NOT NULL DEFAULT "",
	pname            VARCHAR(100) NOT NULL DEFAULT "",
   dob              DATE         NOT NULL DEFAULT 0,
   sex              VARCHAR(12)  NOT NULL DEFAULT "",
   visit            VARCHAR(24)  NOT NULL DEFAULT "",
#------------------------------------------------------------------ Exam done
	accn             VARCHAR(24)  NOT NULL DEFAULT "",
   description      VARCHAR(255) NOT NULL DEFAULT "",
   exam_code        VARCHAR(24)  NOT NULL DEFAULT "",
   body_part        VARCHAR(48)  NOT NULL DEFAULT "",
#--------------------------------------------------------- Timing information
   scheduled        DATETIME     NOT NULL,
	study_done       DATETIME     NOT NULL,
   report_done      DATETIME     NOT NULL,
	first_arrive     DATETIME     NOT NULL,
#----------------------------------- when study was qc'd, and user who did it
   qc_done          DATETIME     NOT NULL DEFAULT 0,
   user_id          VARCHAR(15)  NOT NULL DEFAULT "",
#---------------------- For QC'd studies, the destination selected for resend
   dest_code        VARCHAR(24)  NOT NULL DEFAULT "", 
#-------------------------------------------------------------------- indexes
	INDEX name (pname),
	INDEX id   (pid),
	INDEX seq  (accn),
   INDEX UID  (study_id)
);


SHOW TABLES;
DESCRIBE station;
DESCRIBE resend;
DESCRIBE login;
DESCRIBE study;

#---------------------------------------------------- user accounts

GRANT ALL PRIVILEGES ON image_qc_db.* TO qcprs@localhost     IDENTIFIED BY 'farsi';
GRANT ALL PRIVILEGES ON image_qc_db.* TO qcprs@'10.33.162.%' IDENTIFIED BY 'farsi';
GRANT SELECT         ON image_qc_db.* to spectrum@localhost;

FLUSH PRIVILEGES;
