/****************************************************************************************
 * qc Emageon pre-send qc program: gtk callback routines.
 *---------------------------------------------------------------------------------------
 * gcc -g qc_callbacks.c qc_interface.c qc_main.c qc_support.c -o qc `pkg-config --cflags --libs qc`
 ***************************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/select.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <mysql.h>

#include "qc_callbacks.h"
#include "qc_interface.h"
#include "qc_support.h"

#include "dbg.h"
#include "ini.h"
#include "hl7.h"
#include "alert.h"
#include "hl7query.h"
#include "EQCerrors.h"
#include "log.h"

/*------------------------ Internal function prototypes -------------------------------*/
void init_main(void);
char * trim(char * str);
void message(char * title, char * text);
void stat(GtkWidget * bar, char * msg);
int read_ini(void);
void db_error(char * cmd);
void populate_qc_list(void);
void free_study_struct(void);
void update_images(void);
void on_Button_update_clicked_OK(void);
void find_distributors_to_kick(void);
void kick_distributors(void);
int not_valid_id(char * id);
char * adjust_date(char * date, int days);
int init_info(void);
int num_cmp(char * a, char * b);
int elapsed_time(char * start_time_string);

/*---------------------------- macros -------------------------------------------------*/
#define FREE(A) {free(A);A=NULL;}
#define STR_EQ(A,B) !strcasecmp(A,B)
#define STR(A,B) A=strdup(B)
#define LCHR(A) A[strlen(A) - 1]
#define CLEAR(A) memset(A,0,sizeof(A))
#define LOGIN(A) A = lookup_widget(login,#A)
#define MAIN(A) A = lookup_widget(main,#A)
#define RSEND(A) A = lookup_widget(rsend,#A)
#define QUERY(A) A = lookup_widget(query,#A)
#define PASSWORD(A) A = lookup_widget(password,#A)
#define AYS(A) A = lookup_widget(AreYouSure,#A)
#define INFO(A) A = lookup_widget(info,#A)
#define SET(A,B) if(B) gtk_entry_set_text(GTK_ENTRY(A), B)
#define CLR(A) gtk_entry_set_text(GTK_ENTRY(A),"")
#define GET(A) gtk_entry_get_text(GTK_ENTRY(A))
#define SELECT(A) gtk_entry_select_region(GTK_ENTRY(A),0,-1)
#define CLR_TEXT(A) gtk_text_set_point(GTK_TEXT(A),0); gtk_text_forward_delete(GTK_TEXT(A), gtk_text_get_length(GTK_TEXT(A)))
#define SET_TEXT(A,B) gtk_text_insert(GTK_TEXT(A), NULL, NULL, NULL, B, -1)      
#define ON(A) gtk_widget_set_sensitive(A, TRUE)
#define OFF(A) gtk_widget_set_sensitive(A, FALSE)
#define EDITABLE(A,B) gtk_editable_set_editable (GTK_EDITABLE (A), B)
#define FOCUS(A) gtk_widget_grab_focus(A)
#define SHOW(A) gtk_widget_show(A)
#define HIDE(A) gtk_widget_hide(A)
#define CENTER(A) if (center_windows) gtk_window_set_position (GTK_WINDOW (A), GTK_WIN_POS_CENTER)

/*---------------------------------------------- static widget pointers - login screen */
static GtkWidget *login;
static GtkWidget *Lstatus;
static GtkWidget *user_id;
static GtkWidget *passwd;
/*------------------------------------------------------------------------ main screen */
static GtkWidget *main;
static GtkWidget *Mstatus;
static GtkWidget *Combo_qc_group;
static GtkWidget *Combo_modality;
static GtkWidget *Clist_qc;

static GtkWidget *dicom_pname;
static GtkWidget *dicom_pid;
static GtkWidget *dicom_accn;
static GtkWidget *dicom_DOB;
static GtkWidget *dicom_sex;
static GtkWidget *dicom_modality;
static GtkWidget *dicom_description;
static GtkWidget *dicom_DOS;
static GtkWidget *dicom_station_id;

static GtkWidget *idx_pname;
static GtkWidget *idx_pid;
static gchar idx_pid_value[128];
static GtkWidget *idx_accn;
static gchar idx_accn_value[128];
static GtkWidget *idx_DOB;
static GtkWidget *idx_sex;
static GtkWidget *idx_description;
static GtkWidget *idx_DOS;
static GtkWidget *study_status;

static GtkWidget *Button_quit;
static GtkWidget *Button_refresh;
static GtkWidget *Button_chg_pw;
static GtkWidget *Button_delete;
static GtkWidget *Button_cancel;
static GtkWidget *Button_query;
static GtkWidget *Button_update;
/*---------------------------------------------------------- resend destination screen */
static GtkWidget *rsend;
static GtkWidget *Rstatus;
static GtkWidget *Rclist;
static GtkWidget *ROK;
static GtkWidget *RCancel;
/*----------------------------------------------------------------------- query screen */
static GtkWidget *query;
static GtkWidget *query_pname;
static GtkWidget *query_alias;
static GtkWidget *query_pid;
static GtkWidget *query_DOB;
static GtkWidget *query_dnumber;
static GtkWidget *query_sex;
static GtkWidget *QClist;
static GtkWidget *QButton_cancel;
static GtkWidget *QButton_OK;
/*------------------------------------------------------------- Change password screen */
static GtkWidget * password;
static GtkWidget * password_user_id;
static GtkWidget * password_name;
static GtkWidget * password_old;
static GtkWidget * password_new;
static GtkWidget * password_confirm;
static GtkWidget * password_OK;
static GtkWidget * password_cancel;
static GtkWidget * Pstatusbar;
/*---------------------------------------------------------- Error/Info message screen */
static GtkWidget *msg;
static GtkWidget *Message;
/*------------------------------------------------------- Are You Sure? message screen */
#define MAIN_UPDATE 1
#define MAIN_DELETE 2
static char which_routine; 
static GtkWidget *AreYouSure;
static GtkWidget *AMessage;
static GtkWidget *AButton_OK;
static GtkWidget *AButton_cancel;
/*-------------------------------------------------- system information message screen */
static GtkWidget *info;
static GtkWidget *info_text;
static GtkWidget *info_clear;
static GtkWidget *info_save;
static GtkWidget *Istatusbar;

static gchar * col[5] = {NULL};

/*-------------------------------------------------------------------------- .ini file */
#define DEF_INI_PFN "qc.ini"
static char * cwd;                   /* Current working directory (program dir) */
static char inipfn[256];             /* .ini file pfn (def = cwd/qc.ini) */
FILE * debug_fp;                     /* debug file pointer, or NULL if debug is off */
static char * distributor_name;
static int center_windows = 1;       /* Should windows be centered on desktop? */
static int query_link = 5;           /* Query link to use for MPI queries */
static int days_before_DOS = 5;      /* Days before DOS to cut off MPI queries */

/*---------------------------------------------------------------------------- logging */
static char log_msg[4096];
static time_t elapsed_seconds;
static char elapsed_string[128];

/*----------------------------------------------------------------------- MySQL access */
static char * dbhost;
MYSQL * conn = NULL;
MYSQL_RES * res = NULL;
MYSQL_ROW row = NULL;
static char sql_query[4096];
char * db_err_msg = "Database error:\n%s\n%s\nCall (314)747-3257";

/*----------------------------------------------------------------------- IDXRad query */
#define EXAM MPIQuery.exam[exam]
static struct _hl7_accn_query_ AccnQuery;
static struct _hl7_mpi_query_  MPIQuery;
static struct _hl7_mpi_query_exam_ * MPIQueryExam;
static int exam;
/*------------------------------------------------------------ Station table structure */
#define UNKNOWN_STATION "Unknown station"
#define STATION Station[st]
struct _station_ {
   unsigned long int dba;
   char * facility;
   char * group;
   char * department;
   char * station;
   char * description;
   char * dicom_tag;
   char id_by;
   char * d_station;
};
static struct _station_ * Station = NULL;
static int st, max_st;
/*-------------------------------------------------------------- Login table structure */
struct _login_ {
   unsigned long int dba;
   char * user_id;
   char * password;
   char * name;
   char * qc_entry;
   char top_level;
   char * top;
   char delete_rights; 
   char * last_login;
   unsigned long int last_message;
   char super_user;
};
static struct _login_ Login;     /* This is current logged-in user */

/*--------------------------------------------- informational messages table structure */
#define MESSAGE Mesage[mess]
struct _message_ {
   unsigned long int dba;
   char * text;
};
static struct _message_ * Mesage = NULL;
static int mess, max_message;

/*-------------------------------------------------------------- study table structure */
#define STUDY Study[study]
struct _study_ {
   unsigned long int dba;
   char qc_status;
   char * server;
   char * study_id;
   char * source;
   char * modality;
   char * station_name;
   char * institution_name;
   char * pid;
   char * dnumber;
   char * pname;
   char * dob;
   char * sex;
   char * visit;
   char * accn;
   char * description;
   char * exam_code;
   char * body_part;
   char * scheduled;
   char * study_done;
   char * report_done;
   char * first_arrive;
   unsigned long errors;
   char * qc_done;
   char * user_id;
   char * dest_code;
/*-------------------------------------------------------------------- Items not in db */
   gint row;                             /* row number in Clist, or -1 if not in Clist */
};
static struct _study_ * Study = NULL;
static int study, max_study;

/*------------------------------------------------------------- resend table structure */
#define RESEND Resend[resend]
struct _resend_ {
   unsigned long int dba;
   char * station;
   char * dest_code;
   char * description;
   char def;
};
static struct _resend_ * Resend = NULL;
static int resend, max_resend, selected_resend;
/*---- flag = 1 if we are displaying the resend description, 0 if the destination code */
static char display_dest_code;

/*------------------------------------- Used to "kick" distributor resend_qc processes */
#define MAX_DISTRIBUTOR 10
#define DIST distributor[dist]
static char * distributor[MAX_DISTRIBUTOR +1];
static char dist;

/*---------------------------------------------------------------------------------------
 * Startup routine, called after gtk_init for initial processing.  Creates and shows 
 * login dialog for processing.
 *-------------------------------------------------------------------------------------*/
int qc_init(int argc, char * argv[])
{  char * p;     
/*---------------------------------------------------------------------------------------
 * Program runs out of directory binary is in.  Make that current.  Then use parameter 1
 * or default to complete .ini file pfn.
 *-------------------------------------------------------------------------------------*/
   if (p = MakeProgDirCurrent(argv[0])) STR(cwd,p);
   else { PrtIniErrorString(stdout,"Startup"); exit(1); }
   strcpy(inipfn,cwd); strcat(inipfn,DEF_INI_PFN);

   read_ini();
   hl7IncludeCanceledExams(0);  /* We don't want to include cancelled exams in queries */
   
   msg = create_msg();
   CENTER(msg);
   Message = lookup_widget(msg,"Message");

   main = create_main();
   CENTER(main);
   Mstatus = lookup_widget(main, "statusbar");   
   MAIN(dicom_pname);
   MAIN(dicom_pid);
   MAIN(dicom_accn);
   MAIN(dicom_DOB);
   MAIN(dicom_sex);
   MAIN(dicom_modality);
   MAIN(dicom_description);
   MAIN(dicom_DOS);
   MAIN(dicom_station_id);
   
   MAIN(idx_pname);
   MAIN(idx_pid);
   MAIN(idx_accn);
   MAIN(idx_DOB);
   MAIN(idx_sex);
   MAIN(idx_description);
   MAIN(idx_DOS);
   MAIN(study_status);

   MAIN(Button_delete);
   MAIN(Button_cancel);
   MAIN(Button_query);
   MAIN(Button_update);

   rsend = create_Resend();
   CENTER(rsend);
   RSEND(Rstatus);
   RSEND(Rclist);
   RSEND(ROK);
   RSEND(RCancel);

   query = create_query();
   CENTER(query);
   QUERY(query_pname);
   QUERY(query_alias);
   QUERY(query_pid);
   QUERY(query_DOB);
   QUERY(query_dnumber);
   QUERY(query_sex);
   QUERY(QClist);
   QUERY(QButton_cancel);
   QUERY(QButton_OK);

   AreYouSure = create_AreYouSure();
   CENTER(AreYouSure);
   AYS(AMessage);
   AYS(AButton_OK);
   AYS(AButton_cancel);
   
   password = create_password();
   CENTER(password);
   PASSWORD(password_user_id);
   PASSWORD(password_name);
   PASSWORD(password_old);
   PASSWORD(password_new);
   PASSWORD(password_confirm);
   PASSWORD(password_OK);
   PASSWORD(password_cancel);
   PASSWORD(Pstatusbar);

   info = create_info();
   CENTER(info);
   INFO(info_text);
   INFO(info_clear);
   INFO(info_save);
   INFO(Istatusbar);
   stat(Istatusbar,"Clear msg or Save for next login");

   login = create_login();
   CENTER(login);
   Lstatus = lookup_widget(login, "Lstatusbar");
   LOGIN(user_id);
   passwd = lookup_widget(login, "password");
   SHOW(login);
   
   stat(Lstatus,"Enter login and password");

   return (0); 
}

/****************************************************************************************
 ****************************************************************************************
 * Login dialog routines
 ****************************************************************************************
 ***************************************************************************************/
gboolean on_login_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data) { return FALSE; }

/*---------------------------------------------------------------------------------------
 * Login dialog, OK button clicked.
 * We validate that the user id and password are non-blank, then look them up in the 
 * database.  If they match, we continue on to the main window, otherwise we clear and
 * retry.
 *-------------------------------------------------------------------------------------*/
void on_LButton_OK_clicked (GtkButton * button, gpointer user_data)
{  char uid[24], pw[24];
   
/*---------------------------------------------------------- Pull user id and password */
   strcpy(uid,GET(user_id));
   strcpy(pw,GET(passwd));
   trim(uid);
   trim(pw);

/*--------------------------------------- Validate that they are non-blank (non-empty) */
   if (!strlen(uid) || !strlen(pw))
   {  CLR(user_id);
      CLR(passwd);
      gtk_widget_grab_focus(user_id);
      message("Error!",
              "User Id and Password\n"
              "Must be non-blank");
      return;
   }
/*------------------------------------- Validate user id and password against database */
   if (!conn)                                                         /* open db */
   {  conn = mysql_init(NULL); 
      conn = mysql_real_connect(conn, dbhost, "qcprs", "farsi", "image_qc_db", 0, NULL, 0);
   }      
   if (!conn) {db_error("sqlOpen"); return;}
   CLEAR(sql_query);                                                      /* query */
   sprintf(sql_query,"SELECT * FROM login WHERE user_id = \"%s\"",uid);
   if (mysql_query(conn,sql_query)) {db_error("query"); return;}
   res = mysql_store_result(conn);
   if (!res) {db_error("store result"); return;}
/*------------------------------------------------------ no row found matching user id */
   if (mysql_num_rows(res) != 1)
   {  mysql_free_result(res);
      CLR(user_id);
      CLR(passwd);
      gtk_widget_grab_focus(user_id);
      message("Error!", "Invalid user Id");
      return;
   }
   row = mysql_fetch_row(res);    
   
   if (strcmp(pw,row[2]))                             /* validate password */
   {  message("Error!", "Invalid password");
      return;
   }

   stat(Lstatus,"Logging in");
   
   Login.dba = atoi(row[0]);                         /* Load user information */
   STR(Login.user_id,row[1]);
   STR(Login.password,row[2]);
   STR(Login.name,row[3]);
   STR(Login.qc_entry,row[4]);
   Login.top_level = atoi(row[5]);
   STR(Login.top,row[6]);
   Login.delete_rights = atoi(row[7]);
   STR(Login.last_login,row[8]);
   Login.last_message = atoi(row[9]);
   Login.super_user = atoi(row[10]);

   mysql_free_result(res);
/*------------------------------------------------------------------ Update last_login */
   CLEAR(sql_query);
   sprintf(sql_query,"UPDATE login SET last_login = NOW() WHERE dba = %d", Login.dba);
   mysql_query(conn,sql_query);
   
   HIDE(login);

/*--------------------------------------------- Are there any system messages to show? */
   CLEAR(sql_query);
   sprintf(sql_query,"SELECT * FROM message WHERE dba > %d ORDER BY dba", Login.last_message);
   if (mysql_query(conn,sql_query)|| !(res=mysql_store_result(conn)))
   {  init_main();
      SHOW(main);
      db_error("info query");
      return;
   }
   max_message = mysql_num_rows(res);
/*---------------------------------------------- No messages, go direct to main dialog */
   if (!max_message)
   {  init_main();               
      SHOW(main);
      return;
   } 
/*---------------------------------------------------------------------- Load messages */
   Mesage = malloc(max_message * sizeof(struct _message_));
   memset(Mesage,0,max_message * sizeof(struct _message_));
   for (mess = 0; mess < max_message; mess++)
   {  row = mysql_fetch_row(res);
      MESSAGE.dba = atoi(row[0]);
      STR(MESSAGE.text,row[1]);
   }
   mysql_free_result(res);
   mess = 0;
   init_info();
   return;
}


/****************************************************************************************
 ****************************************************************************************
 * Main dialog routines
 ****************************************************************************************
 ***************************************************************************************/

gboolean on_main_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data)
{
  if (conn) mysql_close(conn);
  return FALSE;
}

/*------------------------------------------ User changed study filter combo box entry */
void on_Combo_qc_group_changed(GtkCombo *combo, gpointer user_data) {populate_qc_list();}
void on_Combo_modality_changed(GtkCombo *combo, gpointer user_data) {populate_qc_list();}

/*----------------------------------------------------- Study to qc selected from list */
void on_clist_qc_select_row (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer user_data)
{  int st;
   for (study=0; study<max_study && STUDY.row != row; study++); /* find study */
/*---------------------------------------------------------------- Display DICOM stuff */
   SET(dicom_pname,STUDY.pname);
   SET(dicom_pid,STUDY.pid);
   SET(dicom_accn,STUDY.accn);
   SET(dicom_DOB,STUDY.dob);
   SET(dicom_sex,STUDY.sex);
   SET(dicom_modality,STUDY.modality);
   SET(dicom_description,STUDY.description);
   SET(dicom_DOS,STUDY.study_done);
   SET(dicom_station_id,STUDY.station_name);
/*-------------------------- Possible overide display station_name, from station table */
   for (st=0; st < max_st; st++)
   {  if(STATION.d_station)
      {  switch (STATION.id_by)
         {  case 1:                                     /* id by station name */
               if (STR_EQ(STUDY.station_name,STATION.station)) SET(dicom_station_id,STATION.d_station);
               break;
            case 2:                                     /* id by institution name */
               if (STR_EQ(STUDY.institution_name,STATION.dicom_tag)) SET(dicom_station_id,STATION.d_station);
               break;
            default:
               break;
         } 
      }
   }
/*------------------------------------------------------------------------- Error text */
   CLR_TEXT(study_status);
   EQC_error = STUDY.errors;
   SET_TEXT(study_status,EQCGetQCErrorsMsg(1));
/*-------------------------------------------------------------------- Clear IDX stuff */
   CLR(idx_pname);
   CLR(idx_DOB);
   CLR(idx_sex);
   CLR(idx_description);
   CLR(idx_DOS);
/*------------------------------------------------------------------------ Set buttons */
   if (Login.delete_rights) ON(Button_delete);
   ON(Button_cancel);
   OFF(Button_query);
   OFF(Button_update);
/*---------------------------------------------------------------------------------------
 * Now we "lead" them to fixing the study, by:
 * 1. Defaulting the idx pid and accn to the DICOM values.
 * 2. If those are both non-blank, and the problems in the study can be fixed by updating
 *    data from an IDXrad query, lighting the query idxrad button.
 * 3. Selecting the idx_pid text and giving that entry focus.
 *-------------------------------------------------------------------------------------*/
   if (EQCQCRepairable())
   {  stat(Mstatus,"Edit idx pid and accession, then click \"Query IDXRad\"");
      SET(idx_pid,STUDY.pid);
      strcpy(idx_pid_value,STUDY.pid);
      EDITABLE(idx_pid,TRUE);
      SELECT(idx_pid);
      SET(idx_accn,STUDY.accn);
      strcpy(idx_accn_value,STUDY.accn);
      EDITABLE(idx_accn,TRUE);
      if ((strlen(idx_pid_value) || strlen(idx_accn_value))) ON(Button_query); 
      else OFF(Button_query);
      FOCUS(idx_pid);
   }
   else
   {  stat(Mstatus,"This study can't be fixed here. Delete, fix on PACS or modality, and resend.");
      SET(idx_pid,"-------");
      EDITABLE(idx_pid,FALSE);
      SET(idx_accn,"-------");
      EDITABLE(idx_accn,FALSE);
      OFF(Button_query);
   }
}

/*----- User changed text in idx pid or accn boxes (only possible when study selected) */
void on_Entry_idx_changed(GtkEntry * entry, gpointer data)
{  if (entry == GTK_ENTRY(idx_pid)) strcpy(idx_pid_value,gtk_entry_get_text(entry));
   else strcpy(idx_accn_value,gtk_entry_get_text(entry));
   if ((strlen(idx_pid_value) || strlen(idx_accn_value)) && EQCQCRepairable()) ON(Button_query); 
   else OFF(Button_query);
}   

void on_Button_refresh_clicked (GtkButton * button, gpointer user_data) {populate_qc_list();}

/*---------------------------------------------------------------------------------------
 * User clicked delete image date button for study (This has an "are you sure?")
 *-------------------------------------------------------------------------------------*/
static char ays_delete_msg[] = {"This will delete images\n"
                                "for the selected study."};
void on_Button_delete_clicked (GtkButton * button, gpointer user_data)
{  OFF(Button_delete);
   which_routine = MAIN_DELETE;
   gtk_label_set_text(GTK_LABEL(AMessage), ays_delete_msg);
   OFF(main);
   SHOW(AreYouSure);
   return;
}

void on_Button_delete_clicked_OK(void)
{  time_t now = time(NULL); 
   char ct[16];
   
   find_distributors_to_kick();
   
   cftime(ct, "%Y%m%d%H%M%S", &now);
/*-------------------- Create query to update qc db to indicate delete (qc_status = 2) */
   CLEAR(sql_query);
   sprintf(sql_query, "UPDATE study SET qc_status = 2, qc_done = %s, user_id =\"%s\" "
                      "WHERE qc_status = 0 AND modality = \"%s\" AND study_id = \"%s\"", 
                      ct, Login.user_id, STUDY.modality, STUDY.study_id);
   if (mysql_query(conn,sql_query)) 
   {  ON(main);
      db_error("query");
      return;
   }
   
/*--------------------------------------------------------------------- QC log message */
   CLEAR(log_msg);
   sprintf(log_msg, "Study qc'd by %s (%s) - Deleted\n", Login.name, Login.user_id);
   sprintf(em,"   pid........ %s\n", STUDY.pid);              strcat(log_msg,em);
   sprintf(em,"   dnumber.... %s\n", STUDY.dnumber);          strcat(log_msg,em);
   sprintf(em,"   accn....... %s\n", STUDY.accn);             strcat(log_msg,em);
   sprintf(em,"   name....... %s\n", STUDY.pname);            strcat(log_msg,em);
   sprintf(em,"   DOB........ %s\n", STUDY.dob);              strcat(log_msg,em);
   sprintf(em,"   sex........ %s\n", STUDY.sex);              strcat(log_msg,em);
   sprintf(em,"   source..... %s\n", STUDY.source);           strcat(log_msg,em);
   sprintf(em,"   modality... %s\n", STUDY.modality);         strcat(log_msg,em);
   sprintf(em,"   exam code.. %s\n", STUDY.exam_code);        strcat(log_msg,em);
   sprintf(em,"   description %s\n", STUDY.description);      strcat(log_msg,em);
   sprintf(em,"   scheduled.. %s\n", STUDY.scheduled);        strcat(log_msg,em);
   sprintf(em,"   study done. %s\n", STUDY.study_done);       strcat(log_msg,em);
   sprintf(em,"   station.... %s\n", STUDY.station_name);     strcat(log_msg,em);
   sprintf(em,"   institution %s\n", STUDY.institution_name); strcat(log_msg,em);
   sprintf(em,"   study UID.. %s\n", STUDY.study_id);         strcat(log_msg,em);

   elapsed_time(STUDY.first_arrive);
   sprintf(em,"   arrived.... %s - time to qc %s\n", STUDY.first_arrive, elapsed_string);
   strcat(log_msg,em);
   
   logMsgLevel("EQChold",log_msg,1);

   
   kick_distributors();
   
   ON(main);
   populate_qc_list();
}


void on_Button_cancel_clicked (GtkButton * button, gpointer user_data)
{  CLR(dicom_pname);
   CLR(dicom_pid);
   CLR(dicom_accn);
   CLR(dicom_DOB);
   CLR(dicom_sex);
   CLR(dicom_modality);
   CLR(dicom_description);
   CLR(dicom_DOS);
   CLR(dicom_station_id);
   
   CLR(idx_pname);
   CLR(idx_pid);
   CLR(idx_accn);
   CLR(idx_DOB);
   CLR(idx_sex);
   CLR(idx_description);
   CLR(idx_DOS);
   CLR_TEXT(study_status);
   
   OFF(Button_delete);
   OFF(Button_cancel);
   OFF(Button_query);
   OFF(Button_update);

   EDITABLE(idx_pid,FALSE);
   EDITABLE(idx_accn,FALSE);

   gtk_clist_unselect_all(GTK_CLIST(Clist_qc));

   stat(Mstatus,"Select study from list to QC");
}

/*---------------------------------------------------------------------------------------
 * User clicked button to do IDXrad query.  This button is only lit when a study is
 * selected and non-blank entries are in the idx_pid and/or idx_accn boxes.
 * ------------------------------------------------------------------------------------*/
void on_Button_query_clicked (GtkButton * button, gpointer user_data)
{  char query_feed = 0;
   char * p;
   int i, j;
   OFF(Button_query);
/*------------------------------------ Clear any old idx data, freeze window for query */
   CLR(idx_pname);
   CLR(idx_DOB);
   CLR(idx_sex);
   CLR(idx_description);
   CLR(idx_DOS);
   OFF(main);
   stat(Mstatus,"IDXRad query in process");

/*---------------------------------------------------------------------------------------
 * validate that at least one of idx pid and accn are numeric, seven digits or more
 *-------------------------------------------------------------------------------------*/ 
   strcpy(idx_pid_value,GET(idx_pid));
   strcpy(idx_accn_value,GET(idx_accn));
   if (not_valid_id(idx_pid_value) && not_valid_id(idx_accn_value))
   {  stat(Mstatus,"Edit IDXRad pid and accession, then click \"Query IDXRad\"");
      ON(main);
      SELECT(idx_pid);
      FOCUS(idx_pid);
      message("Error!","invalid IDXRad patient id\nand accession number");
      return;
   }
   
   hl7SelectHost(NULL, NULL);
   
/*---------------------------------------------------------------------------------------
 * If the accn is valid, we are doing an accn query.  If the pid is also valid, we must
 * match both the pid and accn for a hit.
 *-------------------------------------------------------------------------------------*/
   if (!not_valid_id(idx_accn_value))
   {  memset(&AccnQuery,0,sizeof(AccnQuery));
      AccnQuery.accn = atoi(idx_accn_value); 
/*--------------------------------------------------- Try all the feeds before quiting */
      while (p = (char *)hl7QSelectServer(query_feed++, 1, 0))
      {  sprintf(em,"IDXRad query in process - %s", p); stat(Mstatus,em);
/*-------------------------------------------------------------------- Query this feed */
         if (hl7AccnQuery(&AccnQuery)) 
         {  if (hl7q_errno == HL7Q_CANCELED)  /* exam is canceled */
            {  stat(Mstatus,"Edit IDXRad pid and accession, then click \"Query IDXRad\"");
               ON(main);
               SELECT(idx_accn);
               FOCUS(idx_accn);
               message("Canceled!","accession is for canceled exam");
               return;
            }
            if (hl7q_errno != HL7Q_NAK) continue; /* query error, just try next server */ 
/*------------------------------------------------------------- No such accn in IDXRad */
            stat(Mstatus,"Edit IDXRad pid and accession, then click \"Query IDXRad\"");
            ON(main);
            SELECT(idx_accn);
            FOCUS(idx_accn);
            message("Error!","no such accession in IDXRad");
            return;
         }
/*---------------------------------------- Matching pid and accn and pid doesn't match */
         if (!not_valid_id(idx_pid_value) && AccnQuery.mpi != atoi(idx_pid_value))
         {  stat(Mstatus,"Edit IDXRad pid and accession, then click \"Query IDXRad\"");
            ON(main);
            SELECT(idx_pid);
            FOCUS(idx_pid);
            message("Error!","no such accession for\nthis patient in IDXRad");
            return;
         }
/*------------------------------------------------------ Load, display returned values */
         SET(idx_pname,AccnQuery.pname);
         sprintf(em,"%07d", AccnQuery.mpi);
         SET(idx_pid,em);
         strcpy(idx_pid_value,em);
         sprintf(em,"%07d", AccnQuery.accn);
         SET(idx_accn,em);
         sprintf(em,"%4.4s-%2.2s-%2.2s", AccnQuery.dob, AccnQuery.dob +4, AccnQuery.dob +6);
         SET(idx_DOB, em);
         SET(idx_sex, AccnQuery.sex);
         SET(idx_description, AccnQuery.exam_desc);
         sprintf(em,"%s %4.4s-%2.2s-%2.2s %2.2s:%2.2s", AccnQuery.status, AccnQuery.time, 
            AccnQuery.start_time +4, AccnQuery.start_time +6, AccnQuery.start_time +8, 
            AccnQuery.start_time +10);
         if (AccnQuery.mpi)
         {  SET(idx_DOS,em);
            OFF(Button_query);
            ON(Button_update);
            EDITABLE(idx_pid, FALSE);
            EDITABLE(idx_accn, FALSE);
            stat(Mstatus,"Update Image data with IDXRad data shown, or Cancel");
         }
         else stat(Mstatus,"Not a registered patient");
         ON(main);
         return;
      }
/*---------------------- If the above while loop exits, we were unable to do the query */
      stat(Mstatus,"Edit IDXRad pid and accession, then click \"Query IDXRad\"");
      ON(main);
      message("Error!","Unable to query IDXrad");
      return;
   }
/*---------------------------------------------------------------------------------------
 * Here the pid is valid but the accn isn't, we do a query for all the studies for this 
 * patient, filling in the query window and displaying it. 
 *-------------------------------------------------------------------------------------*/
   hl7QSelectServer(query_link, -1, 1);
   MPIQuery.mpi = atoi(idx_pid_value);
   if (hl7MPIQuery(&MPIQuery))
   {  ON(main);
      sprintf(em,"IDXrad MPI query failed\n%s", GetHl7QErrorString());
      if (hl7q_errno == HL7Q_MPI) strcpy(em,"No such MPI number\nin IDXrad");
      message("Error!",em);
      return;
   }
/*-------------------------- are there any exams which are not cancelled for this mpi */
   if (!MPIQuery.num_exams)
   {  ON(main);
      stat(Mstatus,"No active exams for this patient");
      return;
   }
/*------------------------------------ Do accn query on first exam to get demographics */  
   hl7QSelectServer(0, -1, 1);
   AccnQuery.accn = MPIQuery.exam->accn;
   if (hl7AccnQuery(&AccnQuery))
   {  ON(main);
      sprintf(em,"IDXrad MPI/Accn query failed\n%s", GetHl7QErrorString());
      message("Error!",em);
      return;
   }
/*-------------------------------------------------------------- Load mpi query screen */
   SET(query_pname,AccnQuery.pname);
   SET(query_alias,AccnQuery.alias);
   SET(query_pid,idx_pid_value);
   SET(query_dnumber,AccnQuery.dnumber);
   sprintf(em,"%4.4s-%2.2s-%2.2s", AccnQuery.dob, AccnQuery.dob +4, AccnQuery.dob +6);
   SET(query_DOB,em);
   SET(query_sex,AccnQuery.sex);
/*------------------------------------------------------------------------- load clist */
   gtk_clist_clear(GTK_CLIST(QClist));
   for (exam=0; exam < MPIQuery.num_exams; exam++)
   {  sprintf(em,"%07d", EXAM.accn); 
      STR(col[0],em);
      STR(col[1],EXAM.status);
      sprintf(em,"%4.4s-%2.2s-%2.2s %2.2s:%2.2s", EXAM.date_time, EXAM.date_time +4, 
         EXAM.date_time +6, EXAM.date_time +8, EXAM.date_time +10);
      STR(col[2],em);
      STR(col[3],EXAM.code);
      STR(col[4],EXAM.desc);
      gtk_clist_append(GTK_CLIST(QClist), col);
      for (j=0; j < 5; j++) free(col[j]);
   }
   ON(QButton_cancel);
   OFF(QButton_OK);
/*------------------------------------ If only one line, select it and light OK button */
   if (MPIQuery.num_exams == 1) 
   {  gtk_clist_select_row(GTK_CLIST(QClist), 0, -1);
      exam = 0;
      ON(QButton_OK);
   }

/*----------------------------------------- inactivate main window, light query window */ 
   OFF(main);
   SHOW(query);
   return;
}

/*---------------------------------------------------------------------------------------
 * User clicked Update Image data button for study (This has an "are you sure?")
 *-------------------------------------------------------------------------------------*/
static char ays_update_msg[] = {"This will update images\n"
                                "in the selected study to\n"
                                "match idx information shown" };
void on_Button_update_clicked (GtkButton * button, gpointer user_data)
{  which_routine = MAIN_UPDATE;
   gtk_label_set_text(GTK_LABEL(AMessage), ays_update_msg);
   OFF(main);
   SHOW(AreYouSure);
   return;
}

void on_Button_update_clicked_OK(void)
{  int def;
   display_dest_code = 0; 
/*------------------------------------------------- Lock main window & indicate status */
   OFF(main);
   stat(Mstatus,"Updating images to match IDXRad data shown");
/*------------------------ Query resend table for resend destinations for this station */
   CLEAR(sql_query);
   sprintf(sql_query,"SELECT * FROM resend WHERE station = BINARY \"%s\"", STUDY.station_name);
   if (!Login.super_user) strcat(sql_query," AND resend.super_user = 0");
   if (mysql_query(conn,sql_query)){ON(main); db_error("resend query"); return;}
   res = mysql_store_result(conn);
   if (!res){ON(main); db_error("resend store result"); return; }
   max_resend = mysql_num_rows(res);
/*---------------------------------------------------------------------------------------
 * If there are no resend destinations for this station, regular users are told to call
 * the PACS help desk.  Super users are warned, and then given a list of all destinations
 * in the resend table to choose from.  
 *-------------------------------------------------------------------------------------*/
   if (!max_resend)
   {  if (!Login.super_user) 
      {  ON(main); 
         message("Error!","No resend address for this station\n"
                          "Please call 362-8475 ext 2"); 
         return; 
      }
      else
      {  display_dest_code = 1;
         CLEAR(sql_query);
         strcpy(sql_query,"SELECT MAX(dba), MAX(station), dest_code, MAX(description), "
                          "MAX(def), MAX(super_user) FROM resend GROUP BY dest_code");
         if (mysql_query(conn,sql_query)){ON(main); db_error("resend all query"); return; }
         res = mysql_store_result(conn);
         if (!res){ON(main); db_error("resend all store result"); return; }
         max_resend = mysql_num_rows(res);
         if (!max_resend) 
         {  ON(main);
            message("Error!","No resend destinations at all");
            return;
         }
      }
   }
   Resend = malloc(max_resend * sizeof(struct _resend_));
   memset(Resend,0,max_resend * sizeof(struct _resend_));
   for (resend = 0; resend < max_resend; resend++)
   {  row = mysql_fetch_row(res);
      RESEND.dba = atoi(row[0]);   
      STR(RESEND.station,row[1]);
      STR(RESEND.dest_code,row[2]);
      STR(RESEND.description,row[3]);
      RESEND.def = atoi(row[4]);
   }
   mysql_free_result(res);
/*-------------------------------- If more than one destination, pop the resend window */
   selected_resend = -1;
   OFF(ROK);
   if (max_resend > 1)
   {  gtk_clist_clear(GTK_CLIST(Rclist));
      for (def = resend = 0; resend < max_resend; resend++) 
      {  if (display_dest_code) col[0] = RESEND.dest_code;
         else col[0] = RESEND.description;
         gtk_clist_append(GTK_CLIST(Rclist), col);
         if (!def && RESEND.def)
         {  def = 1;
            selected_resend = resend;
            ON(ROK);
            gtk_clist_select_row(GTK_CLIST(Rclist), resend, -1);
         }
      }
      stat(Rstatus,"Select destination and click OK");
      SHOW(rsend);
      if (display_dest_code)
      {  message("Error!","No resend table entries for this station\n"
                          "All resend destinations are shown");
      }
      return;
   }
   resend = 0;
   update_images();
   return;
}

void on_Button_chg_pw_clicked (GtkButton * button, gpointer user_data)
{  OFF(main);
   SET(password_user_id,Login.user_id);
   SET(password_name,Login.name);
   CLR(password_old);
   CLR(password_new);
   CLR(password_confirm);
   OFF(password_OK);
   stat(Pstatusbar,"Enter current pw, new pw & confirm");
   SHOW(password);
   FOCUS(password_old);
   return;
}

/****************************************************************************************
 ****************************************************************************************
 * Resend destination selection dialog
 ****************************************************************************************
 ***************************************************************************************/
void on_RCancel_clicked (GtkButton * button, gpointer user_data)
{  HIDE(rsend);
   ON(main);
   on_Button_cancel_clicked (NULL, NULL);
}


void on_ROK_clicked (GtkButton * button, gpointer user_data)
{  HIDE(rsend);
   resend = selected_resend;
   update_images();
   return;
}

gboolean on_Resend_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data)
{  on_RCancel_clicked (GTK_BUTTON(RCancel), user_data);
   return TRUE;;
}


void on_Rclist_select_row (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer user_data)
{  selected_resend = row;
   ON(ROK);
   return;
}

/****************************************************************************************
 ****************************************************************************************
 * pid query exam selection dialog
 ****************************************************************************************
 ***************************************************************************************/
void on_Qclist_select_row (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer user_data)
{  exam = row;
   ON(QButton_OK);
   return;
}


void on_QButton_cancel_clicked (GtkButton * button, gpointer user_data)
{  HIDE(query);
   ON(main);
   return;
}

/*---------------------------------------------------------------------------------------
 * OK button on MPI query dialog clicked.  Do accn query on the selected exam and return
 * this data
 *-------------------------------------------------------------------------------------*/
void on_QButton_OK_clicked (GtkButton * button, gpointer user_data)
{  AccnQuery.accn = EXAM.accn;
   if (hl7AccnQuery(&AccnQuery))
   {  ON(main);
      sprintf(em,"IDXrad MPI/Accn query failed\n%s", GetHl7QErrorString());
      message("Error!",em);
      return;
   }
   SET(idx_pname,AccnQuery.pname);
   sprintf(em,"%07d", AccnQuery.mpi);
   SET(idx_pid,em);
   strcpy(idx_pid_value,em);
   sprintf(em,"%07d", AccnQuery.accn);
   SET(idx_accn,em);
   sprintf(em,"%4.4s-%2.2s-%2.2s", AccnQuery.dob, AccnQuery.dob +4, AccnQuery.dob +6);
   SET(idx_DOB, em);
   SET(idx_sex, AccnQuery.sex);
   SET(idx_description, AccnQuery.exam_desc);
   sprintf(em,"%s %4.4s-%2.2s-%2.2s %2.2s:%2.2s", AccnQuery.status, AccnQuery.time, 
   AccnQuery.time +4, AccnQuery.time +6, AccnQuery.time +8, AccnQuery.time +10);
   SET(idx_DOS,em);
   OFF(Button_query);
   ON(Button_update);
   EDITABLE(idx_pid, FALSE);
   EDITABLE(idx_accn, FALSE);
   stat(Mstatus,"Update Image data with IDXRad data shown, or Cancel");
   ON(main);
   HIDE(query);
   return;
}

/****************************************************************************************
 ****************************************************************************************
 * Change Password dialog
 ****************************************************************************************
 ***************************************************************************************/
void on_password_cancel_clicked (GtkButton * button, gpointer user_data)
{  HIDE(password);
   ON(main);
   return;
}
void on_password_OK_clicked (GtkButton * button, gpointer user_data)
{  char old[24],new[24],confirm[24];
   strcpy(old,GET(password_old));
   trim(old);
   if (strcmp(Login.password,old))
   {  CLR(password_old);
      gtk_widget_grab_focus(password_old);
      stat(Pstatusbar,"Enter current pw, new pw & confirm");
      message("Error!",
              "Password Incorrect\n"
              "Remember that passwords\n"
              "are case sensitive");
      return;
   }
   strcpy(new,GET(password_new));
   strcpy(confirm,GET(password_confirm));
   trim(new);
   trim(confirm);
   if (strcmp(new,confirm))
   {  CLR(password_new);
      CLR(password_confirm);
      gtk_widget_grab_focus(password_new);
      stat(Pstatusbar,"Enter new password & confirm");
      message("Error!",
              "New password\n"
              "not confirmed");
      return;
   }
   FREE(Login.password)
   STR(Login.password,new);
   CLEAR(sql_query);
   sprintf(sql_query, "UPDATE login SET password = \"%s\" WHERE dba = %d", Login.password, Login.dba);
   if (mysql_query(conn,sql_query))
   {  message("Error!",
              "Unable to update password");
      stat(Pstatusbar,"Enter current pw, new pw & confirm");
      return;
   }
   HIDE(password);
   ON(main);
   message("Information",
           "Password has been changed");
   return;
}

gboolean on_password_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data)
{  on_password_cancel_clicked(GTK_BUTTON(password_cancel), user_data);
   return TRUE;
}
void on_password_entry_changed(GtkEntry *entry, gpointer user_data)
{  if (strlen(GET(password_old)) && strlen(GET(password_new)) && strlen(GET(password_confirm))) ON(password_OK);
   else OFF(password_OK);
   return;
}

/****************************************************************************************
 * message box.  We create this at the beginning (in qc_init) and pull the message label
 * widget pointer.  Then we just change the title and message and show the widget, then
 * hide it when the delete or OK are clicked.
 ***************************************************************************************/

gboolean on_msg_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{  HIDE(msg);
   return TRUE;
}

void on_msg_OK_clicked(GtkButton * button, gpointer user_data){HIDE(msg);}

void message(char * title_text, char * msg_text)
{  gtk_window_set_title(GTK_WINDOW(msg),title_text);
   gtk_label_set_text(GTK_LABEL(Message),msg_text);
   SHOW(msg);
}

/*---------------------------------------------------------------------------------------
 * Are you sure message box.
 *-------------------------------------------------------------------------------------*/
gboolean on_AreYouSure_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data)
{  on_AreYouSure_cancel_clicked(GTK_BUTTON(AButton_cancel), user_data);
   return (TRUE);
}
void on_AreYouSure_OK_clicked (GtkButton * button, gpointer user_data)
{  HIDE(AreYouSure);
   ON(main);
   switch (which_routine)
   {  case MAIN_UPDATE:
         on_Button_update_clicked_OK();
         break;
      case MAIN_DELETE:
         on_Button_delete_clicked_OK();
         break;
   }
   return;
}
void on_AreYouSure_cancel_clicked (GtkButton * button, gpointer user_data)
{  HIDE(AreYouSure);
   ON(main);
   return;
}


/*------------------------------ determine which distributors will need to be "kicked" */
void find_distributors_to_kick(void)
{  int i;
   for (i=0; i < max_study; i++)
   {  if (STUDY.qc_status == 0 && STR_EQ(STUDY.modality,Study[i].modality) &&
          STR_EQ(STUDY.study_id,Study[i].study_id))
      {  dbg("match");
         for (dist = 0; dist < MAX_DISTRIBUTOR; dist++)
         {  if (!DIST) STR(DIST,Study[i].server);
            if (STR_EQ(DIST,Study[i].server)) break;
         }
         dbg("eo for loop");
         if (dist == MAX_DISTRIBUTOR) dbg("distributor overflow");
      }
   }
}
/*---------------------------------------------------------------------------------------
 * "kick" distributors, i.e. send a signal to the resend_qc process on distributors which
 * now have qc actions to take, so that if they are sleeping, they will wake up.  The
 * resend_qc programs don't sleep very long, so we don't get finicky about the kick; one
 * try and let it go.  We are more interested in keeping the client moving.
 *-------------------------------------------------------------------------------------*/
void kick_distributors(void)
{  int pid;
/*--------------------------------- process all distributors which need to be "kicked" */
   for (dist=0; DIST; dist++)
/*--------------------------- Case where this is the distributor, send signal directly */
   {  if (STR_EQ(DIST,distributor_name)) 
      {  GetIniInt("[MAIN]", "ResendPID", 0, &pid, inipfn);
         if (pid) kill(pid, SIGUSR1);
      }
/*--------------------------------------------- not this distributor, use alert server */
      else alertSendSignal("SIGUSR1", "resend_qc", DIST);
      FREE(DIST)
   }
}


/****************************************************************************************
 * Info dialog: Used to display system update messages at login.
 ***************************************************************************************/
gboolean on_info_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data)
{  mess = max_message;
   init_info();
   init_main();
   SHOW(main);
   return;
}
void on_info_clear_clicked (GtkButton * button, gpointer user_data)
{  CLEAR(sql_query);
   sprintf(sql_query, "UPDATE login SET last_message = %d WHERE dba = %d", MESSAGE.dba,
      Login.dba);
   mysql_query(conn,sql_query);
   mess++;
   if(init_info())
   {  init_main();
      SHOW(main);
   }
   return;
}
void on_info_save_clicked (GtkButton * button, gpointer user_data)
{  OFF(info_clear);
   mess++;
   if(init_info())
   {  init_main();
      SHOW(main);
   }
   return;
}

/*---------------------------------------------------------------------------------------
 * displays info dialog with MESSAGE in it and returns 0. If mess is beyond end of msgs,
 * clears the message struct, hides the info dialog, and returns 1.
 * Also estimates size of dialog box and adjusts it.
 *-------------------------------------------------------------------------------------*/
int init_info(void)
{  char * mm = NULL, *p;
   int num_lines = 0, max_line_length = 0;
   if (mess < max_message)
   {  STR(mm,MESSAGE.text);
      p = strtok(mm,"\n");
      while (p)
      {  num_lines++;
         if (strlen(p) > max_line_length) max_line_length = strlen(p);
         p = strtok(NULL,"\n");
      }
      FREE(mm);
      gtk_widget_set_usize (info, max_line_length * 6 + 15, num_lines * 20 + 40);
      CLR_TEXT(info_text);
      SET_TEXT(info_text,MESSAGE.text);
      SHOW(info);
      return (0);
   }
   if (Mesage)
   {  for (mess = 0; mess < max_message; mess++) FREE(MESSAGE.text);
      FREE(Mesage);
   }
   max_message = 0;
   HIDE(info);
   return (1);
}
      
/****************************************************************************************
 * Internal routines
 ***************************************************************************************/

/*---------------------------------------------------------------------------------------
 * create and initialize main window
 *-------------------------------------------------------------------------------------*/
void init_main(void)
{  char * t[] = {"facility", "qc_group", "department", "station"};
   char * m[] = {"All", "CR", "CT", "DF", "DR", "DS", "MR", "NM", "OT", "PT", "RF", "RG", "SC", "US", "XA", "MG" }; 
   int i, max_i = sizeof(m) / sizeof(char *);
   GList * list = NULL;
   char flag = 0, sflag = 0;
/*-------------------------------------------- Populate QC group filter Combo box list */
   CLEAR(sql_query);
   strcpy(sql_query,"SELECT * FROM station "
                "WHERE (begin_process = 0 OR begin_process < NOW()) "
                "AND (end_process = 0 OR end_process > NOW()) ");
   if (Login.top_level) 
   {  sprintf(em,"AND %s = \"%s\" ", t[Login.top_level -1], Login.top);
      strcat(sql_query,em); 
   }
   strcat(sql_query,"ORDER BY dba");
   
   if (mysql_query(conn,sql_query)) {db_error("station query"); return;}
   res = mysql_store_result(conn);
   if (!res) {db_error("station store result"); return;}

/*-------------------- Super users get special Unknown station item in QC group filter */
   if (Login.super_user)
   {  list = g_list_append(list,UNKNOWN_STATION);
      if (STR_EQ(Login.qc_entry,UNKNOWN_STATION)) flag = 1;
   }

   if (max_st = mysql_num_rows(res))                        /* Got results for this login */
   {  Station = malloc(max_st * sizeof(struct _station_));  /* Allocate station buffer */
      memset(Station,0,max_st * sizeof(struct _station_));
      for (st = 0; st < max_st; st++)                       /* Load results into buffer */
      {  row = mysql_fetch_row(res);
         STATION.dba = atoi(row[0]);
         STR(STATION.facility,row[1]);
         STR(STATION.group,row[2]);
         STR(STATION.department,row[3]);
         STR(STATION.station,row[4]);
         STR(STATION.description,row[5]);
         STR(STATION.dicom_tag,row[6]);
         STATION.id_by = 0;
         if (STR_EQ(row[9],"station"))   STATION.id_by = 1;
         if (STR_EQ(row[9],"dicom_tag")) STATION.id_by = 2;
         if (strlen(row[10])) STR(STATION.d_station,row[10]);
/*--------------------------------------- Add desc to combo list, if it is not blank */
         if (strlen(trim(STATION.description)))
            list = g_list_append(list,STATION.description);  
         if (strlen(STATION.station)) sflag = 1;                /* Found a station */
         if (STR_EQ(STATION.description,Login.qc_entry)) flag = 1; /* Found default station */
      }
   }
   mysql_free_result(res);
   if (!sflag) { message("Error!", "No stations set up for this user"); return; }
   Combo_qc_group = lookup_widget(main,"Combo_qc_group");
   gtk_combo_set_popdown_strings(GTK_COMBO(Combo_qc_group), list);
   gtk_combo_set_value_in_list(GTK_COMBO(Combo_qc_group), TRUE, FALSE); 
   if (flag) gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(Combo_qc_group)->entry), Login.qc_entry);
   else gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(Combo_qc_group)->entry), Station[0].description);
   g_list_free(list);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO(Combo_qc_group)->entry), "changed", 
        GTK_SIGNAL_FUNC (on_Combo_qc_group_changed), NULL);

/*---------------------------------------------------- Populate the modality combo box */
   list = NULL;
   for (i = 0; i < max_i; i++) list = g_list_append(list,m[i]);
   Combo_modality = lookup_widget(main,"Combo_modality");
   gtk_combo_set_popdown_strings(GTK_COMBO(Combo_modality), list);
   gtk_combo_set_value_in_list(GTK_COMBO(Combo_modality), TRUE, FALSE);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(Combo_modality)->entry), m[0]);
   g_list_free(list);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO(Combo_modality)->entry), "changed", 
        GTK_SIGNAL_FUNC (on_Combo_modality_changed), NULL);
/*---------------------------------------------------------------- Populate study list */
   Clist_qc = lookup_widget(main,"Clist_qc");
   populate_qc_list();
   gtk_widget_set_usize(main,700,640);
   return;
}

/*---------------------------------------------------------------------------------------
 * Populate Clist_qc, on the main window, which contains the list of studies which need
 * to be qc'd for the selected qc group and modality.  
 *-------------------------------------------------------------------------------------*/
void populate_qc_list(void)
{  int i, j, bst, flag = 0, inst_flag = 0, stat_flag = 0;
   char * p, * qc_group_text, * modality_text;
   char a[128];
   int sa = strlen(a);
   char inst[2048] = "institution_name IN (", stan[2048] = "station_name IN (";
/*----------------------------- If previous entries are in the list, clear them first. */
   on_Button_cancel_clicked(NULL, NULL);
   gtk_clist_clear(GTK_CLIST(Clist_qc));
   free_study_struct();
/*-------------------------- Fetch values from Combo boxes that filter qc studies list */
   p = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(Combo_qc_group)->entry));
   if (p) STR(qc_group_text,p);
   else STR(qc_group_text,Station[0].description);
/*------------------ If this is changed from login, update login entry with this value */
   if (strcmp(Login.qc_entry,qc_group_text))
   {  FREE(Login.qc_entry)
      STR(Login.qc_entry,qc_group_text);
      CLEAR(sql_query);
      sprintf(sql_query,"UPDATE login SET qc_entry = \"%s\" WHERE dba = %d", Login.qc_entry, Login.dba);
      mysql_query(conn,sql_query);
   }
      
   p = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(Combo_modality)->entry));
   if (p) STR(modality_text,p);
   else STR(modality_text,"All");
/*---------------------------------- Find row in Station struct that matches selection */
   for (st=0; st < max_st; st++) if (STR_EQ(STATION.description,qc_group_text)) break;
   bst = st;

/*----------------------------------------------------------------------- Create query */
   CLEAR(sql_query);
/********************************************************** query for Unknown stations */
   if (STR_EQ(qc_group_text,UNKNOWN_STATION))
   {  strcpy(sql_query,"SELECT * FROM study LEFT JOIN station "
                       "ON (id_by = \"station\" AND station_name = station) OR "
                       "(id_by = \"dicom_tag\" AND institution_name = dicom_tag) "
                       "WHERE station IS NULL AND dicom_tag IS NULL "
                       "AND qc_status = 0");
      flag = 1;
   }
   else
/*********************************************************** Station table lines query */
   {  strcpy(sql_query,"SELECT * FROM study WHERE qc_status = 0");       /* base query */
/********************************************************************* qc_area filters */
      if (strlen(Station[bst].facility))
      {  strcat(sql_query," AND (");                             /* if qc_area filters */
/*------------------------------ Step through Station struct, adding tags to the lists */
         for (st=bst; st < max_st; st++)
         {  switch (STATION.id_by)
            {  case 1:                                     /* id by station name */
                  if (strlen(STATION.station))
                  {  sprintf(em,"\"%s\", ", STATION.station);
                     strcat(stan,em);
                     flag = stat_flag = 1;
                  }
                  break;
               case 2:                                     /* id by institution name */
                  if (strlen(STATION.dicom_tag))               
                  {  sprintf(em,"\"%s\", ", STATION.dicom_tag);
                     strcat(inst,em);
                     flag = inst_flag = 1;   
                  }
                  break;
               default:
                  break;
            }
/*---------------------------------------------------------------------------------------
 * We keep looping, and include all Station lines until one with at least the same number 
 * of blank entries as the selected (first) one in the facility-group-department-station 
 * hierarchy. This is done only after the first line. 
 *-------------------------------------------------------------------------------------*/
            if (st == bst) continue;
            if (strlen(Station[bst].station)) break;
            if (strlen(Station[bst].department) && !strlen(STATION.station)) break;
            if (strlen(Station[bst].group) && !strlen(STATION.department)) break;
            if (strlen(Station[bst].facility) && !strlen(STATION.group)) break;
         }
/*------------------------- If flag not set, there are no stations, so result is blank */
         if (!flag) {stat(Mstatus,"No stations selected"); return;}
/*---------------------------------------- add selected station names to query, if any */
         if (stat_flag)
         {  LCHR(stan) = 0; LCHR(stan) = 0; strcat(stan,")"); /* replace last ", " with ")" */
            strcat(sql_query,stan);
         }
         if (stat_flag && inst_flag) strcat(sql_query," OR ");     /* if both present */
/*-------------------------------------- add select institution names to query, if any */
         if (inst_flag)
         {  LCHR(inst) = 0; LCHR(inst) = 0; strcat(inst,")"); /* replace last ", " with ")" */
            strcat(sql_query,inst);
         }
         strcat(sql_query,")");
      }
   }
/********************************************************************* Modality filter */
   if (strcasecmp(modality_text,"All"))
   {  sprintf(em," AND modality = \"%s\"", modality_text);
      strcat(sql_query,em);
   }
/*-------------------------------------------------------------- now order the studies */
   strcat(sql_query," ORDER BY study_done DESC");
/*----------------------------------------------------------------------- pull studies */ 

   if (mysql_query(conn,sql_query)) {db_error("study query"); return;}
   res = mysql_store_result(conn);
   if (!res) {db_error("study store result"); return;}
   if (!mysql_num_rows(res))                                       /* No studies to qc */
   {  mysql_free_result(res); 
      stat(Mstatus,"No studies to qc for this selection");
      return;
   }    
/*---------------------------------------------------------- Load study into structure */

   max_study = mysql_num_rows(res);
   Study = malloc(max_study * sizeof(struct _study_));
   memset(Study,0,max_study * sizeof(struct _study_));
   for (study = 0; study < max_study; study++)
   {  row = mysql_fetch_row(res);
      STUDY.dba = atoi(row[0]);
      STUDY.qc_status = atoi(row[1]);
      STR(STUDY.server,row[2]);
      STR(STUDY.study_id,row[3]);
      STR(STUDY.source,row[4]);
      STR(STUDY.modality,row[5]);
      STR(STUDY.institution_name,row[7]); 
/*------ If no station name, look up in .ini, with institution_name as key and default */
      if (strlen(row[6])) STR(STUDY.station_name,row[6]);
      else 
      {  GetIniString("[STATIONS]", STUDY.institution_name, STUDY.institution_name, a, sa, inipfn);
         STR(STUDY.station_name,a);
      }
      STR(STUDY.pid,row[8]);
      STR(STUDY.dnumber,row[9]);
      STR(STUDY.pname,row[10]);
      STR(STUDY.dob,row[11]);
      STR(STUDY.sex,row[12]);
      STR(STUDY.visit,row[13]);
      STR(STUDY.accn,row[14]);
      STR(STUDY.description,row[15]);
      STR(STUDY.exam_code,row[16]);
      STR(STUDY.body_part,row[17]);
      STR(STUDY.scheduled,row[18]);
      STR(STUDY.study_done,row[19]);
      STR(STUDY.report_done,row[20]);
      STR(STUDY.first_arrive,row[21]);
      STUDY.errors = strtoul(row[22], (char **)NULL, 10);
      STR(STUDY.qc_done,row[23]);
      STR(STUDY.user_id,row[24]);
      STR(STUDY.dest_code,row[25]);
      STUDY.row = -1;
   }
   mysql_free_result(res);

/*------------------------------ Pass struct and display line which are not duplicates */
   for (study = 0; study < max_study; study++)
/*---------------- If study is duplicate of previous entry in table, do not display it */
   {  for (i=flag=0;i<study;i++)
      {  if(STR_EQ(Study[i].study_id,STUDY.study_id) &&
            STR_EQ(Study[i].modality,STUDY.modality)) {flag=1;break;}
      }
      if (flag) continue;
/*--------------------------------------------------------------- Load and display row */
      STR(col[0],STUDY.pname);
      STR(col[1],STUDY.pid);
      STR(col[2],STUDY.accn);
      STR(col[3],STUDY.modality);
      STR(col[4],STUDY.study_done);
      STUDY.row = gtk_clist_append(GTK_CLIST(Clist_qc), col);  /* Load the row */
      for (j=0;j<5;j++) free(col[j]);
   }
   stat(Mstatus,"Select study from list to QC");
   return;
}
   
void free_study_struct(void)
{  if (!Study) return;
   for (study = 0; study < max_study; study++)
   {  free(STUDY.server);  
      free(STUDY.study_id);
      free(STUDY.source);
      free(STUDY.modality);
      free(STUDY.station_name);
      free(STUDY.institution_name);
      free(STUDY.pid);
      free(STUDY.dnumber);
      free(STUDY.pname);
      free(STUDY.dob);
      free(STUDY.sex);
      free(STUDY.visit);
      free(STUDY.accn);
      free(STUDY.description);
      free(STUDY.exam_code);
      free(STUDY.body_part);
      free(STUDY.scheduled);
      free(STUDY.study_done);
      free(STUDY.report_done);
      free(STUDY.first_arrive);
      free(STUDY.qc_done);
      free(STUDY.user_id);
      free(STUDY.dest_code);
   }
   free(Study);
   Study = NULL;
   return;
}

/*---------------------------------------------------------------------------------------
 * Update images after determination of resend destination
 *-------------------------------------------------------------------------------------*/
void update_images(void)
{  time_t now = time(NULL); 
   char ct[16];

   find_distributors_to_kick();
   
   cftime(ct, "%Y%m%d%H%M%S", &now);
/*------------------------------------------------- Create query to update qc database */
   CLEAR(sql_query);
   sprintf(sql_query, "UPDATE study SET qc_status = 1, pid = \"%07d\", dnumber = \"%s\", "
                      "pname = \"%s\", dob = %s, sex = \"%s\", visit = \"%s\", "
                      "accn = \"%07d\", description = \"%s\", exam_code = \"%s\", "
                      "scheduled = %s00, qc_done = %s, user_id = \"%s\", "
                      "dest_code = \"%s\" WHERE qc_status = 0 AND "
                      "modality = \"%s\" AND study_id = \"%s\"",
                      AccnQuery.mpi, AccnQuery.dnumber, AccnQuery.pname, AccnQuery.dob,
                      AccnQuery.sex, AccnQuery.visit, AccnQuery.accn, 
                      AccnQuery.exam_desc, AccnQuery.exam, AccnQuery.start_time, ct,
                      Login.user_id, RESEND.dest_code, STUDY.modality, STUDY.study_id);
   if (mysql_query(conn,sql_query)) 
   {  ON(main);
      db_error("query");
      return;
   }
/*--------------------------------------------------------------------- QC log message */
   CLEAR(log_msg);
   sprintf(log_msg, "Study qc'd by %s (%s) - Updated and resent to %s\n", Login.name, Login.user_id, RESEND.dest_code);
   sprintf(em,"   pid........ %s", STUDY.pid); strcat(log_msg,em);
   if (atoi(STUDY.pid) != AccnQuery.mpi) {sprintf(em," => %d", AccnQuery.mpi); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   dnumber.... %s", STUDY.dnumber); strcat(log_msg,em);
   if (strcmp(STUDY.dnumber,AccnQuery.dnumber)) {sprintf(em," => %s", AccnQuery.dnumber); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   accn....... %s", STUDY.accn); strcat(log_msg,em);
   if (atoi(STUDY.accn) != AccnQuery.accn) {sprintf(em," => %d", AccnQuery.accn); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   name....... %s", STUDY.pname); strcat(log_msg,em);
   if (strcmp(STUDY.pname,AccnQuery.pname)) {sprintf(em," => %s", AccnQuery.pname); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   DOB........ %s", STUDY.dob); strcat(log_msg,em);
   if (num_cmp(STUDY.dob,AccnQuery.dob)) {sprintf(em," => %s", AccnQuery.dob); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   sex........ %s", STUDY.sex); strcat(log_msg,em);
   if (strcmp(STUDY.sex,AccnQuery.sex)) {sprintf(em," => %s", AccnQuery.sex); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   visit...... %s", STUDY.visit); strcat(log_msg,em);
   if (strcmp(STUDY.visit,AccnQuery.visit)) {sprintf(em," => %s", AccnQuery.visit); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   source..... %s\n", STUDY.source); strcat(log_msg,em);
   sprintf(em,"   modality... %s\n", STUDY.modality); strcat(log_msg,em);
   sprintf(em,"   exam code.. %s", STUDY.exam_code); strcat(log_msg,em);
   if (strcmp(STUDY.exam_code,AccnQuery.exam)) {sprintf(em," => %s", AccnQuery.exam); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   description %s", STUDY.description); strcat(log_msg,em);
   if (strcmp(STUDY.description,AccnQuery.exam_desc)) {sprintf(em," => %s", AccnQuery.exam_desc); strcat(log_msg,em); }
   strcat(log_msg,"\n");
   sprintf(em,"   scheduled.. %s", STUDY.scheduled); strcat(log_msg,em);
   strcat(AccnQuery.start_time,"00");
   if(num_cmp(STUDY.scheduled,AccnQuery.start_time)){sprintf(em," => %s",AccnQuery.start_time);strcat(log_msg,em);}
   strcat(log_msg,"\n");
   sprintf(em,"   study done. %s\n", STUDY.study_done); strcat(log_msg,em);
   sprintf(em,"   station.... %s\n", STUDY.station_name); strcat(log_msg,em);
   sprintf(em,"   institution %s\n", STUDY.institution_name); strcat(log_msg,em);
   sprintf(em,"   study UID.. %s\n", STUDY.study_id); strcat(log_msg,em);

   elapsed_time(STUDY.first_arrive);
   sprintf(em,"   arrived.... %s - time to qc %s\n", STUDY.first_arrive, elapsed_string);
   strcat(log_msg,em);
   
   logMsgLevel("EQChold",log_msg,1);

   kick_distributors();
   
   ON(main);
   populate_qc_list();
   return;
}
/*--------------------------------------------------------- Read .ini file information */
int read_ini(void)
{  char a[1024], main[] = "[MAIN]";
   int sa = sizeof(a);
   
   if (access(inipfn, F_OK)) { printf("%s file not found\n", inipfn); return(1); }
   
   if (!GetIniString(main, "Debug", "stdout", a, sa, inipfn))
   {  PrtIniErrorString(stdout, "ini file error");
      return(1);
   }
   debug_fp = debug_on(a);

   GetIniString(main, "DBHost", "localhost", a, sa, inipfn);
   STR(dbhost,a);

   gethostname(a, sa);
   GetIniString(main, "DistributorName", a, a, sa, inipfn);
   STR(distributor_name,a);
/*---------------------------------------------- Should windows be centered on screen? */ 
   GetIniString(main, "CenterWindows", "YES", a, sa, inipfn);
   if (STR_EQ(a,"YES") || STR_EQ(a,"ON")) center_windows = 1; else center_windows = 0;

/*-------------------------------------------------------- Link to use for MPI queries */
   GetIniInt("[MPIQuery]", "Link", query_link, &query_link, inipfn);
/*------------------------------- Number of days before DOS to cut off Patient Queries */
   GetIniInt("[MPIQuery]", "DaysBeforeDOS", days_before_DOS, &days_before_DOS, inipfn);

/*------------------------------------------------------------- Logging initialization */
   if (logInitialize(inipfn)) dbg(GetLogErrorString());
   
   return 0;
}

/****************************************************************************************
 * Miscl internal routines
 ***************************************************************************************/

/*------------------------------------------------ removes trailing spaces from string */
char * trim(char * str) 
{  while (strlen(str) && (LCHR(str) == ' ')) LCHR(str) = 0;
   return str;
}

/*------------------------------- Puts message in status bar (avoid stacking messages) */
void stat(GtkWidget * bar, char * text)
{  gtk_statusbar_pop(GTK_STATUSBAR(bar),1);
   gtk_statusbar_push(GTK_STATUSBAR(bar),1,text);
}

/*----------------------------------------------- Standard database error message box */
void db_error(char * cmd)
{  sprintf(em,db_err_msg, cmd, mysql_error(conn)); 
   message("Error!",em);
}

/*--------------------------------------------------------------------------------------
 * returns 0 if the imput represents a potentially valid id, i.e., it is numeric and at
 * least 7 digits long.  Also trims it.  Returns non-zero if non-valid.
 *------------------------------------------------------------------------------------*/
int not_valid_id(char * id)
{  char *p;
   trim(id);
   if (strlen(id) < 7) return -1;
   for (p=id; *p; p++) if (!isdigit(*p)) return 1;
   return 0;
}

/*---------------------------------------------------------------------------------------
 * Adjusts a mysql format (YYYY-MM-DD HH:MM:SS) date backward by days and returns an HL7
 * format date (YYYYMMDD) string. If date is zero, uses current date and adds 2 to days.
 *-------------------------------------------------------------------------------------*/
char * adjust_date(char * date, int days)
{  time_t hack;
   struct tm in_tm, * out_tm;
   char * p, wd[24];
   static char out_date[10];
   if (strcmp(date,"0000-00-00 00:00:00"))
   {  strcpy(wd,date);                                /* Valid date input */
      memset(&in_tm,0,sizeof(struct tm));
      in_tm.tm_year = atoi(strtok(wd,"-")) - 1900;
      in_tm.tm_mon = atoi(strtok(NULL,"-")) - 1;
      in_tm.tm_mday = atoi(strtok(NULL," "));
      hack = mktime(&in_tm) - (days * 86400);
   }
   else hack = time(NULL) - ((days + 2) * 86400);     /* Zero date input */

   out_tm = localtime(&hack);                         /* This is cuttoff date */

   sprintf(out_date,"%04d%02d%02d", out_tm->tm_year + 1900, out_tm->tm_mon + 1, out_tm->tm_mday);
   return (out_date);
}

/*---------------------------------------------------------------------------------------
 * Compares the numeric digits in the two arguments, ignoring any non-numeric characters,
 * and returning 1 if they are not equal & 0 if they are. Signs (+&-) are treated as non-
 * numeric characters
 *-------------------------------------------------------------------------------------*/
int num_cmp(char * a, char * b)
{  char i[128], j[128];
   char * p;
   CLEAR(i);
   CLEAR(j);
   for (p=a;*p;p++) if (isdigit(*p)) i[strlen(i)] = *p;
   for (p=b;*p;p++) if (isdigit(*p)) j[strlen(j)] = *p;
   if (strcmp(i,j)) return 1;
   else return 0;
}

/*---------------------------------------------------------------------------------------
 * Compares a mysql format date-time stamp with the current time. Puts the elapsed time
 * in elapsed_seconds, and a character version in elapsed_string.  Returns zero if OK,
 * non-zero on error.
 *-------------------------------------------------------------------------------------*/
int elapsed_time(char * start_time_string)
{  time_t a;
   int i;
   struct tm tm;
/*------------------------------- Pull date out of mysql format; adjust year and month */
   sscanf(start_time_string,"%4u%*c%2u%*c%2u%*c%2u%*c%2u%*c%2u", &tm.tm_year, &tm.tm_mon,
      &tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
   tm.tm_year -= 1900; tm.tm_mon--; tm.tm_isdst = -1;
/*----------------------------------------------------- compute elapsed time until now */
   a = mktime(&tm);
   if (a == -1) 
   {  CLEAR(elapsed_string);
      elapsed_seconds = 0;
      return 1;
   }
   a = elapsed_seconds = difftime(time(NULL), a);
/*------------------------------------------------------------- character elapsed time */
   CLEAR(elapsed_string);
   i = a / (60 * 60);
   if (i) 
   {  sprintf(elapsed_string,"%d:", i);
      a = a - (i * 60 * 60);
   }
   i = a / 60;
   sprintf(em,"%02d:", i); strcat(elapsed_string, em);
   a = a - (i * 60);
   sprintf(em,"%02d", a); strcat(elapsed_string, em);
   return 0;
}
