/***********************************************************************
 * PACSO_CTRL.C
 * PACS out link control program.                          R Moulton
 * ---------------------------------------------------------------------
 * 08/03 Original write.
 * 07/04 Rewrite to use .ini file 
 *----------------------------------------------------------------------
 * gcc -g PACSO_ctrl.c dbg.c ini.c -o PACSO_ctrl
 **********************************************************************/

/* #define _VMS_MODE_ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <dirent.h>

#ifdef _VMS_MODE_

#include <unixio.h>
#include <descrip.h>

#include "[source.common]ini.h"
#include "[source.common]dbg.h"

#else

#include "ini.h"
#include "dbg.h"

#endif

/*--------------------------------------------------------------------------------------- Macros */
#define MAKE_STR(A,B) { if (A=malloc(strlen(B)+3)) strcpy(A,B); }
#define FREE_STR(A) if (A) { free(A); A=NULL; }
#define NEW_STR(A,B) { if (A) free(A); if (A=malloc(strlen(B)+3)) strcpy(A,B); }
#define STR_EQ(A,B) !strcasecmp(A,B)

/*----------------------------------------------------------------------
 * Function declarations
 * -------------------------------------------------------------------*/
void read_ini(void);
void clear_screen(void);
void link_function(char * pfn, char * command);  
void view_status_file(int j);
void view_dm_file(void);
void view_pending(int j);
/*----------------------------------------------------------------------
 * Parameters
 * -------------------------------------------------------------------*/
#define PAGE_LENGTH            20                         /* # of detail lines on screen page */

#ifdef _VMS_MODE_

#define BIND "RADA:[RADIOLOGY.PACSO]"
static char * pacso_status_pfn = BIND "PACSO_STATUS.DAT;1";
static char * pacso_dm_status_pfn = BIND "PACSO_DM_STATUS.DAT;1";
#define FILESUFFIX ";1"

#else

#define BIND "./"
static char * pacso_status_pfn = BIND "pacso_status.dat";
static char * pacso_dm_status_pfn = BIND "pacso_dm_status.dat";
#define FILESUFFIX ""

#endif

struct _pfns_ {
   char * sname;
   char * desc;
   char * status;
   char * type;
   char * status_pfn;
   char * control_pfn;
   char * high;
   char * low;
   char * A08;
};
static struct _pfns_ pfns[20];
static int max_pfn = 0; 

/*-------------------------------------------------------------- Menu */
#define O_NA      0

#define O_V       1
#define O_B       2
#define O_D       3
#define O_P       4
#define O_QUIT    5
#define O_VIEW    6
#define O_DM      7

struct _menu_item_ { int code;      /* Entry routine code */
                     char * entry;  /* Entry to select this menu item */
                     char * dline;  /* Menu line to display */
                   };
static const struct _menu_item_ menu[] = {
   { O_NA,    0,  "PACS Outbound Link status & control Utility\n\n" },
   { O_VIEW, "Vv", "  <V>IEW   - View IDX Feed Link status\n" },
   { O_DM,   "Mm", "  <M>onitor - View Directory Monitor status\n" },
   { O_NA,    0,  "\nServers: (follow command letter with server #)\n" },
   { O_NA,    0,  NULL },
   { O_V,    "Ss", "  <S#>Status  - View server status\n" },
   { O_P,    "Pp", "  <P#>Pending - # Transaction pending for server\n" },
   { O_B,    "Bb", "  <B#>OUNCE   - Bounce Link server\n" },
   { O_D,    "Dd", "  <D#>OWN     - Gracefully bring Link server down\n\n" },
   { O_QUIT, "Qq", "  <Q>uit   - Exit this program\n\n" },
   { O_NA,    0,  "Enter command: " },
};
static unsigned short int max_menu_item = sizeof(menu) / sizeof(struct _menu_item_);

static char buffer[256];
/***********************************************************************
 ************************** Main Function ******************************
 **********************************************************************/
int main (int argc, char **argv)
{  int i, function, j, flag, k;                        
/*------------------------------------------------------------- Setup */
   read_ini();
/*---------------------------------------------- Process command menu */
   while (1)
   {   
/*------------------------------------------------- Display main menu */
      clear_screen();
      for (i = 0; i < max_menu_item; i++) 
      {  if (menu[i].dline) printf("%s", menu[i].dline);
         else for (k = 1; k < max_pfn; k++) printf("   %d - %12s %6s %s\n", k, pfns[k].desc, pfns[k].type, pfns[k].status);
      } 
/*--------------------------------------- Get user entry to main menu */
      if (!gets(buffer)) { clear_screen(); return 0; }
/*---------------------------------------- Match keyword to menu list */
      function = O_NA;
      for (i = 0; i < max_menu_item; i++)
      {  if (menu[i].code == O_NA) continue;
         if (buffer[0] != menu[i].entry[0] && buffer[0] != menu[i].entry[1]) continue;
         function = menu[i].code;
         break;
      }
      if (function == O_NA)
      {  printf("Invalid - \"Enter\" to continue.");
         gets(buffer);
         continue;
      }
/*----------------------------------------------------- process entry */
      if (buffer[1] > '0' && buffer[1] <= ('0' + max_pfn)) 
      { flag = 1; j = buffer[1] - '0'; }
      else flag = 0;
      switch (function)
      {  case O_VIEW:
            view_status_file(0);
            break;
         case O_DM:
            view_dm_file();
            break;
         case O_V:
            if (flag) view_status_file(j);
            else 
            {  printf("Invalid - \"Enter\" to continue.");
               gets(buffer);
            }
            break;
         case O_P:
            if (flag) view_pending(j);
            else 
            {  printf("Invalid - \"Enter\" to continue.");
               gets(buffer);
            }
            break;
         case O_B:
            if (flag) link_function(pfns[j].control_pfn, "BOUNCE");
            else 
            {  printf("Invalid - \"Enter\" to continue.");
               gets(buffer);
            }
            break;
         case O_D:
            if (flag) link_function(pfns[j].control_pfn, "EXIT");
            else 
            {  printf("Invalid - \"Enter\" to continue.");
               gets(buffer);
            }
            break;
         case O_QUIT:
            clear_screen();
            return 0;
         default: 
            printf("Not implemented - \"Enter\" to continue.");
            gets(buffer);
      }
      continue;
   }
}

/***********************************************************************
 * Pass command code in argument to BSPOE link program via control file
 * ********************************************************************/
void link_function(char * pfn, char * command)
{  FILE * fp;
   char * p;
/*--------------------------------- Check for command already pending */
   fp = fopen(pfn, "r");
   if (fp)
   {  fgets(buffer, sizeof(buffer), fp);
      if (feof(fp)) 
      {  fclose(fp); 
         remove(pfn); 
      }
      else  
      {  fclose(fp);
         printf("%s command already pending - \"Enter\" to continue\n", buffer);
         gets(buffer);
         return;
      }
   }
/*------------------------------------------------ Put our command in */
   fp = fopen(pfn, "w");
   if (!fp)
   {  printf("Unable to create control file: %s - \"Enter\" to continue\n", pfn);
      gets(buffer);
      return;
   }
   fprintf(fp, "%s\n", command);
   fclose(fp);
   printf("%s command sent to server - \"Enter\" to continue\n", command);
   gets(buffer);
   return;
}

void view_status_file(int j)
{  FILE * fp;
   char * p;
   time_t now;
   char * pfn = pfns[j].status_pfn;
/*----------------------------------------------------------- Open file */
   fp = fopen(pfn, "r");
   if (!fp)
   {  printf("No status file - %s \"Enter\" to continue\n", pfn);
      gets(buffer);
      return;
   }
   clear_screen();
   now = time(NULL);
   printf("%s, type: %s, Status %s\n", pfns[j].desc, pfns[j].type, pfns[j].status);
   printf("Current system time: %s\n", asctime(localtime(&now)));
   while (1)
   {  fgets(buffer, sizeof(buffer), fp);
      if (feof(fp)) break;
      printf("%s", buffer);
   }
   fclose(fp);
   printf("\n\n\"Enter\" to continue\n");
   gets(buffer);
   return;
}

void view_dm_file(void)
{  FILE * fp;
   char * p;
   time_t now;
   char * pfn = pacso_dm_status_pfn;
/*----------------------------------------------------------- Open file */
   fp = fopen(pfn, "r");
   if (!fp)
   {  printf("No dm status file - %s \"Enter\" to continue\n", pfn);
      gets(buffer);
      return;
   }
   clear_screen();
   now = time(NULL);
   printf("Directory Monitor status:\n");
   printf("Current system time: %s\n", asctime(localtime(&now)));
   while (1)
   {  fgets(buffer, sizeof(buffer), fp);
      if (feof(fp)) break;
      printf("%s", buffer);
   }
   fclose(fp);
   printf("\n\n\"Enter\" to continue\n");
   gets(buffer);
   return;
}

void view_pending(int j)
{  DIR * d;
   struct dirent * e;
   int i = 0, k = 0, a = 0;
   if (d = opendir(pfns[j].high))
   {  while (e = readdir(d)) if (strstr(e->d_name,".DAT")) i++;
      closedir(d);
   }
   if (d = opendir(pfns[j].low))
   {  while (e = readdir(d)) if (strstr(e->d_name,".DAT")) k++;
      closedir(d);
   }
   if (d = opendir(pfns[j].A08))
   {  while (e = readdir(d)) if (strstr(e->d_name,".DAT")) a++;
      closedir(d);
   }
   printf("%s, type: %s, Status %s\n", pfns[j].desc, pfns[j].type, pfns[j].status);
   if (pfns[j].high) printf("\n\n%4d files waiting in %s\n", i, pfns[j].high);
   if (pfns[j].low)  printf("%4d files waiting in %s\n",     k, pfns[j].low); 
   if (pfns[j].A08)  printf("%4d files waiting in %s\n\n\n", a, pfns[j].A08); 
   printf("\n\n\"Enter\" to continue\n");
   gets(buffer);
   return;
}
   
   
void clear_screen()
{  printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
   printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
}

/******************************************************************************
 * Load info on feeds from pacso.ini
 *****************************************************************************/
void read_ini(void)
{  char a[2048], pfn[] = BIND "pacso.ini", * p, pt[2];
   int sa = sizeof(a), i;

   MAKE_STR(pfns[0].desc,"IDX PACSO Feed")  /* Main Feed items */
   MAKE_STR(pfns[0].type,"Feed Link")
   MAKE_STR(pfns[0].status,"ON")
   MAKE_STR(pfns[0].status_pfn,pacso_status_pfn)
   max_pfn = 1;

   if (!GetIniSectionNames(a, sa, pfn))  /* Load section names */
   {  PrtIniErrorString(stdout, "GetIniSectionNames");
      exit(1);
   }
/*--------------------------------------- Pull FEED_ sections from .ini file */
   for (p = strtok(a,"[]"); p; p = strtok(NULL,"[]"))
   {  if (strstr(p,"FEED_")) 
      {  MAKE_STR(pfns[max_pfn].sname,p)
         max_pfn++;
      }
   }
   if (max_pfn == 1) { printf("No Feed sections in pacso.ini\n"); exit(1); }

/*----------------------------------- Now load other stuff from each section */
   for (i = 1; i < max_pfn; i++)
   {  GetIniString(pfns[i].sname,"Description",pfns[i].sname,a,sa,pfn);
      MAKE_STR(pfns[i].desc,a)
      GetIniString(pfns[i].sname,"ProcessType", "S", a, sa, pfn);
      MAKE_STR(pfns[i].type,a)
      pt[0] = a[0]; pt[1] = 0;
      sprintf(a,"%spacso_%s%s_status.dat%s", BIND, pt, pfns[i].sname + 5, FILESUFFIX);
      MAKE_STR(pfns[i].status_pfn,a)
      sprintf(a,"%spacso_%s_control.dat%s", BIND, pfns[i].sname + 5, FILESUFFIX);
      MAKE_STR(pfns[i].control_pfn,a)
      GetIniString(pfns[i].sname,"Status", " ", a, sa, pfn);
      MAKE_STR(pfns[i].status,a)
      GetIniString(pfns[i].sname,"HighMsgDir", "\0", a, sa, pfn);
      if (strlen(a)) MAKE_STR(pfns[i].high,a)
      GetIniString(pfns[i].sname,"LowMsgDir", "\0", a, sa, pfn);
      if (strlen(a)) MAKE_STR(pfns[i].low,a)
      GetIniString(pfns[i].sname,"MsgDir", "\0", a, sa, pfn);
      if (strlen(a)) NEW_STR(pfns[i].high,a)
      GetIniString(pfns[i].sname,"CombineA08s", "\0", a, sa, pfn);
      if (strlen(a)) NEW_STR(pfns[i].A08,a)
   }
}
