/*******************************************************************
 * SentTest.c                                              R Moulton
 * Program examines broadcast rslt file to determine if there is a
 * successful xmit for each image in the image file for each 
 * destination.  Invocation is:
 *
 * SentTest image_file_pfn result_file_pfn number_of_destinations
 *
 * Program exit status:
 *    0 = OK, but xmits are not complete
 *    1 = Error occurred
 *    2 = OK, xmits complete for these images
 *
 ******************************************************************
 *
 * Compile using:  gcc SentTest.c -o SentTest
 *
 *-----------------------------------------------------------------
 * 2-21-2 R Moulton Modified to return OK in cases where rslt file
 *        shows images that are not on file, as long as all images
 *        which are on file are present.
 * 2-22-2 R Moulton Modified to print statistics line
 ******************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

char images_pfn[32] = " ";
char rslt_pfn[32]   = " ";
char images_record[256];
char rslt_record[256];
char * images_image;
char * rslt_image;
char * ptr;
int num_destinations = 0;
int retv, i, total_images, sent_images, image_sent, read_rslt;
FILE * images, * rslt;

int main(int numpars, char * par[])
/*----------------------------------------------- Load parameters */
{  total_images = 0; sent_images = 0; read_rslt = 1;
   if (numpars != 4) { printf("    SentTest: Inv # of params\n"); return (1); }
   strcpy(images_pfn, par[1]);
   strcpy(rslt_pfn, par[2]);
   num_destinations = atoi(par[3]);
   if (num_destinations < 1) { printf("    SentTest: Inv # of destinations\n"); return (1); }

/*---------------------------------------------------- Open files */
   images = fopen(images_pfn, "r");
   if (!images) { printf("    SentTest: images file open error\n"); return (1); }
   rslt = fopen(rslt_pfn, "r");
   if (!rslt) { printf("    SentTest: rslt file open error\n"); return (1); }

/*------------------------------------------- Pass through images */
   while (1)
   {  ptr = fgets(images_record, 256, images);      /* Read image */
      if (!ptr) 
      {  if(total_images == sent_images)
	 { printf("    All %d images previously sent\n", total_images); 
	   return (2); 
	 }
	 else
	 { printf("    %d of %d images previously sent\n", sent_images, total_images); 
	   return (0); 
	 }
      }
      total_images += 1;
/*------------------------- Last token on the line is the image id */
      images_image = strtok(images_record, "/\n");
      while (ptr = strtok(NULL, "/\n")) images_image = ptr;
      if (!images_image) { printf("    SentTest: images file token error\n"); return (1); }
/*-------------------------------------------------------------------
 * We need to match images in image file to image in rslt file once
 * for each destination
 *-----------------------------------------------------------------*/
      image_sent = 1;
      for (i = 0; i < num_destinations; i++)
      {  while (1)
	 {  if (read_rslt)
            {  ptr = fgets(rslt_record, 256, rslt);
	       if (!ptr) 
	       { printf("    %d of %d images previously sent\n", sent_images, total_images); 
	         return (1); 
	       }
/*-------- In this case, image id is after first /, before first ^ */
               if (!strtok(rslt_record, "/")) 
	          { printf("    SentTest: rslt file token error\n"); return (1); }
               if (!(rslt_image = strtok(NULL, "^"))) 
	          { printf("    SentTest: rslt file token error\n"); return (1); } 
            }
            read_rslt = 1;
/*----------------------------------------------------- Must match */
            if (strcmp(rslt_image, images_image) <  0) continue;
            if (strcmp(rslt_image, images_image) == 0) break;
            if (strcmp(rslt_image, images_image) >  0) 
	       { image_sent = 0; read_rslt = 0; break; }
	 }
	 if (!image_sent) break;
      }
      if (image_sent) sent_images += 1;
   }
}
