/**************************************************************************************************
 * dbg.c                                                                                  R Moulton
 * Routines to handle debugging prints.
 *-------------------------------------------------------------------------------------------------
 * gcc -c dbg.c
 *-------------------------------------------------------------------------------------------------
 * 11-3 R Moulton Initial write
 *************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <errno.h>
#include "dbg.h"

/*--------------------------------------------------------------------------------------- Macros */
#define MAKE_STR(A,B) if (A=malloc(strlen(B)+3)) strcpy(A,B);
#define FREE_STR(A) if (A) { free(A); A=NULL; }
#define STR_EQ(A,B) !strcasecmp(A,B)

static char * debug_pfn = NULL;
static FILE * fp = NULL;
/*-------------------------------------------------------------------------------------------------
 * Turns on debugging, sending messages to indicated file.  File can be stdout or stderr, in which
 * case standard FILE * are used.  Otherwise, file is opened, set to line buffering. An initial
 * debugging on message is written.  FILE * is returned in case calling program wants to send some
 * of its own messages there also.  Returns NULL if this can't be accomplished.
 *-----------------------------------------------------------------------------------------------*/
FILE * debug_on(char *pfn)
{  char a[128];
   if (debug_pfn) debug_off();
   if (!pfn || !strlen(pfn))
   {  printf("debug_on: Invalid pfn");
      return NULL;
   }
   if (STR_EQ(pfn,"NULL")) return NULL;
   MAKE_STR(debug_pfn,pfn);
   if (STR_EQ(debug_pfn,"stdout")) fp = stdout;
   else if (STR_EQ(debug_pfn,"stderr")) fp = stderr;
   else 
   {  fp = fopen(debug_pfn,"a");
      if (!fp)
      {  perror("debug_on, fopen");
         FREE_STR(debug_pfn);
         return NULL;
      }
      if (setvbuf(fp, NULL, _IOLBF, 0))
      {  perror("debug_on, setvbuff");
         FREE_STR(debug_pfn)
         fclose(fp);
         fp = NULL;
         return NULL;
      }
   }
   sprintf(a,"Debug on, pfn=%s", debug_pfn);
   dbg(a);
   return fp;
}

void debug_off(void)
{  if (fp) 
   {  dbg("Debug off");
      if (fp != stdout && fp != stderr) fclose(fp); 
      fp = NULL; 
   }
   FREE_STR(debug_pfn)
   return;
}

void dbg(char * msg)
{  char * m = msg, * def = "debug called with NULL msg";
   if (!m) m = def;
   if (fp) fprintf(fp, "%s\n", m); 
   return;
}
void dbg_exit(char * msg, int status) 
{  if (!fp) fp = stderr;
   dbg(msg); 
   exit(status); 
}

void dbge(char * prefix)
{  char * p, * def = "No errno text available";
   if (fp)
   {  if (!(p = strerror(errno))) p = def;
      fprintf(fp,"%s: %s\n", prefix, p);
   }
   return;
}
void dbge_exit(char * msg, int status) 
{  if (!fp) fp = stderr;
   dbge(msg); 
   exit(status); 
}
   
FILE * debug_is_on(void) { return(fp); }
void debug_sleep(int mins) { if (fp) sleep(mins * 60); }
