/*********************************************************************
* dups (Check for duplicate transmits)
* Program reads an input file created by SQL query against the 
* studylog file, sorted by accession number, finds consecutive records
* which indicate multiple transmits of the same study (i.e. same accn,
* pid, pname, and study id).  Generates stdout file which lists
* studies which have been transmitted more than once, and summary
* statistics indicating the total amount of duplicate transmission.
*---------------------------------------------------------------------
* Input record format, tab delimited: accn, pid, pname, study id,
* source, num images, num mb, other fields...
*---------------------------------------------------------------------
* Calling format: dups [input file=dups_input] 
*---------------------------------------------------------------------
* Compile and link with: gcc dups.c -o dups
*********************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>

#define ITEMS 7
#define MATCH 7
#define MB 6
#define NAME 2
#define SIZE 256

int main(int numpars, char *par[])
{  FILE *in, *out;
   char in_pfn[24]  = "dups_input";
   char curr[ITEMS][SIZE], this[ITEMS][SIZE];
   unsigned long int study_mb = 0, xmit_mb = 0, study_count = 0, xmit_count = 0;
   int i, j;
   
/*---------------------------------- load parameter value overides */
   if (numpars > 1) strcpy(in_pfn,par[1]);

   in = fopen(in_pfn, "r");
   if (!in) { printf("Input file open error\n"); exit(1); }

   for (i = 0; i < ITEMS; i++) memset(curr[i],0,SIZE);
/*---------------------------------------- Process input file loop */
   while (1)
   {  char buff[1024], *p;
      int same_study, mb;
/*------------------------------------------- Read input file line */
      memset(buff,0,sizeof(buff));
      fgets(buff, sizeof(buff), in);
      if (feof(in)) { fclose(in); break; }
/*------------------------------------------- Load items from line */
      for (i = 0; i < ITEMS; i++)
      {	 memset(this[i],0,SIZE);
	 if (i) p = strtok(NULL,"\t\n\r");
	 else p = strtok(buff,"\t\n\r");
	 if (p) strcpy(this[i],p);
	 else { printf("Input file format error\n"); exit(1); }
      }
      mb = atoi(this[MB]) / 1024.0 + .5;
/*-------------------------------------- Match items (same study?) */
      same_study = 1;
      for (i = 0; i < MATCH; i++)
      	 if (strcmp(curr[i],this[i])) { same_study = 0; break; }
/*---------------------------- Dup study, bump xmits and continue */
      if (same_study)
      {	 printf("Dup %s, %dMB\n", curr[NAME], mb); 
         xmit_count++;
	 xmit_mb += mb;
	 continue;
      }
/*--------------- New study, load defaults, bump counts & totals */
      for (i = 0; i < MATCH; i++) strcpy(curr[i],this[i]);
      xmit_count++; study_count++;
      xmit_mb += mb; study_mb += mb;
   }
   printf("\n%d distinct studies, %d MB\n", study_count, study_mb);
   printf("%d study transmits, %d MB\n", xmit_count, xmit_mb);
   printf("%d dup transmits, %d MB\n", xmit_count - study_count, xmit_mb - study_mb);
   exit(0);
}
   
