#ifndef _MAP_H_
#define _MAP_H_
/****************************************************************************************
 * mircs_mapping.h                                                              R Moulton
 * Header file for MIR Computer section DICOM header mapping functions.
 ***************************************************************************************/

/*************************** structure definitions *************************************/

/*---------------------------------------------------------------------------------------
 * Map Tag structure.  One per Tag entry in the mapping file.
 *-------------------------------------------------------------------------------------*/
struct _map_tag_ {
   struct _map_tag_ * pmtag;
   char * name;                            /* Name of tag, from Tag line */
   unsigned int group;                     /* DICOM tag group & element from tag line */
   unsigned int element;
   DCM_ELEMENT e;
   char * study_value;                     /* Value for first image in this study */
   char * image_value;                     /* Value for this image of study */
   

};
typedef struct _map_tag_ * PMTAG;
#define SMTAG sizeof(struct _map_tag_)

/*---------------------------------------------------------------------------------------
 * Map If structure.  One per IF line in the mapping.
 *-------------------------------------------------------------------------------------*/
struct _map_if_ {
   struct _map_if_ * pmi;                  /* ptr to next IF line, if any */
   PMTAG left_pmtag;                       /* left side tag pointer */
   PMTAG right_pmtag;                      /* right side tag pointer, if any */
   char * right_side;                      /* right side string, if any */
   char lt;                                /* is left < right "TRUE" */
   char eq;                                /* is left = right "TRUE" */
   char gt;                                /* is left > right "TRUE" */
   char logical;                           /* 1 = &&, 0 = || (pmi is NULL on last line) */ 
};
typedef struct _map_if_ * PMI;
#define SMI sizeof(struct _map_if_)

/*---------------------------------------------------------------------------------------
 * Map Then structure.  One per THEN line in the mapping.
 *-------------------------------------------------------------------------------------*/
struct _map_then_ {
   struct _map_then_ * pmt;                /* ptr to next IF line, if any */
   PMTAG left_pmtag;                       /* left side tag pointer */
   PMTAG right_pmtag;                      /* right side tag pointer, if any */
   char * right_side;                      /* right side string, if any */

};
typedef struct _map_then_ * PMT;
#define SMT sizeof(struct _map_then_)

/*---------------------------------------------------------------------------------------
 * Map Log tag structure.  One per LogTags entry in the mapping header.
 *-------------------------------------------------------------------------------------*/
struct _map_logtag_ {
   struct _map_logtag_ * pmlt;
   PMTAG pmtag;
};
typedef struct _map_logtag_ * PMLT;
#define SMLT sizeof(struct _map_logtag_)

/*---------------------------------------------------------------------------------------
 * Map structure.  One per active mapping in the file
 *-------------------------------------------------------------------------------------*/
struct _map_map_ {
   struct _map_map_ * pmm;   
   char * name;                            /* name of mapping, from [name] line */
   char level;                             /* evaluate at 0=study level, 1=image level */
   char * log_name;                        /* log name for map, or NULL */
   char log_level;                         /* logging level */
   char * log_msg;                         /* log message */
   PMI   pmi;                              /* If lines */
   PMT   pmt;
   
};
typedef struct _map_map_ * PMM;
#define SMM sizeof(struct _map_map_)
   
/*---------------------------------------------------------------------------------------
 * Overall map struct; pointer to this is passed from routines. All the sub structures 
 * are of the pointer link variety 
 * ------------------------------------------------------------------------------------*/
struct _map_whole_ {
   FILE * debug_fp;
   char * inipfn;
   char * log_name;
   PMTAG  pmtag;                            /* Tags list */ 
   PMLT   pmlt;                             /* Log tags */
   PMM    pmm;                              /* Maps list */    
};
typedef struct _map_whole_ * PMAP;
#define SMAP sizeof(struct _map_whole_)

/*------------------------------------------------------------------------ error codes */
unsigned int map_errno;

#define MAP_R_OK   0
#define MAP_R_FAIL 1

#define MAP_OK        0      /* This is OK */
#define MAP_NOINI     1
#define MAP_ININOF    2
#define MAP_INIREAD   3
#define MAP_NOINISEC  4
#define MAP_SECNOF    5
#define MAP_INIERR    6
#define MAP_NOTON     7      /* This is OK */
#define MAP_MALLOC    8
#define MAP_MAPNOF    9
#define MAP_MAPREAD  10
#define MAP_MAPOPEN  11
#define MAP_INT      12
#define MAP_NOTAGS   13
#define MAP_INVTAG   14
#define MAP_TAGNAME  15
#define MAP_TAGDUP   16
#define MAP_TAGGRP   17
#define MAP_TAGELM   18
#define MAP_HDRINV   19
#define MAP_HDRLTAG  20
#define MAP_IFINV    21
#define MAP_IFTAG    22
#define MAP_IFREL    23
#define MAP_NOTHEN   24
#define MAP_THENINV  25
#define MAP_THENTAG  26
#define MAP_THENEQ   27
#define MAP_INVMAP   28
#define MAP_FE       29
/*************************** function prototypes ***************************************/

int mapSetup(char * section, char * inipfn);
int mapFirstObject(PMAP pmap, DCM_OBJECT * obj);
int mapObject(PMAP pmap, DCM_OBJECT * obj);
int mapFree(PMAP pmap);
char * GetMapErrorString(void);
void PrtMapErrorString(FILE * file, char * prefix);

#endif
