#ifndef _FIFO_H_
#define _FIFO_H_

/**************************************************************************************************
 * fifo.h                                                                                 R Moulton
 * Header file for disc buffered FIFO's.
 *-------------------------------------------------------------------------------------------------
 * 2-9-4 R R Moulton  Initial write
 *************************************************************************************************/

/*------------------------------------------------------------------------ Struct to handle fifo */
#define FIFO_MAX_CLIENTS 20
struct _fifo_client_ {              /* Clients for server get */
   char * name;                     /* client host name from .ini */
   unsigned char ip[4];             /* .ip */
   char rtr_msg[6];
   int sock;
   char rts;
   char * buffer;
};

struct _fifo_ {
   FILE * fp;          /* file pointers to fifos (.put for put, .get for get */
   FILE * fp_rtr;      /* .rtr fifo, used with fifo get */
   char rtr_msg[6];    /* Control message buffer for get */
   char * buffer;
   char * name;        /* base name of fifo (without extension) */
   int sock;           /* socket for TCP/IP connection (connect for put, server for get */
   char * host;        /* TCP/IP host information for put */
   char * ip;
   int port;           /* port on Server */
   char direction;     /* IO direction (FIFO_INPUT or FIFO_OUTPUT) */
   char blocking;      /* IO is supposed to block until complete */
   int bufsiz;         /* Size to make buffers */
   char rts;
   char sbeb;          /* Should SB and EB bytes be on output mesgs? */
   struct sockaddr_in * ptr; /* If server, ptr to structure */
   struct _fifo_client_ clients[FIFO_MAX_CLIENTS];
   char max_client;    /* actual number of clients loaded */
};

typedef struct _fifo_ FIFO;

#define FIFO_INPUT    0
#define FIFO_OUTPUT   1
#define FIFO_BLOCK    1
#define FIFO_IOLBF    2

#define FIFO_R_OK     0
#define FIFO_R_FAIL   1

unsigned int fifo_errno;

#define FIFO_OK                0   /* returns OK */
#define FIFO_INVFIFO           1
#define FIFO_NOSUCHFIFO        2
#define FIFO_INVDIRECTION      3
#define FIFO_MEM               4
#define FIFO_PERMISSIONS       5
#define FIFO_FORK              6
#define FIFO_BFAIL             7
#define FIFO_OPEN              8
#define FIFO_INVMSG            9
#define FIFO_EOF              10   /* returns OK */
#define FIFO_ERROR            11
#define FIFO_CONNECT          12
#define FIFO_SELECT           13
#define FIFO_RETRIES          14
#define FIFO_LAUNCH           15
#define FIFO_SERVER           16
#define FIFO_INI              17
#define FIFO_CLIENTS          18
#define FIFO_SERVER_INIT      19

/*-------------------------------------------------------------------------------------------------
 * Open a buffered fINifo. name is the base name of the fifo (not including any extension). 
 * direction is directio of IO (FIFO_INPUT if you are Geting information out, FIFO_OUTPUT if you
 * are Putting information in). Flags are or'd, FIFO_BLOCKING if you want IO to block until 
 * complete, and IOLBF if you want the IO to be line buffered (sent immediately).  Returns a 
 * pointer to a _fifo_ struct, which has been allocated and loaded if the open is successful, NULL 
 * otherwise.  This routine will load the buffer process if needed.
 *-----------------------------------------------------------------------------------------------*/
FIFO * fifoOpen(char * name, char direction, unsigned char flags);
/*-------------------------------------------------------------------------------------------------
 * Closes the open fifo and deallocates the memory used by the struct (after this call the pointer
 * will no longer be valid).
 *-----------------------------------------------------------------------------------------------*/
int fifoClose(FIFO * fifo);
/*-------------------------------------------------------------------------------------------------
 * Put msg to the open fifo.  msg must be delimited as indicated in the .ini file for the fifo.
 *-----------------------------------------------------------------------------------------------*/
int fifoPut(FIFO * fifo, char * msg);
/*-------------------------------------------------------------------------------------------------
 * Get next msg from open fifo. msg must be previously defined area of at least size bytes. msg
 * will be returned with delimiters (<SB> & <EB>, but not CR) in place.  Remainder of msg will be
 * NULL.  If non-blocking, msg will return empty for eof.  Returns msg empty and FIFO_R_FAIL for
 * errors.
 *-----------------------------------------------------------------------------------------------*/
int fifoGet(FIFO * fifo, char * msg, size_t size); 

char * GetFifoErrorString(void);
void PrtFifoErrorString(FILE *, char * prefix);

#endif
