#ifndef _SQL_H_
#define _SQL_H_

/**************************************************************************************************
 * sql.h                                                                                  R Moulton
 * Header file for mysql database routines
 *-------------------------------------------------------------------------------------------------
 * 13-3 R Moulton Initial write
 *************************************************************************************************/
#include <mysql.h>

/*----------------------- This is created one more than # of fields, with last entry being NULLs */
struct _sql_table_ {
   MYSQL_FIELD * field;
   char * value;
};
/*-------------------------------------------------------------------------------------------------
 * Error numbers below 1000 are assigned by sql.c.  1### numbers are mysql client errors 
 * (see errmsg.h), and 2### numbers are server errors (see mysqld_error.h).
 *-----------------------------------------------------------------------------------------------*/
unsigned int sql_errno;

#define SQL_OK            0
#define SQL_NODBNAME      1
#define SQL_NOUSER        2
#define SQL_NOINIPFN      3
#define SQL_INI           4
#define SQL_INITFAIL      5
#define SQL_CONN          6
#define SQL_NOTABLE       7
#define SQL_NOFIELDS      8
#define SQL_MALLOC        9
#define SQL_FIELD        10
#define SQL_OPFAILED     11
#define SQL_ININOF       12
#define SQL_SECNOF       13

/*------------------------------------------------------------------------------- Flag constants */
#define SQLF_NO_FLAGS      0x0000
#define SQLF_LOW_PRIORITY  0x0001
#define SQLF_DELAYED       0x0002
#define SQLF_IGNORE        0x0004

int sqlSetIniPfn(char * pfn);
MYSQL * sqlOpen(char * db, char * user);
int sqlClose(MYSQL * conn);
char * GetSqlErrorString(MYSQL * conn);
void PrtSqlErrorString(MYSQL * conn, FILE * file, char * prefix);
struct _sql_table_ * sqlLoadRecordDef(MYSQL * conn, char * table_name);
int sqlFreeRecord(struct _sql_table_ * tbl);
int sqlClearRecord(struct _sql_table_ * tbl);
int sqlSetValue(struct _sql_table_ * tbl, char * name, char * value);
int sqlInsertRecord(MYSQL * conn, struct _sql_table_ * tbl, unsigned int flags, unsigned long * dba);
int sqlQuery(MYSQL * conn, char * query);

#endif
	
