/***********************************************************************
 * tb.c "test broker" client.  accepts messages from server, echoes
 * them, and logs everything to stdout.
 ************************************************************************
 *
 * Compile and link with: gcc tb.c -o tb -lsocket -lnsl
 *
 ***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>

char target_host[24]       = "relay1a";
char target_ip[]           = "10.41.2.32";
char target_port[24]       = "10000";
int wait_seconds = 0;
#define BEGIN_MSG 11
#define END_MSG   28

/***********************************************************************
 * Other global working storage
 **********************************************************************/
char a[512];
int srv_socket, i, j, retval;
struct hostent *psrvent, srvent;
struct sockaddr_in srv_sock_name;
fd_set socket_set;
struct timeval timeout;

void dbg(char*msg){printf("%s\n",msg);}

void dbgm(char*msg, char*buff)
{  char p[103], *ptr;
   int piece = sizeof(p) - 3, cont = 1;
   if (strlen(msg) > (sizeof(p) -3)) dbg(msg);
   else
   {   memset(p,0,sizeof(p)); strcpy(p,msg);
       if (p[strlen(p)-1] != ' ')p[strlen(p)] = ' ';
       memset(p+strlen(p),'-',(sizeof(p)-strlen(p)-1));
       dbg(p);
   }
   if (!buff) { dbg("NULL buffer\n"); return; }
   while (cont)
   {  memset(p,0,sizeof(p)); strcpy(p,"> ");
      if (strlen(buff) < piece) { strcat(p,buff); cont = 0; }
      else { memcpy(p+2,buff,piece); buff += piece; }
      if (ptr = strchr(p,BEGIN_MSG)) *ptr = '.';
      if (ptr = strchr(p,END_MSG)) *ptr = '#';
      while (ptr = strchr(p,'\r')) *ptr = '~';
      dbg(p);
   }
   dbg(" ");
}
   
/***********************************************************************
 * close out socket
 **********************************************************************/
cleanup()
{  retval = shutdown(srv_socket, 2);
   if(retval != 0) perror("shutdown");
   retval = close(srv_socket);
   if(retval) perror("close");
}

/***********************************************************************
 * main function
 **********************************************************************/
int main(int numpars, char *par[])
{  char buffer[20000];
   int message_number = 0;
   
   printf("HL7 interface Test Client\n  %s port %s\n\n",
      target_host, target_port);
   
   printf("  Creating socket\n");
   srv_socket = socket(AF_INET, SOCK_STREAM, 0);
   if(srv_socket == -1) {perror("socket"); exit(1);}

   printf("  Get host by name\n");
   psrvent = gethostbyname(target_host);
   if(psrvent == NULL) 
   {  printf("   Get host by IP\n");
      psrvent = gethostbyname(target_ip);
   }
   if(psrvent == NULL) 
   {  perror("gethost");
      cleanup();
      exit(1);
   }
   printf("  Setting socket name entries\n");
   srvent = *psrvent;
   srv_sock_name.sin_family = srvent.h_addrtype;
   srv_sock_name.sin_port = htons(atoi(target_port));
   srv_sock_name.sin_addr = *((struct in_addr *)srvent.h_addr);
   
   printf("  Connect to server\n");
   i = sizeof(srv_sock_name);
   retval = connect(srv_socket, (struct sockaddr *)&srv_sock_name, i);
   printf("  Connect return = %d\n", retval);
   if (retval < 0) { perror("Connect"); exit(1); }

   while (1)
   {  char buff[10000], *bp; 

      memset(buffer,0,sizeof(buffer));
      while (!strchr(buffer,END_MSG))
      {  FD_ZERO (&socket_set); 
         FD_SET (srv_socket, &socket_set);
         j = srv_socket + 1;
         printf("  Select server for read\n");
         retval = select(j, &socket_set, NULL, NULL, NULL);
         printf("  Select return = %d\n", retval);
         if (retval < 1) {  perror("read select"); cleanup(); exit(1); }

         printf("  Receive message from server\n");
         retval = recv(srv_socket, buffer + strlen(buffer), sizeof(buffer) - strlen(buffer), 0);
         printf("  Receive return = %d\n", retval);
         if (retval == 0) { printf("Socket closed by server"); cleanup(); exit(1); }
         if (retval < 0) { perror("recv"); cleanup(); exit(1); }
      }
      dbgm("  Message received:", buffer);

      FD_ZERO (&socket_set);
      FD_SET (srv_socket, &socket_set);
      j = srv_socket + 1;
      printf("  Select server for write\n");
      retval = select(j, NULL, &socket_set, NULL, NULL);
      printf("  Select return = %d\n", retval);
      if (retval < 1) {perror("write select"); cleanup(); exit(1);}

      printf("  Send message to server\n");
      i = strlen(buffer);
      retval = send(srv_socket, buffer, i, 0);
      printf("  Send return = %d\n", retval);
      if(retval == -1) {  perror("send"); cleanup(); exit(1); }
      
      memset(buffer,0,sizeof(buffer));
      sleep(wait_seconds);
   }

   printf("  Cleanup and end\n");
   cleanup();
}
